********************************************************************************
*                                                                              *
*                      PROGRAM MAGRAV - REVISED                                *
*                                                                              *
*  PURPOSE         : Interactive Gravity and Magnetic Modelling                *
*                                                                              *
*  ORIGINAL AUTHOR : I. Wells                                                  *
*                                                                              *
*  WRITTEN FOR     : DR. Haworth,                                              *
*                    Atlantic Geoscience Centre                                *
*                    Bedford Institute of Oceanography                         *
*                    Dartmouth, Nova Scotia, Canada                            *
*                                                                              *
*  REFERENCE       : Wells, I. (1979) MAGRAV - A computer program to create    *
*                    two-dimensional gravity and/or magnetic models,           *
*                    Computer Science Centre, Open File 597.                   *
*                                                                              *
*  REVISION BY     : P.H. McGrath,                                             *
*                    Continental Geoscience Division,                          *
*                    Geological Survey of Canada,                              *
*                    1 Observatory Crescent,                                   *
*                    Ottawa, Ontario, Canada, K1A 0Y3                          *
*                    Tel. (613)995-4075, FAX (613)947-1819                     *
*                                                                              *
*  VERSION 3.0     : September 1993                                            *
*                                                                              *
********************************************************************************
C
C  A history of major changes to the Wells program follows:
C
C	1) Modified by Peter McGrath, 1982, to permit automatic adjustment of
C		body magnetization (density) contrasts and/or body shape using
C		non-linear least squares (Powell, 1965, The Computer Journal,
C		vol. 7, p. 303).      
C
C	2) Modified by D. Clarke and J. Rupert, 1985 and 1986, for VAX/VMS.
C
C	3) MAGRAV2, a rewritten version in Microsoft FORTRAN 77 for use on an
C		IBM-PC was released as Gelogical Survey of Canada, Open File
C		1334, by John Broome, 1986.
C 
C	4) Modified by Peter McGrath, 1988, to permit
C		- gravity modelling constrained by isostasy,
C		- automatic simultaneous changes to body shape(s), and/or
C		  body contrast(s),and/or the offset value of the calculated
C		  anomaly profile with respect to the observed data,
C		- the ability to screen out aberrant observed data points from
C		  the automatic curve-fitting subroutine.
C
C	5) Modified by Peter McGrath, 1993, to include
C		- interactive modification of the parameters, ESC and FRAC, used
C		  by the non-linear least squares subroutine DSF001 from their
C		  default values of 100. and 0.001 respectively,
C		- gravity and magnetics modified for asymmetric 2 1/2 D bodies,
C		  (Rasmussen and Pedersen, 1979, Geophysical Prospecting, 27,
C		  p. 749-760),
C		- an option to produce a hard copy of a model on a VERSATEC
C		  plotter.
C
C  The program is written in SUN FORTRAN 77, and runs on a SPARC 2 Workstation.
C      During execution two windows are opened, a TEXT window for display of
C      text options and help files, and a GRAPHICS window for display of the
C      gravity (magnetic) profile and the geological cross section. Graphic
C      presentations are plotted using the FGL/GRAPHICS system, Version 6.3B,
C      obtained from UNIRAS of Soborg, Denmark. The User must compile this
C      source code, and link it with the GFL GRAPHICS system. An executable
C      version of MAGRAV is not supplied as part of the Open File.
C
C  The program can also be run from a mouse-equipped, IBM personal or compatible
C      computer which acts as an X Terminal, and uses the Windows version of 
C      HCL-EXCEED and PCNFS as its network transport to connect in multiple 
C      window mode over a UNIX network to a SUN Workstation.
C
C  I/O
C
C  INPUT  : CURSOR INPUT FROM GRAPHICS OR TEXT TERMINAL
C  OUTPUT : GRAPHICS TERMINAL - SETUP FOR A 17 INCH SCREEN
C             OR ANY TEXT TERMINAL
C  UNIT 1 : DIRECT ACCESS DISK STORAGE FOR MODELS
C  UNIT 2 : USED DURING DELETION OF A MODEL FROM UNIT 1.
C  UNIT 7 : DISK FILE OUTPUT
C  UNIT 8 : TO INPUT FILE OF OBSERVED DATA
C  UNIT 9 : TEMPORARY DISK FILE OF MODELS FOR ERROR RECOVERY
C
C  VARIABLES IN COMMON/MOD/
C
C  NAME   : A10 MODEL NAME - SET USING THE NAME OPTION
C  KOMMNT : A80 COMMENTS ABOUT MODEL
C  ISCOPE : 1 ALL GRAPHICS IN EFFECT
C           2 ALL GRAPHICS SUPPRESSED, FOR TEXT-ONLY TERMINALS
C  MODE   : 1 DRAW MODE
C           2 SKETCH MODE
C  ITYPE  : 1 FOR GRAVITY MODELS
C           2 FOR MAGNETIC MODELS
C  XMAX   : RIGHT HAND SIDE OF GRAPH IN DRAW MODE
C           NOTE: ALL BODIES AND FIELD POINTS SHOULD BE LESS THAN THIS
C  ZMAX   : MAXIMUM DISTANCE FROM SURFACE (0.) TO BOTTOM OF SCREEN
C  XMINS(ITYPE)  : X DISPLACEMENT.
C                  X VALUE OF FIRST FIELD POINT
C  XMAXS(ITYPE)  : COORDINATE OF LAST OBSERVED VALUE
C  NFIELD(ITYPE) : NUMBER OF FIELD POINTS
C  SPACE(ITYPE)  : SPACING BETWEEN FIELD POINTS IN KM
C  OBS(IARAY,MAXOBS) : IARAY = 1, OBSERVED GRAVITY VALUES,
C                            = 2, OBSERVED MAGNETIC VALUES,
C                            = 3, GRAVITY STATION ELEVATIONS IN KM,
C                            = 4, EQUALS 1(0) IF GRAVITY DATA AT POINT
C                                 IS TO BE INCLUDED(DELETED) DURING
C                                 AUTOMATIC CURVE-FITTING,
C                            = 5, EQUALS 1(0) IF MAGNETIC DATA AT POINT
C                                 IS TO BE INCLUDED(DELETED) DURING
C                                 AUTOMATIC CURVE-FITTING.
C  LSTART(ITYPE) : FIRST DATA POINT IN ARRAY OBS
C  LEND(ITYPE)   : LAST DATA POINT IN ARRAY OBS
C  ANOMIN(ITYPE) : MINIMUM VALUE ON ANOMALY SCALE
C  ANOMAX(ITYPE) : MAXIMUM VALUE ON ANOMALY SCALE
C  IDIFF         : PLOTS DIFFERENCES BETWEEN CALCULATED AND OBSERVED DATA
C  DIFMIN(ITYPE) : LOWER BOUNDARY OF GRAPH IN DIFFERENCE MODE
C  DIFMAX(ITYPE) : UPPER BOUNDARY OF GRAPH IN DIFFERENCE MODE
C  SKXMAX        : CURRENT RIGHT HAND SCREEN LIMIT BEING USED FOR GRAPHICS
C  SKXMIN        : CURRENT LEFT HAND SCREEN LIMIT
C  SKZMAX        : CURRENT LOWER SCREEN LIMIT
C  SKZMIN        : CURRENT UPPER SCREEN LIMIT
C  ISUPG  : 1 GRID LINES PLOTTED
C          -1 GRID LINES SUPPRESSED
C  GX, GZ : SPACING OF GRID LINES IN KM.
C  NBODS  : NUMBER OF BODIES IN CURRENT MODEL
C  NPTS(MAXBOD) : NUMBER OF POINTS CURRENTLY IN EACH BODY
C  XTON : X AXIS TO GEOGRAPHIC NORTH, POSITIVE ANGLE IS
C         MEASURED CLOCKWISE FROM GEOGRAPHIC NORTH TO THE +X AXIS.
C  FDEC  : DECLINATION OF FIELD. (MAG ONLY)
C         0 - 360 DEGREES, CLOCKWISE FROM GEOGRAPHIC NORTH
C  FINCL : INCLINATION OF THE GEOMAGNETIC FIELD. (MAG ONLY)
C         POSITIVE IN NORTHERN HEMISPHERE
C  BINCL(MAXBOD)  : INCLINATION OF POLARIZATION VECTOR FOR EACH BODY. (MAG
C  BDEC(MAXBOD)   : DECLINATION OF POLARIZATION VECTOR FOR EACH BODY. ONLY)
C                  0 - 360 DEGREES, CLOCKWISE FROM GEOGRAPHIC NORTH
C  BDY(MAXBOD,2) : HALF-STRIKE LENGTHS OF EACH BODY
C       SEE RASMUSSEN AND PEDERSEN (1979), FIGURE 1
C                   BDY(IBOD,1) = YMIN
C                   BDY(IBOD,2) = YMAX
C  X(MAXNPT+1,MAXBOD) : X COORDINATES OF BODY POINTS
C  Z(MAXNPT+1,MAXBOD) : Z COORDINATES OF BODY POINTS
C                        NOTE : (0,0) IS NOT AN ACCEPTIBLE BODY POINT
C  RHOMAG(MAXBOD,1)   : DENSITY OF EACH BODY. (GRAVITY ONLY)
C  RHOMAG(MAXBOD,2    : MAGNETIZATION OF EACH BODY EMU*100000
C  RMMIN(MAXBOD,ITYPE) : BODY CONTRAST LIMIT - SMALLEST OR MOST NEGATIVE
C  RMMAX(MAXBOD,ITYPE) : BODY CONTRAST LIMIT - LARGEST OR MOST POSITIVE
C  ZCON(ITYPE) : A CONSTANT ADDED TO Z COORD OF ALL BODIES
C  ICROSS      : 0 ON GRAPHICS OUTPUT BODIES ARE DRAWN AS SET OF LINES
C                1 DRAWS CROSSES AT EACH VERTEX
C                2 FILLS BODIES WITH COLOR 
C                3 FILLS BODIES AND DRAWS CROSSES AT EACH VERTEX
C  OFFSET(ITYPES) : BASE LEVEL SHIFT ADDED TO CALCULATED DATA FOR PLOTTING ONLY
C  IANOM(MAXBOD)  : 0 ANOMALY HAS NOT BEEN CALCULATED FOR THIS BODY
C                  -1 ANOMALY HAS BEEN CALCULATED FOR THIS BODY
C                    AND IS STORED IN CALC(MAXCAL,MAXOBS)
C  IWFLAG : OPTIMIZATION CONTROL
C            = 1, FITS CALCULATED TO OBSERVED PROFILE
C            = 2, USES DIFFERENTIAL COLUMN PRESSURES IF WW = 0
C  WW        : FACTOR WHICH COMBINES CONSTRAINTS, IWFLAG =  1 AND 2
C  PMAX,PMIN : DIFFERENTIAL COLUMN PRESSURE LIMITS FOR GRAPH IN BARS
C  YANG(MAXBOD) : ANGLE MEASURED COUNTERCLOCKWISE FROM THE POSITIVE Y AXIS TO
C                 THE BODY STRIKE DIRECTION
C  BKDEN : BACKGROUND DENSITY VALUE WHICH IS ADDED TO THE DENSITY CONTRAST OF
C            EACH BODY. THE DEFAULT VALUE IS ZERO.
C-------------------------------------------------------------------------------
C  NOTE   : ALL DATA IN COMMON/MOD/ ARE WRITTEN TO DISK WHEN SAVING MODELS             C-------------------------------------------------------------------------------
C
C  VARIABLES IN COMMON/PROG/
C
C  SDEN(MAXOBS) : SURFACE DENSITIES DETERMINED AND USED BY SUBROUTINE ISOS
C  WD(MAXOBS)   : DIFFERENCES IN WEIGHT OF 1*1M COLUMNS AND THE AVERAGE WEIGHT
C                 OF ALL COLUMNS EXPRESSED AS DIFFERENTIAL PRESSURES(BARS), 
C                 WHERE 1 GIGAGRAM = 98 BARS
C  WFACT  : RATIO OF RMS DIFFERENCES BETWEEN THE OBSERVED AND CALCULATED GRAVITY
C           VALUES AND THE RMS OF THE DIFFERENTIAL COLUMN PRESSURES
C  NP,IB,XX,E,W,F : ARRAYS USED BY SUBROUTINE DFS001
C  NBACK  : MODEL IS BACKED UP AFTER EVERY NBACK CHANGES
C  MODEL  : UNIT NUMBER OF MODELS FILE
C  NTYPES : NUMBER OF TYPES OF MODELS THIS PROGRAM WILL HANDLE. (MAG,GRAV)
C  XDIS   : DISTANCE IN SCALED UNITS IN X DIRECTION FOR RESOLUTION
C           ON PINPOINTING FOR CURSOR INPUT OF BODIES
C  ZDIS   : SAME IN Z DIRECTION
C  IOFS   : THIS IS A SWITCH, SEE GRAPHICS HELP FOR O
C              IOFS = 0, THE BASE LEVEL VALUE OFFSET(ITYPE) IS NOT ALTERED
C                        BY THE AUTOMATIC CURVE FITTING ROUTINES (DEFAULT)
C              IOFS = 1, THE BASE LEVEL VALUE OFFSET(ITYPE) CAN BE ALTERED BY
C                        THE AUTOMATIC CURVE FITTING ROUTINES,
C  MAXNPT : MAXIMUM NUMBER OF POINTS PER BODY
C  MAXBOD : MAXIMUM NUMBER OF BODIES
C  MAXOBS : MAXIMUM NUMBER OF OBSERVATIONS
C  MAXCAL : MAXIMUM NUMBER OF BODIES PLUS ONE
C  ESC    : PARAMETER USED BY SUBROUTINE DFS001 WHICH CONTROLS THE AMOUNT OF
C            MOVEMENT DURING ONE ITERATION.
C  FRAC   : PARAMETER USED BY SUBROUTINE DFS001  
C  ISCR   : UNIT NUMBER OF SCRATCH FILE.
C           AFTER EVERY NBACK CHANGES TO THE MODEL,
C           IT IS BACKED UP ON THIS UNIT.
C           AFTER 300 BACKUPS THE FILE IS REWOUND.
C  CALC(MAXBOD,MAXOBS)      : ANOMALY CALCULATED FOR EACH BODY
C                              AT EACH OBSERVED POINT
C  CALC(MAXBOD + 1, MAXOBS) : STORES TOTAL ANOMALY

      PROGRAM MAGRAV
      CHARACTER*10 IBLANK, LIST(20), INAME
      CHARACTER IANS*4, MODFILE*50, SCRATCH*50, OBSFILE*50
      CHARACTER MODF*3,MDL*4,SCR*4,FDUMP*30,BELL*1,JBLANK*75

      INCLUDE 'magrav.cmn'

      REAL*8 CALC, F, W

      COMMON /CALDAT/ CALC(21,200)
      COMMON /DFS/ F(200)
      COMMON /WORK/ W(4900)
      COMMON /LIST/ ILIST, LIST
      COMMON /VPST/ WIDMM,HTMM,NXPIX,NYPIX,ITERM,CHEI,YOR
      COMMON /SAVEIT/ NSAVE

      DATA IBLANK/'          '/
      DATA JBLANK/'                                                     
     +                           '/
      DATA LIST/20*'          '/
      DATA MDL,SCR/'.mdl','.scr'/
C  XSM and YSM are the size of the GRAPHICS window in mm.
C  NCOL and NROW are the number of columns and rows in the GRAPHICS window
      DATA XSM,YSM,NCOL,NROW/278.086,150.117,1017,549/
      DATA PI/3.14159265/

      ISCR = 9
      BELL=CHAR(7)
      MODEL = 1
      MODDEL = 2
      JZ = 6
      IVA = -1
      ISET = 0
      MAXNPT = 24
      MAXBOD = 20
      MAXOBS = 200
      MAXCAL = MAXBOD + 1
      NTYPES = 2
      XDIS = 0.
      ZDIS = 0.
      IOFS = 0
      ITERM = 1
      IWRITE = 0
      CHEI=3.
      ISCOPE = 2
      WRITE(JZ,2)
1     WRITE(JZ,3)
2     FORMAT(15(/),25x,' Welcome to MAGRAV',/29X,'UNIX Version',/,
     +23X,'Last Edited: September, 1993',5(/))
3     FORMAT(/,' SET TERMINAL FUNCTION',/,
     *'        1 - SUN TERMINAL',/,
     *'        2 - VERSATEC PLOT')
      READ(*,*) ITERM
      IF(ITERM.LE.0.OR.ITERM.GE.3) GOTO 1
      IF(ITERM.EQ.1) THEN
      CALL GROUTE('SEL MX11;AREA 278.2,150.2;EX')
      YOR=0.
      ELSE 
      WRITE(JZ,4)
4     FORMAT(' SELECT SCALE:',$)
      READ(*,*) SCAL
      WRITE(JZ,5)
5     FORMAT(' WHAT IS THE LENGTH OF THE X AXIS IN KM.:',$)
      READ(*,*) XMAX
      WIDMM = XMAX*1.D6/SCAL
      HTMM = WIDMM*YSM/XSM
      NXPIX = NCOL
      NYPIX = NROW
      YOR=80.
      CALL GROUTE('SEL GVCE3436;EX')
      ENDIF
      CALL GHALF8
      CALL GOPEN
      CALL RTXFON('SIMP',0)
      CALL RTXHEI(CHEI)
      CALL RTXJUS(0,1)
      CALL RTXSPM(1)
      CALL GWICOL(.05,1)
      CALL GCLEAR
      CALL GCLIP
      IF(ITERM.EQ.1) THEN
      CALL GRPSIZ(WIDMM,HTMM)
      CALL GRPPIX(NXPIX,NYPIX)
      ENDIF
      ISCOPE = 1
      WRITE(JZ,*)WIDMM,HTMM,NXPIX,NYPIX
      GOTO 7
6     WRITE(JZ,*) ' '
      WRITE(JZ,*) ' *** ERROR *** file is not a model data file '
      WRITE(JZ,*) ' '
C OPEN MODEL FILE
7     WRITE(JZ,8) 
8     FORMAT(' Enter 3 character name of model data file: ',$)
      READ(*,9) MODF
9     FORMAT(A3)
      SCRATCH=MODF//SCR
      MODFILE=MODF//MDL
      OPEN(MODEL, FILE=MODFILE, ACCESS='DIRECT', STATUS='UNKNOWN',
     +     FORM='UNFORMATTED', RECL=9318, ERR=6)
      OPEN(ISCR, FILE=SCRATCH, ACCESS='SEQUENTIAL', STATUS='UNKNOWN', 
     +     FORM='UNFORMATTED', ERR=6)
      ILIST = 1
12    READ(MODEL, REC=ILIST, END=13) LIST(ILIST), KOMMNT, ISCOP, MODE,
     + ITYPE, XMAX, ZMAX, XMINS(1), XMINS(2), XMAXS(1), XMAXS(2),
     + NFIELD(1), NFIELD(2), SPACE(1), SPACE(2), ((OBS(J,I),J=1,5),I=1,
     + 200), (LSTART(I), LEND(I),I=1,2),
     + ANOMAX(1), ANOMAX(2), ANOMIN(1), ANOMIN(2),
     + IDIFF, DIFMIN(1), DIFMIN(2), DIFMAX(1), DIFMAX(2),
     + SKXMAX, SKXMIN, SKZMAX, SKXMIN, ISUPG, GX, GZ, NBODS, (NPTS(I),
     + I=1,20), XTON, FDEC, FINCL, (BINCL(I),I=1,20), (BDEC(I),I=1,20),
     + ((BDY(J,I),J=1,20),I=1,2), ((X(J,I), J=1,20), I=1,25),
     + ((Z(J,I), J=1,20),I=1,25), ((RHOMAG(J,I), J=1,20), I=1,2),
     + ((RMMIN(J,I), J=1,20),I=1,2), ((RMMAX(J,I), J=1,20), I=1,2),
     + ZCON(1), ZCON(2), ICROSS, OFFSET(1), OFFSET(2), (IANOM(I),I=1,20)
     +, IWFLAG, WW, PMAX, PMIN, (YANG(I),I=1,20), BKDEN 
      ILIST = ILIST + 1
      GOTO 12
13    ILIST=ILIST-1
      IF(LIST(1).NE.IBLANK) THEN
        WRITE(JZ, *) 'LIST OF MODELS IN MODEL FILE :', MODFILE
      ENDIF
      IF(ILIST.EQ.0) THEN
        WRITE(JZ,*) ' *** NEW FILE ***  NO MODELS'
        WRITE(JZ,*) BELL
        WRITE(JZ,*) BELL
      ELSE
        DO I = 1, ILIST
          WRITE(JZ, *) LIST(I)
        ENDDO
      ENDIF
      NSAVE=0
14    READ(ISCR,END=15)
      NSAVE=NSAVE+1
      GOTO 14
15    IF(NSAVE.NE.0) THEN
       WRITE(JZ,16)
16    FORMAT(//,'       ********************************************',/,
     1          '       *                                          *',/,
     2          '       * NOTE: The last time your models file was *',/,
     3          '       *       used the program crashed, you may  *',/,
     4          '       *       recover your model by using the    *',/,
     5          '       *       RECO command.                      *',/,
     6          '       *                                          *',/,
     7          '       ********************************************'//)
      ENDIF
      IDIFF = 1
      NBACK = 3
      ITYPE = 1
      PMAX = 300.
      PMIN = -300.
      IWFLAG = 1
      IWFP = IWFLAG
      WW = 0.
      NBODS = 0
      XMAX = 100.
      ZMAX = 10.
      SKXMAX = XMAX
      SKXMIN = 0.
      SKZMAX = ZMAX
      SKZMIN = 0.
      ICROSS = 3
      XTON = 0.
      FDEC = 0.
      FINCL = 90.
      ZCON(1) = .001
      ZCON(2) = .001
      GX = 10.
      GZ = 1.
      MODE = 1
      ISUPG = -1
      ESC = 100.
      FRAC = 0.001
      BKDEN = 0.
      DO J = 1, NTYPES
        OFFSET(J) = 0.
        LSTART(J) = 0
        LEND(J) = 0
        ANOMAX(J) = 100.
        ANOMIN(J) = -100.
        DIFMIN(J) = -50.
        DIFMAX(J) = 50.
        NFIELD(J) = 0
        XMAXS(J) = XMAX
        XMINS(J) = 0.
        SPACE(J) = 1.
      ENDDO
      DO I = 1, MAXNPT
        DO J = 1,MAXBOD
          X(J,I) = 0.
          Z(J,I) = 0.
        ENDDO
      ENDDO
      DO J = 1, 3
        DO I = 1, MAXOBS
          OBS(J,I) = -9999.
        ENDDO
      ENDDO
      DO I = 1, MAXOBS
        OBS(4,I) = 0.
        OBS(5,I) = 0.
      ENDDO
      CALL REINIT
      DO I = 1, MAXBOD
        NPTS(I) = 0
        RHOMAG(I,1) = 0.
        RHOMAG(I,2) = 0.
        RMMIN(I,1) = 0.
        RMMIN(I,2) = 0.
        RMMAX(I,1) = 0.
        RMMAX(I,2) = 0.
        BDEC(I) = 0.
        BINCL(I) = 0.
        BDY(I,1) = -10000.
        BDY(I,2) = 10000.
        YANG(I) = 0.
      ENDDO
      KOMMNT = JBLANK
      NAME = IBLANK
      WRITE(JZ,360)
360   FORMAT(//,'ENTER, GRAV - FOR A NEW GRAVITY MODEL',/,
     +'       MAGN - FOR A NEW MAGNETIC MODEL',/,
     +'       NAME - TO READ AN OLD MODEL FROM MODEL FILE, AND',/,
     +'                  AFTER NAMING MODEL CHOOSE THE READ OPTION')
      GOTO 400

C ========================================
C    OPTIONS MODE
C ========================================
C      EXECUTE COMMAND INPUT FROM TERMINAL

350   CALL SAV(NBACK)

400   CALL WHAT(IWHAT,1)
      IF(IWHAT.EQ.0) THEN
        WRITE(JZ, *) 'Pardon ?'
        GOTO 400
      ENDIF

C  NOTE***************************************************************
C  The following GOTO statement controls branching for the TEXT window
500   GOTO(1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000,9000,
     +    10000, 11000, 12000, 13000, 14000, 15000, 16000, 17000,
     +    18000, 19000, 20000, 21000, 22000, 23000, 24000, 25000,
     +    26000, 27000, 28000, 29000, 30000, 31000, 32000, 33000,
     +    34000, 35000, 36000, 37000, 20000, 20000, 38000, 39000, 
     +    40000, 41000, 42000, 400), IWHAT


*<<*>>* SET SCALE OF GRAPH
*+++++++++++++++++++++++++

1000  IF(ITYPE.EQ.1) WRITE(JZ, *) 'GRAVITY MODEL'
      IF(ITYPE.EQ.2) WRITE(JZ, *) 'MAGNETIC MODEL'
      WRITE(JZ,1001)XMINS(ITYPE),XMAX,ZMAX,SPACE(ITYPE),NFIELD(ITYPE),
     1 ANOMIN(ITYPE),ANOMAX(ITYPE),DIFMIN(ITYPE),DIFMAX(ITYPE),GX,GZ
1001  FORMAT(' A -- X Displacement (km)              : ',f10.3,/,
     1       ' A -- Total Profile Length, XMAX (km)  : ',f10.3,/,
     2       ' B -- Maximum depth, ZMAX  (km)        : ',f10.3,/,
     3       ' C -- Field point spacing    (km)      : ',f10.3,/,
     4       ' D -- Number of calculated points      : ',i10,/,
     5       ' E -- Anomaly minimum scale            : ',f10.3,/,
     6       ' E -- Anomaly maximum scale            : ',f10.3,/,
     7       ' F -- Anomaly difference minimum scale : ',f10.3,/,
     8       ' F -- Anomaly difference maximum scale : ',f10.3,/,
     9       ' G -- X axis grid spacing              : ',f10.3,/,
     A       ' H -- Z axis grid spacing              : ',f10.3,/,
     B       ' X -- EXIT TO COMMAND LINE',//,
     C       ' Enter A to K or X : ',$)
      READ(*,'(A1)',ERR=1000) ANS
      IF(ANS.EQ.'X'.OR.ANS.EQ.'x') THEN
       SKXMIN = XMINS(ITYPE)
       SKXMAX = XMAXS(ITYPE)
       SKZMIN = 0.
       SKZMAX = ZMAX
       CALL REINIT
       GOTO 350
      ELSE IF(ANS.EQ.'A'.OR.ANS.EQ.'a') THEN
1002   WRITE(JZ,1003) 
1003   FORMAT(' Enter minimum field point co-ordinate (Kilometers) :',$)
       READ(*,*,ERR=1002) XMINS(ITYPE)
1004   WRITE(JZ,1005) 
1005   FORMAT(' Enter maximum field point co-ordinate (Kilometers):',$)
       READ(*,*,ERR=1004) XMAX
       IF(XMINS(ITYPE).GE.XMAX) THEN
        WRITE(JZ,*) '*** ERROR *** minimun greater then maximum'
        WRITE(JZ,*) BELL
        GOTO 1002
        ENDIF
      ZMAX=XMAX*0.7*HTMM/WIDMM
      ELSE IF(ANS.EQ.'B'.OR.ANS.EQ.'b') THEN
1006   WRITE(JZ,1007) 
1007   FORMAT(' Enter maximum depth (Kilometers) :',$)
       READ(*,*,ERR=1006) ZMAX
       IF(ZMAX.LE.0.0) THEN
        PRINT *,' *** ERROR *** Depth must be greater 0.0'
        WRITE(JZ,*) BELL
        goto 1006
       endif
      ELSE IF(ANS.EQ.'C'.OR.ANS.EQ.'c') THEN
1008   WRITE(JZ,1009) 
1009   FORMAT(' Enter field point spacing (Kilometers) :',$)
       READ(*,*,ERR=1008) SPACE(ITYPE)
       IF(SPACE(ITYPE).LE.0.0) THEN
        PRINT *,' *** ERROR *** spacing must be greater than 0.0'
        WRITE(JZ,*) BELL
        GOTO 1008
       ENDIF
      ELSE IF(ANS.EQ.'D'.OR.ANS.EQ.'d') THEN
1010   WRITE(JZ,1011) 
1011   FORMAT(' Enter no. of field points :',$)
       READ(*,*,ERR=1010) NFIELD(ITYPE)
       IF(NFIELD(ITYPE).GT.MAXOBS.OR.NFIELD(ITYPE).LT.0) GOTO 1008 
       XMAXS(ITYPE) = COORD(NFIELD(ITYPE))
       IF(XMAXS(ITYPE).GT.XMAX) WRITE(JZ, 1012) XMAX, XMAXS(ITYPE)
1012   FORMAT(' *WARNING*', F10.2, ' IS THE EDGE OF SCREEN, '
     + ,F10.2,' IS LAST FIELD POINT')
      ELSE IF(ANS.EQ.'E'.OR.ANS.EQ.'e') THEN
1013   WRITE(JZ,1014) 
1014   FORMAT(' Enter anomaly minimum :',$)
       READ(*, *,ERR=1013) ANOMIN(ITYPE)
1015   WRITE(JZ,1016) 
1016   FORMAT(' Enter anomaly maximum :',$)
       READ(*, *,ERR=1015) ANOMAX(ITYPE)
       IF(ANOMIN(ITYPE).GE.ANOMAX(ITYPE)) THEN
        WRITE(JZ,*) 'ERROR ANOMALY MINIMUM IS GREATER THEN MAXIMUM'
        GOTO 1013
       ENDIF
      ELSE IF(ANS.EQ.'F'.OR.ANS.EQ.'f') THEN
1017   WRITE(JZ,1018) 
1018   FORMAT(' Enter diff. minimum :',$)
       READ(*, *,ERR=1017) DIFMIN(ITYPE)
1019   WRITE(JZ,1020) 
1020   FORMAT(' Enter diff. maximum :',$)
       READ(*, *,ERR=1019) DIFMAX(ITYPE)
       IF(DIFMIN(ITYPE).GE.DIFMAX(ITYPE)) THEN
        WRITE(JZ,*) 'ERROR DIFF. MINIMUM IS GREATER THEN MAXIMUM'
        GOTO 1017
       ENDIF
      ELSE IF(ANS.EQ.'G'.OR.ANS.EQ.'g') THEN
1025   WRITE(JZ,1030) 
1030   FORMAT(' Enter gridx :',$)
       READ(*, *,ERR=1025) GX
       IF(GX.LT.0.0) THEN
        PRINT 1031
1031    format(' *** ERROR *** grid must be greater than 0.0')
        WRITE(JZ,*) BELL
        goto 1025
       endif
      ELSE IF(ANS.EQ.'H'.OR.ANS.EQ.'h') THEN
1032   WRITE(JZ,1033) 
1033   FORMAT(' Enter gridz :',$)
       READ(*, *,ERR=1032) GZ
       IF(GZ.LT.0.0) THEN
        PRINT 1031
        WRITE(JZ,*) BELL
        goto 1025
       endif
      ELSE
       PRINT *,' *** ERROR *** illegal input'
       WRITE(JZ,*) BELL
      ENDIF
      goto 1000

*<<*>>* INPUT BODY
*+++++++++++++++++

2000  WRITE(JZ, 2001) 
2001  FORMAT(' Enter body number :',$)
      READ(*, *,ERR=2000) IBOD
      IF(IBOD.GT.MAXBOD.OR.IBOD.LT.1) THEN
        WRITE(JZ, *) 'NON VALID BODY NUMBER'
        GOTO 2000
      ENDIF
2002  WRITE(JZ,2003) 
2003  FORMAT(' Enter # of points, enter 0 to delete this body :',$)
      READ(*, *,ERR=2002) NPT
      IF(NPT.GT.MAXNPT) THEN
        WRITE(JZ, *) ' MAXIMUM NUMBER OF POINTS PER BODY IS :', MAXNPT
        GOTO 2002
      ELSE IF(NPT.EQ.0) THEN
        GOTO 2180
      ELSE IF(NPT.LT.3) THEN
        WRITE(JZ,*) ' MINIMUM NUMBER OF POINTS PER BODY IS : 3'
        GOTO 2002
      ENDIF
      IF(NPTS(IBOD).GT.0) THEN
        WRITE(JZ,2005) 
2005    FORMAT(' DUPLICATE BODY. OVERWRITE?  (Y/N) :',$)
        READ(*, 2110) IANS
2110    FORMAT(A1)
        IF(IANS.NE.'y'.AND.IANS.NE.'Y') GOTO 350
      ENDIF
      IANOM(IBOD) = 0
      IF(IBOD.GT.NBODS) NBODS = IBOD
2115  WRITE(JZ,2116) 
2116  FORMAT(' Enter half-strike lengths (YMIN, YMAX) of body :',$)
      READ(*, *,ERR=2115) BDY(IBOD,1), BDY(IBOD,2)
2129  WRITE(JZ,2130)
2130  FORMAT(' Enter Y axis to strike direction :',$)
      READ(*, *,ERR=2129) YANG(IBOD)
      IF(NPTS(IBOD).GT.0) THEN
        WRITE(JZ,2117) 
2117    FORMAT(' Do you wish to enter body points?  (y/n)',$)
        READ(*, 2110) IANS
        IF(IANS.NE.'y'.AND.IANS.NE.'Y') GOTO 350
      ENDIF
      NPT = NPT + 1
      NPTS(IBOD) = NPT
      NPT1 = NPT - 1
      GOTO 2120
2119  WRITE(JZ,*)'ERROR WITH LAST ENTRY, PLEASE ENTER ALL POINTS AGAIN'
2120  WRITE(JZ, *) 'Enter ', npt1, ' Xs & Zs'
      READ(*, *,ERR=2119) (X(IBOD,I),Z(IBOD,I),I = 1,NPT1)
      X(IBOD,NPT) = X(IBOD,1)
      Z(IBOD,NPT) = Z(IBOD,1)
      GOTO 350
2180  WRITE(JZ, *) 'BODY ', IBOD, ' DELETED'
      WRITE(JZ,2185) IBOD
2185  FORMAT(' Do you want to type body #',I3, ' (y/n) :',$)
      READ(*, 2110) IANS
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') CALL TYPBOD(IBOD)
2190  IANOM(IBOD) = 0
      RHOMAG(IBOD,ITYPE) = 0.
      RMMAX(IBOD,ITYPE) = 0.
      RMMIN(IBOD,ITYPE) = 0.
      NPTS(IBOD) = 0

C IF BODY BEING DELETED REDUCE BODY COUNTER
      J = 0
      DO I = 1, NBODS
        IF(NPTS(I).GT.0) J = I
      ENDDO
      NBODS = J
      GOTO 350


3000  CALL ALTPOIN
      GOTO 350


*<<*>>* ENTER BODY PARAMETERS
*++++++++++++++++++++++++++++

4000  CALL PARAM(IBOD)
      GOTO 350


*<<*>>* ENTER PARAMETER ADJUST MODE
*++++++++++++++++++++++++++++++++++

4999  WRITE(JZ,*) ' ERROR - ENTRY MUST BE A NUMBER '
5000  WRITE(JZ,5001)
5001  FORMAT( ' ENTER BODY NUMBER,  0 = ALL :',$)
      READ(*,*,ERR=4999) IMOD
      IF(IMOD.LT.0.OR.IMOD.GT.NBODS) GOTO 4999
      CALL AMPL(IMOD)
      GOTO 350

*<<*>>* WRITE MAXIMUM ARRAY STATISTICS
*+++++++++++++++++++++++++++++++++++++

6000  WRITE(JZ, *) 'MAXIMUM NUMBER OF POINTS PER BODY : ', MAXNPT
      WRITE(JZ, *) 'MAXIMUM NUMBER OF BODIES          : ', MAXBOD
      WRITE(JZ, *) 'MAXIMUM NUMBER OF OBSERVATIONS    : ', MAXOBS
      GOTO 400

*<<*>>* DELETE A MODELS FILE

7000  CALL DELMOD(IRET,MODEL,INAME, MODDEL, MODFILE)
      IF(IRET.EQ.-1) THEN
        WRITE(JZ, *) ' MODEL ', INAME, ' DELETED'
      ELSE
        WRITE(JZ, *) ' MODEL ', INAME, ' NOT DELETED'
      ENDIF
      GOTO 400

*<<*>>* READ MODEL
*+++++++++++++++++

8000  IF(NAME.NE.IBLANK) THEN
        IRET = 0
        CALL READF(IRET,MODEL)
        IF(IRET.EQ.-1) THEN
          WRITE(JZ, *) 'Model ', name, ' read'
        ELSE
          WRITE(JZ, *) 'Model ', name, ' not read'
        ENDIF
      ELSE
        WRITE(JZ, *) '** CANNOT READ AN UNKNOWN QUANTITY'
        GOTO 400
      ENDIF
      GOTO 350


*<<*>>* WRITE MODEL
*++++++++++++++++++

9000  IF(NAME.NE.IBLANK) THEN
        CALL WRITEF(IRET,MODEL)
        IF(IRET.EQ.-1) THEN
          WRITE(JZ, *) 'Model ', name, ' written'
        ELSE
          WRITE(JZ, *) 'Model ', name, ' not written'
        ENDIF
      ELSE
        WRITE(JZ, *) ' NAME MODEL BEFORE WRITING '
      ENDIF
      GOTO 400


*<<*>>* CALCULATES ANOMALIES
*+++++++++++++++++++++++++++

10000 CALL REINIT
      CALL CALCAN
      WRITE(JZ,*) ' ANOMALY CALCULATED '
      GOTO 400


*<<*>>* OUTPUTS ANOMALIES
*++++++++++++++++++++++++

11000 IF(DABS(CALC(MAXCAL,1)).LT.1.D-9) THEN
      WRITE(JZ,*)' WARN - COMPUTE CALCULATED ANOMALY PROFILE FIRST '
      GOTO 350
      ENDIF
      GOTO 11001
10999 WRITE(JZ,*) 'ERROR, ENTRY MUST BE A NUMBER'
11001 WRITE(JZ,11200) 
11200 FORMAT(' Enter body number (0 = all) :',$)
      READ(*, *,ERR=10999) IBOD
      IF(IBOD.GE.0.AND.IBOD.LE.MAXBOD) THEN
      NF = NFIELD(ITYPE)
      WRITE(JZ,11300)NF
11300 FORMAT(' Number of profile points = :',I5)
      GO TO 11400
11399 WRITE(JZ,*) ' ERROR, RE-ENTER NUMBER '
11400 WRITE(JZ,11405)
11405 FORMAT(' Enter first point to be typed :',$) 
      READ(*,*,ERR=11399) IST
      IF(IST.LE.0.OR.IST.GE.MAXOBS) GOTO 11399
      GO TO 11500
11499 WRITE(JZ,*) ' ERROR, RE-ENTER NUMBER '
11500 WRITE(JZ,11505)
11505 FORMAT(' Enter last point to be typed :',$)
      READ(*,*,ERR=11499) NF
      IF(NF.LE.IST.OR.NF.GT.MAXOBS) GOTO 11499
      CALL TYPANO(IBOD,IST,NF)
      GOTO 400
      ENDIF
      WRITE(JZ, *) '** ERROR'
      GOTO 400


*<<*>>* UPDATE OBSERVATIONS
*++++++++++++++++++++++++++

12000 WRITE(JZ,12001)
12001 FORMAT(' Routine to enter or update Observations',//,
     1 ' Do you have a file with your Observations in it... y or n :',$)
      READ(*,2110) IANS
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
12002  WRITE(JZ,12003) 
12003  FORMAT(' Enter the name of your Observation file :',$)
       READ(*,*) OBSFILE
       OPEN(8,FILE=OBSFILE,STATUS='OLD',FORM='FORMATTED',ERR=12004)
       GOTO 12007
12004  WRITE(JZ,12005) OBSFILE
12005  FORMAT(' *** ERROR *** cannot open file : ',a30)
       GOTO 400
12006  WRITE(JZ,*) '*** READ ERROR , only ',num0,' Observations read'
       CLOSE(8)
       GOTO 350
12007  num0 = 0
       IF(ITYPE.EQ.1) THEN
         DO I = 1, MAXOBS
           OBS(1,I) = -9999.
           OBS(3,I) = -9999.
         ENDDO
       ELSE
         DO I =1, MAXOBS
           OBS(2,I) = -9999.
         ENDDO
       ENDIF
       DO I=1,MAXOBS
         OBS(ITYPE+3,I)=0.
       ENDDO
12010  IF(ITYPE.EQ.1) THEN
          READ(8,*,END=12015,ERR=12006) J,OB,OE
       ELSE
          READ(8,*,END=12015,ERR=12006) J,OB
       ENDIF
       IF(J.LE.0) GOTO 12015
       IF(J.LE.NFIELD(ITYPE)) THEN
        OBS(ITYPE,J) = OB
        NUM0 = NUM0 + 1
        IF(ITYPE.EQ.1) OBS(3,J) = OE
       ELSE
        WRITE(JZ,*) '** ERROR : field point # ',J,' is illegal'
       ENDIF
       GOTO 12010
12015  WRITE(JZ,*) ' Number of Observations read was ',num0
       CLOSE(8)
       GOTO 12500
      ELSE
       IF(NFIELD(ITYPE).LE.0) THEN
12200    WRITE(JZ,*) ' Enter number of Observed data points :'
         READ(*,*,ERR=12200) NF
         IF(NF.LE.0.OR.NF.GT.MAXOBS) GOTO 12200
         NFIELD(ITYPE) = NF
       ENDIF
       IF(ITYPE.EQ.1) THEN
         WRITE(JZ,*) ' ENTER OBSERVED GRAVITY DATA '
       ELSE
         WRITE(JZ,*) ' ENTER OBSERVED MAGNETIC DATA '
       ENDIF
       GOTO 12207
12206  WRITE(JZ,*) 'ERROR, ENTRIES MUST BE NUMBERS, PLEASE TRY AGAIN'
12207  IF(ITYPE.EQ.1) THEN
         WRITE(JZ,*) ' Enter no., gravity and elev. (0 0 0 to exit) :'
         READ(*,*,ERR=12206) J, OB, OE
       ELSE
         WRITE(JZ, *) 'Enter no. and magnetic value (0 0 to exit) :'
         READ(*, *,ERR=12206) J, OB
       ENDIF
       IF(J.EQ.0) GOTO 12500
       IF(J.LE.NFIELD(ITYPE)) THEN
         OBS(ITYPE,J) = OB
         IF(ITYPE.EQ.1) OBS(3,J) = OE
       ELSE
         WRITE(JZ, *) '** ERROR : ', J, ' ILLEGAL'
       ENDIF
       GOTO 12207
      ENDIF
12500 DO I = 1, MAXOBS
        IF(OBS(ITYPE,I).LE.-9999.) GOTO 12510
        ISTART = I
        GOTO 12520
12510 ENDDO
12520 NOBS = 0
      DO I = ISTART, MAXOBS
        IF(OBS(ITYPE,I).LE.-9999.) GOTO 12540
        NOBS = NOBS + 1
        OBS(ITYPE+3,I)=1.
      ENDDO
12540 LEND(ITYPE) = ISTART + NOBS - 1
      LSTART(ITYPE) = ISTART
      GO TO 350

*<<*>>* PRINT OUT OBSERVATIONS
*+++++++++++++++++++++++++++++

13000 WRITE(JZ, *) 'OBSERVATIONS'
      NF = NFIELD(ITYPE)
      IST =1
      WRITE(JZ,13300)NF
13300 FORMAT(' Number of profile points = :',I5)
      GO TO 13400
13399 WRITE(JZ,*) ' ERROR, RE-ENTER NUMBER '
13400 WRITE(JZ,13405)
13405 FORMAT(' Enter first point to be typed :',$) 
      READ(*,*,ERR=13399) IST
      IF(IST.LE.0.OR.IST.GE.MAXOBS) GOTO 13399
      GO TO 13500
13499 WRITE(JZ,*) ' ERROR, RE-ENTER NUMBER '
13500 WRITE(JZ,13505)
13505 FORMAT(' Enter last point to be typed :',$)
      READ(*,*,ERR=13499) NF
      IF(NF.LE.IST.OR.NF.GT.MAXOBS) GOTO 13499
      IF(ITYPE.EQ.1) WRITE(JZ,13506)
      IF(ITYPE.EQ.2) WRITE(JZ,13507)
13506 FORMAT(' Point   X(km)     mGal     Elev(km)  Screen')
13507 FORMAT(' Point   X(km)      nT     Screen')
      DO I = IST, NF
        IF(ITYPE.EQ.1) THEN
          WRITE(JZ, 13550) I,COORD(I),OBS(ITYPE,I),OBS(3,I),OBS(4,I)
13550     FORMAT(1X,I3,1X,F10.3,1X,F10.3,1X,F10.4,1X,F3.0)
        ELSE
          WRITE(JZ, 13551) I, COORD(I), OBS(ITYPE,I), OBS(5,I)
13551     FORMAT(1X,I3,1X,F10.3,1X,F10.3,1X,F3.0)
        ENDIF
      ENDDO
13600 WRITE(JZ,*)' DO YOU WISH TO CHANGE THE SCREENING ARRAY, Y or N'
      READ(*,'(A1)',ERR=13600) IANS
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
13601   WRITE(JZ,*) ' DO YOU WISH TO ELIMINATE ALL SCREENING, Y or N'
        READ(*,'(A1)',ERR=13601) IANS
        IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
          ISTART=LSTART(ITYPE)
          IEND=LEND(ITYPE)
          DO I = ISTART,IEND
            OBS(ITYPE+3,I) = 1.
          ENDDO
          WRITE(JZ,*) ' SCRATCH FILE RESET '
        ELSE
13610     WRITE(JZ,*)' DO YOU WISH TO CHANGE STATUS OF SCREENED POINTS,
     + Y or N'
          READ(*,'(A1)',ERR=13610) IANS
          IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
13615       WRITE(JZ,*) ' ENTER NUMBER OF POINT TO BE CHANGED '
            READ(*,*,ERR=13615) ISCRT
            IF(ISCRT.GT.0.AND.ISCRT.LE.MAXOBS) THEN
              IF(OBS(ITYPE+3,ISCRT).GT.1.E-1) THEN
                OBS(ITYPE+3,ISCRT) = 0.
              ELSE
                OBS(ITYPE+3,ISCRT) = 1.
              ENDIF
              GOTO 13615
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      GOTO 400


*<<*>>* INPUT COMMENTS
*+++++++++++++++++++++

14000 WRITE(JZ, *) 'Enter comments (up to 75 char) :'
      READ(*, 14010) KOMMNT
14010 FORMAT(A80)
      GOTO 400


*<<*>>* OUTPUT COMMENTS
*++++++++++++++++++++++

15000 WRITE(JZ, 15010) KOMMNT
15010 FORMAT(A80)
      GOTO 350


*<<*>>* INPUT MODEL NAME
*+++++++++++++++++++++++

16000 WRITE(JZ,16005) 
16005 FORMAT(' Enter model name (10 char max) :',$)
      READ(*, 16010) NAME
16010 FORMAT(A10)
      GOTO 350


*<<*>>* READ IN A MODEL FROM THE FILE
*++++++++++++++++++++++++++++++++++++

17000 IRET = 1
      CALL READF(IRET,MODEL)
      GOTO 400


*<<*>>* INSERTS BODY POINTS
*++++++++++++++++++++++++++

18000 CALL INSE
      GOTO 350


*<<*>>* DELETES BODY POINTS
*++++++++++++++++++++++++++

19000 CALL DELE
      GOTO 350


*<<*>>* EXIT
*+++++++++++

20000 CLOSE(MODEL)
      CLOSE(ISCR,STATUS='DELETE')
      WRITE(JZ, *) 'ADIOS AMIGO'
      CALL GCLOSE
      STOP

**** GRAPHICS
*************
21000 IF(ITERM.LE.2) THEN
        CALL GCLEAR
        CALL GRAP
      ENDIF
      GOTO 400


*<<*>>* RECOVER
*++++++++++++++

22000 CALL RECO
      GOTO 400


*<<*>>* PRINT OUT SCALES AND OBSERVATIONS, OR READ A *.DMP FILE
*++++++++++++++++++++++++++++++++++++++++
23000 WRITE(JZ,22901)
22901 FORMAT(' DO YOU WANT TO READ A *.DMP FILE - Y OR N :',$)
      READ(*,2110,ERR=23000) IANS
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
22902 WRITE(JZ,22903)
22903 FORMAT(' ENTER FILE NAME OF THE *.DMP FILE :',$)
      READ(*,23008,ERR=22902)FDUMP
      JJZ=8
      OPEN(JJZ,FILE=FDUMP,FORM='FORMATTED',STATUS='OLD',ERR=23000)
      JZ=8
      CALL RDUMP
      CLOSE(JZ)
      JZ=6
      WRITE(JZ,*)' NAME OF *.DMP FILE READ : ',FDUMP
      CALL REINIT
      GOTO 400
      ENDIF
22999 WRITE(JZ,23001) 
23001 FORMAT(' Do you want to create a *.DMP file - Y or N : ',$)
      READ(*,2110,ERR=22999) IANS
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
23007   WRITE(JZ,23002)
23002   FORMAT(' Enter file name for the dump : ',$)
        READ(5,23008,ERR=23007) FDUMP
23008   FORMAT(A30)
        JJZ=8
        OPEN(JJZ,FILE=FDUMP,FORM='FORMATTED',STATUS='UNKNOWN',ERR=23007)
        JZ=8
      CALL TSCALE
      CALL TYPBOD(0)
      WRITE(JZ, 23030) OFFSET(ITYPE)
23030 FORMAT('VALUE ADDED TO CALCULATED DATA IS : ',F10.2)
      WRITE(JZ,*) 'OBSERVATIONS'
      NF = NFIELD(ITYPE)
      DO I = 1, NF
        IF(ITYPE.EQ.1) THEN
          WRITE(JZ, 23100) I, COORD(I), OBS(ITYPE,I), OBS(3,I),
     1    CALC(MAXCAL,I)+OFFSET(ITYPE), OBS(4,I)
23100     FORMAT(I4,4F11.3,F4.0)
        ELSE
          WRITE(JZ, 23101) I,COORD(I),OBS(ITYPE,I),
     1     CALC(MAXCAL,I)+OFFSET(ITYPE),OBS(5,I)
23101     FORMAT(I4,3F11.3,F4.0)
        ENDIF
      ENDDO
      CLOSE(JZ)
      JZ=6
      ENDIF
      GOTO 400

*<<*>>* CHANGES PARAMETER ESCALE AND FRACTIONAL ACCURACIES FOR DFS001

24000 WRITE(JZ, *) 'OLD ESC AND FRAC VALUES',ESC,FRAC
      WRITE(JZ, *) 'ENTER NEW VALUES, OR / TO RETAIN OLD VALUES'
      READ(*, *) ESC,FRAC
      WRITE(JZ, *) 'NEW ESC AND FRAC VALUES',ESC,FRAC
      GOTO 400      

*<<*>>* DISPLAYS BODY PARAMETERS
*+++++++++++++++++++++++++++++++

25000 CALL TYPAR(0)
      GOTO 400


*<<*>>* INPUT PARAMETERS FOR MAG MODEL
*+++++++++++++++++++++++++++++++++++++

26000 ITYPE = 2
      WRITE(JZ, *) 'MAGNETIC MODEL'
26001 WRITE(JZ,26002) 
26002 FORMAT(' Enter angle measured clockwise from Geographic North',
     +' to the X Axis,',/,' or / to retain old value :',$)
      READ(*, *,ERR=26001) XTON
      XTON = AMOD(XTON,360.)
      IF(XTON.LT.0.) XTON = XTON + 360.
26003 WRITE(JZ,26004) 
26004 FORMAT(' Enter declination, or / to retain old value :',$)
      READ(*, *,ERR=26003) FDEC
      FDEC = AMOD(FDEC,360.)
      IF(FDEC.LT.0.) FDEC = FDEC + 360.
26005 WRITE(JZ,26006) 
26006 FORMAT(' Enter inclination, or / to retain old value :',$)
      READ(*, *,ERR=26005) FINCL
      FINCL = AMOD(FINCL,360.)
      IF(FINCL.LT.0.) FINCL = FINCL + 360.
26007 WRITE(JZ,26008) 
26008 FORMAT(' Enter Survey Elevation, or / to retain old value :',$)
      READ(*, *,ERR=26007) ZCON(2)
      IF(ZCON(2).LE.0.001) ZCON(2) = .001
      IWFP = IWFLAG
      IWFLAG = 1
26009 WRITE(JZ,26010) 
26010 FORMAT(' If this is a new model',/,
     +'      1) Choose the NAME Option to name the model',/,
     +'      2) Choose the COMM Option to enter comments',/,
     +'      3) Choose the SCAL Option to enter scale parameters',/,
     +'      4) Choose the OBSE Option to enter the observed data',/,
     +'      5) Choose the BODY Option to enter bodies',/,
     +'      6) choose the PARA Option to enter body parameters.')
      CALL REINIT
      GOTO 350


*<<*>>* INPUT PARAMETERS FOR GRAV MODEL
*++++++++++++++++++++++++++++++++

27000 IF(ITYPE.NE.1) IWFLAG = IWFP
      ITYPE = 1
      WRITE(JZ, *) 'GRAVITY MODEL'
27001 WRITE(JZ,27002)
27002 FORMAT(' Enter angle measured clockwise from Geographic North',
     +' to the X Axis,',/,' or / to retain old value :',$)
      READ(*, *,ERR=27001) XTON
      XTON = AMOD(XTON,360.)
      IF(XTON.LT.0.) XTON = XTON + 360.
27003 WRITE(JZ,27004)
27004 FORMAT(' Enter Background Density Value,'
     +' or / to retain old value :',$)
      READ(*,*,ERR=27003) BKDEN
27005 WRITE(JZ,27006) 
27006 FORMAT(' If this is a new model',/,
     +'      1) Choose the NAME Option to name the model',/,
     +'      2) Choose the SCAL Option',/,
     +'      3) Choose the OBSE Option to enter the observed data',/,
     +'      4) Choose the BODY Option to enter bodies',/,
     +'      5) choose the PARA Option to enter body parameters.')
      CALL REINIT
      GOTO 350


*<<*>>* CROSS TOGGLE
*+++++++++++++++++++

28000 IF(ICROSS.EQ.0.OR.ICROSS.EQ.2) THEN
        ICROSS = ICROSS + 1
        WRITE(JZ, *) 'Crosses enabled'
      ELSE
        ICROSS = ICROSS - 1
        WRITE(JZ, *) 'Crosses disabled'
      ENDIF
      GOTO 400


*<<*>>* ZOOM
*+++++++++++

29000 WRITE(JZ,29010) 
29010 FORMAT(' Enter min X :',$)
      READ(*, *,ERR=29000) SKXMIN
29020 WRITE(JZ,29030) 
29030 FORMAT(' Enter max X :',$)
      READ(*, *,ERR=29020) SKXMAX
29040 WRITE(JZ,29050) 
29050 FORMAT(' Enter min Z :',$)
      READ(*, *,ERR=29040) SKZMIN
29060 WRITE(JZ,29070) 
29070 FORMAT(' Enter max Z :',$)
      READ(*, *,ERR=29060) SKZMAX
      IF(SKXMIN.GE.SKXMAX) GOTO 29000
      IF(SKZMIN.GE.SKZMAX) GOTO 29000
      IF(SKZMIN.LT.0.) GOTO 29000
      GOTO 350


*<<*>>* DETERMINES MAX. AND MIN. VALUES OF OBSERVED DATA
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++

30000 CALL LIMI
      GOTO 400


*<<*>>* GRAPHICS SUPPRESSION
*+++++++++++++++++++++++++++

31000 WRITE(JZ,31005) 
31005 FORMAT(' Do you want graphics disabled? (y/n) :',$)
      READ(*, 31010) IANS
31010 FORMAT(A1)
      IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
        ISCOPE = 2
      ELSE
        ISCOPE = 1
        WRITE(JZ,31020) 
31020   FORMAT(' Do you want grid suppressed? (y/n) :',$)
        READ(*, 31010) IANS
        IF(IANS.EQ.'y'.OR.IANS.EQ.'Y') THEN
          ISUPG = -1
        ELSE
          ISUPG = 1
        ENDIF
      ENDIF
      GOTO 350


*<<*>>* OUTPUT BODY
*++++++++++++++++++

32000 WRITE(JZ,32003)
32003 FORMAT(' ENTER BODY NUMBER, 0=ALL :',$) 
      READ(*, *,ERR=32000) IBOD
      IF(IBOD.GE.0.AND.IBOD.LE.NBODS) THEN
        CALL TYPBOD(IBOD)
        GOTO 400
      ENDIF
      WRITE(JZ, *) '** ERROR ** invalid body number'
      GOTO 400


*<<*>>* PRINT OUT SCALES
*+++++++++++++++++++++++

33000 CALL TSCALE
      GOTO 400


*<<*>>* CALCULATE OFFSET VALUE
*+++++++++++++++++++++++++++++

34000 SUM = 0.
      IF(DABS(CALC(MAXCAL,1)).LT.1.D-9) THEN
      WRITE(JZ,*)' WARN - COMPUTE CALCULATED ANOMALY PROFILE FIRST '
      GOTO 350
      ENDIF
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      DO I = ISTART, IEND
        SUM = SUM + OBS(ITYPE,I) - CALC(MAXCAL,I)
      ENDDO
      OFFSET(ITYPE) = SUM/(IEND - ISTART + 1)
      IF(ITYPE.EQ.1) WRITE(JZ,*)' GRAVITY OFFSET = :', OFFSET(ITYPE)
      IF(ITYPE.EQ.2) WRITE(JZ,*)' MAGNETIC OFFSET = :', OFFSET(ITYPE)
      CALL SAV(NBACK)
      GOTO 350


*<<*>>* TOGGLES DIFFERENCE MODE
*++++++++++++++++++++++++++++++

35000 IDIFF = -IDIFF
      GOTO 400


*<<*>>* MENU OF COMMANDS
*+++++++++++++++++++++++

36000 WRITE(JZ, *) ' MENU OF TEXT WINDOW COMMANDS'
      WRITE(JZ,36100)
36100 FORMAT(' BODY - INPUT BODY POINTS               ',
     +       ' CANO - CALCULATE MODEL ANOMALY     ',/,
     1       ' COMM - INPUT MODEL COMMENTS            ',
     2       ' CONT - AUTO ADJUSTS BODY CONTRASTS ',/,
     3       ' CROS - TOGGLE BODY POINT CROSSES       ',
     4       ' DELE - DELETE A BODY POINT         ',/,
     5       ' DELM - DELETE A MODEL FILE             ',
     6       ' DIFF - PLOTS ANOMALY DIFFERENCES   ',/,
     7       ' DUMP - LIST ALL INFORMATION ON FILE    ',
     8       ' EXIT - EXIT FROM MAGRAV            ',/,
     9       ' FILL - TOGGLE BODY FILLING             ',
     A       ' GRAP - ENTER GRAPHICS MODE         ')
      WRITE(JZ,36150)
36150 FORMAT(' GRAV - SWITCH MODEL TYPE TO GRAVITY    ',
     B       ' HALF - CHANGE BODY HALF-LENGTHS    ',/,
     C       ' INSE - INSERT A BODY POINT             ',
     D       ' ISOS - FIT CONSTRAINED BY ISOSTASY ',/,
     E       ' LIMI - OBS. ANOM. MAX. AND MIN.        ',
     F       ' MAGN - SWITCH MODEL TYPE TO MAGN.  ',/,
     G       ' NAME - INPUT MODEL NAME                ',
     H       ' OBSE - ENTER OBSERVATIONS          ',/,
     I       ' OFFS - CALCULATED ANOM. OFFSET         ',
     J       ' PARA - ENTER BODY PARAMETERS       ')
      WRITE(JZ,36200)
36200 FORMAT(' POIN - MOVE A BODY POINT               ',
     K       ' READ - READ A MODEL FROM DISK      ',/,
     L       ' RECO - RECOVER MODEL AFTER CRASH       ',
     M       ' REGC - REGIONAL CORRECTION         ',/,
     N       ' REST - RESET ESC AND FRAC              ',
     O       ' RMSR - RMS ERROR OF FIT            ',/,
     P       ' SCAL - SET SCALES FOR PLOT             ',
     Q       ' STAT - MAXIMUM NUMBER OF POINTS ETC',/,
     R       ' SUPP - GRAPHICS SUPPRESSION TOGGLE     ',
     S       ' TANO - LIST ANOMALIES              ',/,
     T       ' TBOD - LIST BODY POINTS                ',
     U       ' TCOM - LIST MODELS COMMENT         ',/,
     V       ' TMOD - LIST MODELS ON FILE             ',
     W       ' TOBS - LIST OBSERVATIONS           ',/,
     X       ' TPAR - LIST BODIES PARAMETERS          ',
     Y       ' TSCA - LIST SCALES FOR PLOT        ',/,
     Z       ' WRIT - WRITE CURRENT MODEL TO DISK     ',
     +       ' IWRT - CHANGE AUTO MODELLING OUTPUT',/,
     +       ' ZOOM - SET LIMITS FOR SUB AREA     ')
36300 CALL WHAT(IWHAT,2)
      IF(IWHAT.EQ.0) THEN
        WRITE(JZ,*) 'PARDON ?'
        GOTO 36300
      ENDIF
      IF(IWHAT.EQ.45) GOTO 400
      CALL IHELP(IWHAT)
      WRITE(JZ,36400)
36400 FORMAT(/'Hit ENTER to continue')
      READ(*,*)
      GOTO 36000
*<<*>* TOGGLES BODY FILLING 
*++++++++++++++++++++++++++

37000 IF(ICROSS.GT.1) THEN
       ICROSS=ICROSS-2
      ELSE
       ICROSS=ICROSS+2
      ENDIF
      GOTO 400

*<<*>>* CHANGES HALF-STRIKE LENGTH AND Y AXIS TO BODY STRIKE DIRECTION
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

38000 WRITE(JZ,2001) 
      READ(*, *,ERR=38000) IBOD
      IF(IBOD.GT.MAXBOD.OR.IBOD.LT.1) THEN
        WRITE(JZ, *) 'NON VALID BODY NUMBER'
        GOTO 38000
      ENDIF
38100 WRITE(JZ,38200) IBOD,BDY(IBOD,1),BDY(IBOD,2)
38200 FORMAT(' Body # ',I2,' half-strike lengths are : ',2f10.2,/,
     1 ' Enter new half-strike lengths :',$)
      READ(*, *,ERR=38100) BDY(IBOD,1), BDY(IBOD,2)
38300 WRITE(JZ,38400)
38400 FORMAT(' Enter Y axis to strike direction :',$)
      READ(*, *,ERR=38300) YANG(IBOD)
      IANOM(IBOD) = 0
      GOTO 350

*<<*>>* USES COLUMN DIFFERENTIAL PRESSURES IN MODELLING PROCEDURE
*+++++++++++++++++++++++++++++++++++++++++++++++++

39000 CALL ISOSTA(0, IERS)
      GO TO 350

*<<*>>* APPLIES A LINEAR REGIONAL CORRECTION TO THE OBSERVED DATA
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

40000 WRITE(JZ,40001)
40001 FORMAT(/'WARNING - EXECUTION OF THIS OPTION ALTERS THE OBSERVED',
     */' DATA WHICH BECOMES PERMANENT WHEN THE WRIT OPTION IS EXECUTED'
     */' AND MAGRAV IS EXITED. TO RECOVER THE OBSERVED DATA USE THE',
     */' READ OPTION IF THE WRIT OPTION HAS NOT BEEN EXECUTED,',
     */' OTHERWISE USE THE RECO OPTION BEFORE EXITING MAGRAV.'/)
      WRITE(JZ,*)' ENTER VALUE TO BE SUBTRACTED FROM THE OBSERVED DATA'
      WRITE(JZ,*)' OR ENTER 0 TO HAVE THE LINEAR LEAST SQUARES TREND'
      WRITE(JZ,*)' OF THE DIFFERENCES REMOVED :'
      READ(*,*,ERR=40000) RCOR
      CALL REGC(RCOR)
      GOTO 350

*<<*>>* DETERMINES RMS OF DIFFERENCES BETWEEN OBSERVED AND CALC. DATA
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

41000 CALL RMSR(RMS,0)
      GOTO 400

*<<*>>* RESETS THE VALUE OF IWRITE
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

42000 WRITE(JZ,42001) IWRITE
42001 FORMAT(' Current value of IWRITE is ', I5,' The default is 0')
      WRITE(JZ,42003)
42003 FORMAT(' Enter new IWRITE, or / to retain the old value',
     */' Enter -1 to monitor optimization',$)
      READ(*,*,ERR=42000) IWRITE
      WRITE(JZ,42002) IWRITE
42002 FORMAT(' IWRITE = ',I5)
      GOTO 400
      END

*****************************************************************
C  OPTIONS SUBROUTINE
C
C  READS USER COMMAND AND RETURNS APPROPRIATE VALUE OF IWHAT
C    IWHAT = 0 : UNRECOGNIZED COMMAND

      SUBROUTINE WHAT(IWHAT,L)

      CHARACTER*4 IANS, LCKOMMS(45), UCKOMMS(45)

      INCLUDE 'magrav.cmn'

      DATA LCKOMMS/'scal','body','poin','para','cont','stat','delm',
     +    'read','writ','cano','tano','obse','tobs','comm','tcom',
     +    'name','tmod','inse','dele','exit','grap','reco','dump',
     +    'rest','tpar','magn','grav','cros','zoom','limi','supp',
     +    'tbod','tsca','offs','diff','help','fill','stop','quit',
     +    'half','isos','regc','rmsr','iwrt','x'/

      DATA UCKOMMS/'SCAL','BODY','POIN','PARA','CONT','STAT','DELM',
     +    'READ','WRIT','CANO','TANO','OBSE','TOBS','COMM','TCOM',
     +    'NAME','TMOD','INSE','DELE','EXIT','GRAP','RECO','DUMP',
     +    'REST','TPAR','MAGN','GRAV','CROS','ZOOM','LIMI','SUPP',
     +    'TBOD','TSCA','OFFS','DIFF','HELP','FILL','STOP','QUIT',
     +    'HALF','ISOS','REGC','RMSR','IWRT','X'/

      DATA NKOMMS/45/

      IF(L.EQ.1) THEN
      WRITE(JZ, 1)
      ELSE
      WRITE(JZ,2)
      ENDIF 
1     FORMAT(' Option :',$)
2     FORMAT(' Enter command or x to return to Option menu:',$)
      READ (5, 150) IANS
150   FORMAT(A4)
      DO IWHAT = 1, NKOMMS
        IF(LCKOMMS(IWHAT).EQ.IANS) RETURN
        IF(UCKOMMS(IWHAT).EQ.IANS) RETURN
      ENDDO
      IWHAT = 0
      RETURN
      END



*********************************************************************
C
C   ALTER A POINT IN BODY SPECIFIED

      SUBROUTINE ALTPOIN

      INCLUDE 'magrav.cmn'
 
1200  WRITE(JZ, 1) 
1     FORMAT(' Enter body number :',$)
      READ(*, *,ERR=1200) IBOD
      IF(IBOD.GT.NBODS.OR.IBOD.LT.1) THEN
       PRINT *,' ** ERROR ** INVALID BODY NUMBER' 
       RETURN
      ENDIF
      CALL TYPBOD(IBOD)
1201  WRITE(JZ, 2) 
2     FORMAT(' Enter point to be changed :',$)
      READ(*, *,ERR=1201) NPT
      IF(NPT.GE.NPTS(IBOD).OR.NPT.LE.0) GOTO 400
1202  WRITE(JZ, 3) 
3     FORMAT(' Enter new X value :',$)
      READ(*, *,ERR=1202) XXX
1203  WRITE(JZ, 4) 
4     FORMAT(' Enter new Z value :',$)
      READ(*, *,ERR=1203) ZZ
      X(IBOD,NPT) = XXX
      Z(IBOD,NPT) = ZZ
      IANOM(IBOD) = 0
      IF(NPT.NE.1) RETURN

C  FIRST AND LAST POINTS CHANGED
      NPT = NPTS(IBOD)
      X(IBOD,NPT) = X(IBOD,1)
      Z(IBOD,NPT) = Z(IBOD,1)
      RETURN

400   WRITE(JZ, *) '** ERROR ** invalid body point number'
      RETURN
      END



********************************************************************
C INSERT POINTS INTO A BODY

      SUBROUTINE INSE

      INCLUDE 'magrav.cmn'

10    WRITE(JZ, 1)
1     FORMAT(' Enter body number :',$)
      READ(*, *,ERR=10) IBOD
      IF(IBOD.GT.NBODS.OR.IBOD.LT.1) THEN
       PRINT *,' ** ERROR ** INVALID BODY NUMBER' 
       RETURN
      ENDIF
      CALL TYPBOD(IBOD)
11    WRITE(JZ, 2)
2     FORMAT(' Enter insertion point :',$)
      READ(*, *,ERR=11) NPT
      IF(IBOD.GE.1.AND.IBOD.LE.NBODS) THEN
        IF(NPT.LT.NPTS(IBOD)) THEN
12        WRITE(JZ, 3)
3         FORMAT(' Enter X co-ord :',$)
          READ(*, *,ERR=12) XXX
13        WRITE(JZ, 4) 
4         FORMAT(' Enter Z co-ord :',$)
          READ(*, *,ERR=13) ZZ
          CALL INSERT(XXX,ZZ,IBOD,NPT,IRET)
          IF(IRET.LT.0) RETURN
        ENDIF
      ENDIF
      WRITE(JZ, *) '** ERROR'
      RETURN
      END


***********************************************************************
C
C   DELETES A POINT FROM A BODY

      SUBROUTINE DELE

      INCLUDE 'magrav.cmn'

10    WRITE(JZ, 1)
1     format(' Enter body number :',$)
      READ(*, *,ERR=10) IBOD
      IF(IBOD.GT.NBODS.OR.IBOD.LT.1) THEN
       PRINT *,' ** ERROR ** INVALID BODY NUMBER' 
       RETURN
      ENDIF
      CALL TYPBOD(IBOD)
      IF(NPTS(IBOD).LE.4) THEN
       WRITE(JZ,3)
3      FORMAT(' *** ERROR *** cannot delete a point from a body with',/,
     +        '               only three points. Use BODY command to',/,
     +        '               delete all of the body.')
       return
      ENDIF
11    WRITE(JZ, 2) 
2     format(' Enter point to be deleted :',$)
      READ(*, *,ERR=11) NPT
      IF(NPT.LT.1.OR.NPT.GE.NPTS(IBOD)) GOTO 2930
      XXX = X(IBOD,NPT)
      ZZ = Z(IBOD,NPT)
      CALL DELETE(IBOD,NPT,IRET)
      IF(IRET.LT.0) WRITE(JZ,2920) NPT,XXX,ZZ
2920  FORMAT(' Point',i3,' deleted = ',2f10.2)
      RETURN

2930  WRITE(JZ, *) '** ERROR ** invalid body point'
      RETURN
      END


***********************************************************************
C
      SUBROUTINE RECO

      INCLUDE 'magrav.cmn'
 
      COMMON /SAVEIT/ NSAVE

10    WRITE(JZ,11) nsave,nsave 
11    FORMAT(' You have',i5,' models that are recoverable',/,
     + ' 1 is the oldest and',i5,' is the newest,',/,
     + ' Which one would you like or 0 to abort recover :',$)
      READ(*, *,ERR=10) IRET1
      IF(IRET1.LT.0.OR.IRET1.GT.NSAVE) GOTO 10
      IF(IRET1.EQ.0) THEN
       WRITE(JZ,*) 'RECOVER ABORTED'
       RETURN
      ENDIF
      IRET = -IRET1
      CALL READF(IRET,ISCR)
      IF(IRET.GE.0) THEN
        WRITE(JZ, *) 'Model ', name, ' not recovered'
      ELSE
        WRITE(JZ, *) 'Model ', name, ' recovered'
        NSAVE = IRET1
      ENDIF
      RETURN
      END

***********************************************************************
C
C     FINDS THE MAX. AND MIN. VALUES OF THE OBSERVED DATA
C
      SUBROUTINE LIMI

      INCLUDE 'magrav.cmn'
 
      IF(NFIELD(ITYPE).EQ.0) RETURN
      AAMAX = -9999.
      AAMIN = +9999.
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      DO J = ISTART, IEND
        VAL = OBS(ITYPE,J)
        IF(VAL.GT.AAMAX) AAMAX = VAL
        IF(VAL.LT.AAMIN) AAMIN = VAL
      ENDDO
      WRITE(JZ,4070) AAMAX, AAMIN
4070  FORMAT(' MAXIMUM VALUE OF OBSERVED DATA : ',F10.2/' MINIMUM VALUE 
     +OF OBSERVED DATA : ',F10.2)
      RETURN
      END


*********************************************************************** 
C
C     FINDS THE ROOT MEAN SQUARE OF THE DIFFERENCES BETWEEN THE
C     OBSERVED AND CALCULATED DATA.
C
      SUBROUTINE RMSR(RMS,IGO)

      INCLUDE 'magrav.cmn'
        
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      IF(NBODS.EQ.0.OR.NFIELD(ITYPE).EQ.0) RETURN
      SUM=0.
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      IF(DABS(CALC(MAXCAL,ISTART)).LT.1.D-9.AND
     *.DABS(CALC(MAXCAL,IEND)).LT.1.D-9) THEN
      WRITE(JZ,*)' WARN - COMPUTE CALCULATED ANOMALY PROFILE FIRST '
      RETURN
      ENDIF
      ZDUM = OFFSET(ITYPE)
      DO J = ISTART, IEND
        SUM = SUM + (CALC(MAXCAL,J) + ZDUM - OBS(ITYPE,J))**2
      ENDDO
      RMS = SQRT(SUM/(IEND - ISTART + 1))
      IF(IGO.EQ.3)RETURN
      IF(IGO.EQ.1)GO TO 2
      WRITE(JZ,4081)RMS
4081  FORMAT( ' RMS OF RESIDUALS = :',E10.4)
C
C      OPTION TO OFFSET CALCULATED DATA WHEN PLOTTING
C
20    WRITE(JZ,4082) OFFSET(ITYPE)
4082  FORMAT(' OFFSET = :',F10.4,/,
     *' Enter new offset value, or / to retain the old value:',$)
      READ(*,*,ERR=20) OFFSET(ITYPE)
      IF(ITYPE.EQ.2.OR.IWFLAG.EQ.1)RETURN
2     WSUM=0.
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      DO I=ISTART,IEND
        WSUM=WSUM+(WD(I)**2)
      ENDDO
      IF(WSUM.EQ.0.)GO TO 99
      WRMS=SQRT(WSUM/(IEND-ISTART + 1))
      WFACT=WRMS/RMS
      IF(IGO.EQ.1)RETURN
      WRITE(JZ,100) WRMS
100   FORMAT(' RMS OF DIFFERENTIAL COLUMN PRESSURES = :',E10.4)
      RETURN
99    WRITE(JZ,*) ' WARN - WEIGHTS NOT DETERMINED '
      IWFLAG = 1
      RETURN
      END


***********************************************************************
C
C  PURPOSE TO REINITIALIZE ARRAY CALC AND IANOM     

      SUBROUTINE REINIT

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      DO I = 1, MAXCAL
        DO J = 1, MAXOBS
          CALC(I,J) = 0.D0
        ENDDO
      ENDDO
      DO I = 1, MAXBOD
        IANOM(I) = 0
      ENDDO
      RETURN
      END




********************************************************************
C
C  PURPOSE WRITES A NEW MODEL ON LOGICAL UNIT LUN
C IF LUN IS THE SCRATCH FILE A RECORD IS ADDED TO THE END
C IF A DUPLICATE NAME IS FOUND ON FILE IT MAY BE OVERWRITTEN
C IF THIS IS THE SCRATCH FILE IT IS ASSUMED TO BE AT THE EOF
C
C IRET : -1 O.K
C         0 EOF
C         1 PARITY
C         2 DISALLOWED DUPLICATE NAME OR PARITY ERROR ON READ

      SUBROUTINE WRITEF(IRET,LUN)

      CHARACTER LIST(20)*10

      INCLUDE 'magrav.cmn'
 
      COMMON /LIST/ ILIST, LIST

C  IF THIS IS SCRATCHFILE WRITE RECORD HERE
      IF(LUN.EQ.ISCR) GOTO 2000

C  ELSE SEARCH THRU DISK FILE TO CHECK FOR DUPLICATE NAMES
704   DO I = 1, ILIST
      IF(NAME.EQ.LIST(I)) GOTO 705
      ENDDO
      GOTO 1000 
C  DUPLICATE NAME
705   WRITE(JZ,706) 
706   format(' DUPLICATE MODEL. OVERWRITE? (Y/N) :',$)
      READ(*, 710) IANS
710   FORMAT(A1)
      IF(IANS.NE.'y'.AND.IANS.NE.'Y') THEN
        WRITE(JZ,711) 
711     format( ' Change model name? (y/n) :',$)
        READ(*, 710) IANS
        IF(IANS.NE.'y'.AND.IANS.NE.'Y') THEN
          IRET = 2
          RETURN
        ELSE
          WRITE(JZ,712) 
712       format(' Enter new name :',$)
          READ(*, 720) NAME
720       FORMAT(A10)
          GOTO 704
        ENDIF
      ENDIF
1000  IRET = -1
      DO I = 1, ILIST
      IF(NAME.EQ.LIST(I)) THEN
      IREC=I
      GOTO 779
      ENDIF
      ENDDO
      ILIST=ILIST+1
      IREC=ILIST
      LIST(ILIST)=NAME
779   CONTINUE
      CALL MODWRITE(MODEL,NAME,IREC)

C  SCRATCH FILE - ADD ON TO END
2000  CALL BUFFOUT(ISCR)
C      IRET = ISCR
      RETURN
      END



****************************************************************
C
C  TYPES BODY CO ORDINATES AND PARAMETERS
C

      SUBROUTINE TYPBOD(IBOD)

      INCLUDE 'magrav.cmn'
 
C  IBOD = 0  -  WRITE ALL BODIES
C  ELSE IBOD = BODY NUMBER

      IF(NBODS.EQ.0) RETURN
C   ALL BODIES
C     OR
C   ONE BODY
      IF(IBOD.EQ.0) THEN
        IBEGIN = 1
        IEND = NBODS
      ELSE
        IBEGIN = IBOD
        IEND = IBOD
      ENDIF
      WRITE(JZ,*) ' '

      DO I = IBEGIN, IEND 
        IF(NPTS(I).NE.0) THEN
          WRITE(JZ, 200) I
200       FORMAT('Body :',I5)
          WRITE(JZ,210) BDY(I,1),BDY(I,2)
210       FORMAT('Half-strike lengths :',2F11.3)
          WRITE(JZ,215) YANG(I)
215       FORMAT('Y axis to Strike Angle :',F5.1)
          IF(ITYPE.EQ.1)
     +      WRITE(JZ, 270) RMMIN(I,1), RHOMAG(I,1), RMMAX(I,1)
270         FORMAT(5X,'rho = ',3F10.3)
          IF(ITYPE.EQ.2)
     +      WRITE(JZ, 280) RMMIN(I,2), RHOMAG(I,2),
     +                      RMMAX(I,2), BDEC(I), BINCL(I)
280       FORMAT('Magnetizations',3f10.3/'dec ',f10.2,' incl',f10.2)
          NPT1 = NPTS(I) - 1
          DO J = 1, NPT1
            WRITE(JZ,290) J, X(I,J), Z(I,J)
290         FORMAT(I5,2F11.4)
          ENDDO
          IF(JZ.EQ.8) THEN
           IED=999
           WRITE(JZ,300) IED
300        FORMAT(I5)
         ELSE
           WRITE(JZ,*) ' '
         ENDIF
        ENDIF
      ENDDO
          IF(JZ.EQ.8) THEN
           IED=999
           WRITE(JZ,301) IED
301        FORMAT(6X,I5)
         ELSE
           WRITE(JZ,*) ' '
         ENDIF
      RETURN
      END



***************************************************************
C
C  SUBROUTINE TO WRITE BODY PARAMETERS

      SUBROUTINE TYPAR(IBOD)

      INCLUDE 'magrav.cmn'

C  GRAVITY
      IF(ITYPE.NE.2) THEN
        WRITE(JZ, *) ' BODY         RHO'
        DO I = 1, NBODS
          IF(NPTS(I).GT.0) THEN
            WRITE(JZ, 150) I, RMMIN(I,1), RHOMAG(I,1), RMMAX(I,1)
150         FORMAT(I5,1X,3F10.3)
          ENDIF
        ENDDO
        RETURN
      ENDIF

C  MAGNETIC
2000  WRITE(JZ, 2100)
2100  FORMAT(' BODY',15X,'MAGNETIZATION',10X,' DEC  ',' INCL  ')
      DO I = 1, NBODS
        IF(NPTS(I).GT.0) THEN
          WRITE(JZ, 151) I, RMMIN(I,2), RHOMAG(I,2),
     +                    RMMAX(I,2), BDEC(I), BINCL(I)
151       FORMAT (I5,1X,5F10.3)
        ENDIF
      ENDDO
      RETURN
      END



************************************************************
C
C  PURPOSE WRITES OUT ANOMALY FOR BODY IBOD
C   IF IBOD EQ.0 OR MAXBOD + 1,TOTAL ANOMALY IS PRINTED

      SUBROUTINE TYPANO(JBOD,IST,NF)

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      WRITE(JZ,300)OFFSET(ITYPE)
300   FORMAT(' VALUE ADDED TO CALCULATED DATA IS ',F10.2)
      IBOD = JBOD
      IF(JBOD.EQ.0)IBOD = MAXCAL
      IF(IBOD.LT.MAXCAL) WRITE(JZ, *) 'ANOMALY BODY :', IBOD
      IF(IBOD.EQ.MAXCAL) WRITE(JZ, *) 'TOTAL ANOMALY'
      IF(ITYPE.EQ.1) WRITE(JZ,350)
      IF(ITYPE.EQ.2) WRITE(JZ,351)
350   FORMAT(' POINT    X(km)     mGal')
351   FORMAT(' POINT    X(km)      nT')
      DO I = IST, NF
        IF(CALC(IBOD,I).LE.-1.E-20.OR.CALC(IBOD,I).GE.1.E-20) THEN
          WRITE(JZ, 400) I, COORD(I), CALC(IBOD,I)+OFFSET(ITYPE)
400       FORMAT(2X,I3,1X,F10.3,1X,F10.3)
        ENDIF
      ENDDO
      RETURN
      END




****************************************************************
C
C  PURPOSE : TO READ THE DISK FILE LUN AND LOOK
C            FOR FILE NAME INNAME
C  OPERATION IS DETERMINED BY IRET
C  INPUT : IRET = 0 SEARCH FOR AND FIND FILE INNAME
C                 1 READ EACH RECORD AND LIST FILE NAME
C                < 0 READ THE -IRET'TH LAST RECORD
C
C
C  OUTPUT : IRET = -1 O.K
C                   0 NOT FOUND
C                   1 PARITY ERROR

      SUBROUTINE READF(IRET,LUN)

      CHARACTER LIST(20)*10

      INCLUDE 'magrav.cmn'
 
      COMMON /LIST/ ILIST, LIST

      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      KOUNT = 1

      IF(IRET.EQ.1) GOTO 2000
      IF(IRET.EQ.0) GOTO 3000

C      READ THE -IRET'TH LAST RECORD

1200  REWIND  LUN
      NTIMES = -IRET
      DO I = 1, NTIMES
        IRET=LUN
        CALL BUFFIN(IRET)
        IF(IRET.NE.-1) GOTO 1210
      ENDDO
1210  CALL REINIT
      RETURN

C  IRET = 1 LIST  FILE NAMES

2000  DO I = 1, ILIST
        WRITE(JZ, *) LIST(I)
      ENDDO
2101  IRET = -1
      RETURN

C  READ APPROPRIATE FILE

3000  CALL SAV(100)

C     FIRST CHECK TO MAKE SURE THERE IS SUCH A FILE

      DO I = 1, ILIST
        IF(LIST(I).EQ.NAME) THEN
        IREC=I
        GOTO 3200
        ENDIF
      ENDDO
      IRET = 0

C     MODEL NAME NOT FOUND
      RETURN

3200  CALL MODREAD(MODEL,IREC)

C     CHECK OVER THE MODELS AND FIX UP
C     ANY OPEN BODIES
C      (I HOPE THIS IS NEVER NEEDED!)

      DO 3300 IBOD = 1, NBODS
        IF(NPTS(IBOD).LE.0) GOTO 3290
        NPT = NPTS(IBOD)

C   IF A BODY IS NOT CLOSED, CLOSE IT

        IF(X(IBOD,1).NE.X(IBOD,NPT).OR.Z(IBOD,1).NE.Z(IBOD,NPT)) THEN
          IF(NPT.EQ.MAXNPT) NPT = NPT - 1
          NPT = NPT + 1
          NPTS(IBOD) = NPT
          IANOM(IBOD) = 0
          X(IBOD,NPT) = X(IBOD,1)
          Z(IBOD,NPT) = Z(IBOD,1)
          WRITE(JZ, *) 'Body :', ibod, ' closed'
        ENDIF

C  IF ANY BODY HAS DUPLICATE CONSECUTIVE POINTS,
C   DELETE ONE OF THEM

3250    NPT1 = NPTS(IBOD) - 1
        DO KOUNT = 1, NPT1
          IF(X(IBOD,KOUNT).EQ.X(IBOD,KOUNT + 1)) THEN
            IF(Z(IBOD,KOUNT).EQ.Z(IBOD,KOUNT + 1)) THEN
              CALL DELETE(IBOD,KOUNT,IRET)
              IF(IRET.NE.0) THEN
                WRITE(JZ, *) 'Body : ', ibod, ' point : ', kount,
     +                           ' duplicate and deleted'
                GOTO 3250
              ENDIF
            ENDIF
          ENDIF
        ENDDO

        IF(NPTS(IBOD).LE.2) THEN
          NPTS(IBOD) = 0
          WRITE(JZ, *) 'Body ', ibod, ' deleted'
        ENDIF

C RESET ALL BODY CO-ORDS TO (0,0), IF BODY BEING DELETED

3290    IF(NPTS(IBOD).LE.0) THEN
          DO KOUNT = 1, NPT1
            X(IBOD,KOUNT) = 0.
            Z(IBOD,KOUNT) = 0.
          ENDDO
        ENDIF
3300  CONTINUE
      CALL REINIT
      IRET = -1
      RETURN
      END


************************************************************************
C  PURPOSE : 
C           TO SAVE THE CURRENT MODEL IN CASE OF SYSTEM CRASH
C           OR USER ERROR
C
C    MODEL IS BACKED UP AFTER EVERY NBACK REVISIONS
C
C   AFTER NREW RECORDS ARE OUTPUT, BACKUP FILE IS REWOUND
C      SO THAT FILE IS NOT TOO BIG

      SUBROUTINE SAV(KOUNT)

      INCLUDE 'magrav.cmn'
 
      COMMON /SAVEIT/ NSAVE

      DATA ITOT/0/, TOTT/300./, NREW/300/

      ITOT = ITOT + KOUNT
      IF(ITOT.LT.NBACK) RETURN
      ITOT = 0
      IREW = IREW + 1
      IF(IREW.GE.NREW) THEN
        REWIND ISCR
        IREW = 0
        NSAVE = 0
      ENDIF
      CALL WRITEF(IRET,ISCR)
      NSAVE = NSAVE + 1
      RETURN
      END



C*******************************************************************
C  PURPOSE TO PLOT THE OBSERVED DATA ON THE TEKTRONIX SCOPE AS A
C    DASHED LINE. EACH DASH BEGINS AND ENDS ON A DATA POINT

      SUBROUTINE PLOBS

      INCLUDE 'magrav.cmn'

      IF(ISCOPE.EQ.2) RETURN
      CALL SETSCL(1,0,0)

C  PLOT ALL POINTS

      NF = NFIELD(ITYPE)
      IF(NF.EQ.0) RETURN
      CALL GWICOL(.01,3)
      ISTART = LSTART(ITYPE) 
      IEND = LEND(ITYPE)
      DO I = ISTART, IEND, 2 
        XXX = COORD(I)
        ZZ = OBS(ITYPE,I)
        CALL POINTA(XXX,ZZ)
        IF(I.EQ.IEND) GOTO 10
        XXX = COORD(I+1)
        ZZ = OBS(ITYPE,I+1)
        CALL DRAWA(XXX,ZZ,2)
      ENDDO
10    CALL GWICOL(.01,1)
      RETURN
      END



***************************************************
C  PURPOSE TO CALCULATE AND/OR PLOT THE ANOMALY
C  JBOD BODY NUMBER

      SUBROUTINE PLANOM(JBOD,KALK)

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

C  IF KALK = 1, CALCULATE ANOMALY, ELSE PLOT

      IF(KALK.EQ.1) CALL CALCAN
      IF(ISCOPE.EQ.2) RETURN
      IBOD = JBOD
      IF(JBOD.EQ.0) IBOD = MAXCAL
      CALL SETSCL(1,0,0)

C  PLOT ANOMALY

      CALL GWICOL(.01,2)
      NF = NFIELD(ITYPE)
      ZDUM = OFFSET(ITYPE)
      IF(IDIFF.LE.0) THEN
        IBOD = MAXCAL
        ISTART = LSTART(ITYPE)
        ZP = CALC(IBOD,ISTART) - OBS(ITYPE,ISTART) + ZDUM
        XXX = COORD(ISTART)
        CALL POINTA(XXX,ZP)
        IST = ISTART + 1
        IEND = LEND(ITYPE)
        DO I = IST, IEND
          XXX = COORD(I)
          ZZZ = CALC(IBOD,I) - OBS(ITYPE,I) + ZDUM
          CALL DRAWA(XXX,ZZZ)
        ENDDO
        GOTO 150
      ENDIF
      IF(IBOD.NE.MAXCAL) THEN
        ZDUM = 0.
        DO I = 1, NF
        ZDUM = ZDUM + CALC(IBOD,I)
        ENDDO
        ZDUM = -ZDUM/NF
        ZDUM = ZDUM + (ANOMAX(ITYPE) - ANOMIN(ITYPE))/2 + ANOMIN(ITYPE)
      ENDIF
      ZP = CALC(IBOD,1) + ZDUM
      XXX = COORD(1)
      CALL POINTA(XXX,ZP)
      DO I = 2, NF
        XXX = COORD(I)
        ZZZ = CALC(IBOD,I) + ZDUM
        CALL DRAWA(XXX,ZZZ)
      ENDDO

C  TO PLOT A BASE LINE

      IF(ISUPG.LT.0) GOTO 200
      DUM = ANOMAX(ITYPE) * ANOMIN(ITYPE)
150   IF(IDIFF.LT.0) DUM = DIFMAX(ITYPE) * DIFMIN(ITYPE)
      CALL GWICOL(.01,1)
      IF(DUM.LE.0.) THEN
        X1 = COORD(1)
        X2 = COORD(NF)
        CALL POINTA(X1,0.)
        CALL DASHA(X2,0.,8)
      ENDIF
200   CONTINUE
      CALL GWICOL(.01,1)
      RETURN
      END


****************************************************
C  PURPOSE TO PLOT BODIES
C    BODY IBOD IS PLOTTED
C   IF IBOD = 0,ALL BODIES ARE PLOTTED

      SUBROUTINE PLBOD(IBOD)

      DIMENSION PX(25),PZ(25),IC(20)

      INCLUDE 'magrav.cmn'
 
      DATA IC/4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23/

      IF(ISCOPE.EQ.2) RETURN
      CALL SETSCL(2,0,0)
      IST = IBOD
      JEND = IBOD
      IF(IBOD.EQ.0) THEN
        IST = 1
        JEND = NBODS
      ENDIF
      DO I = IST, JEND
       DO J = 1, NPTS(I)
        PX(J)=X(I,J)
        PZ(J)=-Z(I,J)
        IF(ICROSS.EQ.1.OR.ICROSS.EQ.3) CALL DRCROS(PX(J),PZ(J))
       ENDDO
       IF(NPTS(I).GT.3) THEN
        IF(ICROSS.LE.1) CALL GVECT(PX,PZ,NPTS(I))
        IF(ICROSS.GE.2) CALL RSURF(PX,PZ,NPTS(I),IC(I),0.0)
       ENDIF
      ENDDO
      RETURN
      END

**********************************************************
C   DRAWS A CROSS AT POINT X,Z

      SUBROUTINE DRCROS(XXX,ZZ)

      INCLUDE 'magrav.cmn'
 
      XDIS2 = XDIS
      ZDIS2 = ZDIS
      CALL POINTA(XXX + XDIS2,ZZ)
      CALL DRAWA(XXX-XDIS2,ZZ)
      CALL POINTA(XXX,ZZ + ZDIS2)
      CALL DRAWA(XXX,ZZ-ZDIS2)
      CALL POINTA(XXX,ZZ)
      RETURN
      END

*******************************************************
C  PURPOSE TO SET UP WINDOWS AND SCALES FOR GRAPHING
C   SCALES MAY OPTIONALLY BE PUT ON
C
C  IWHICH = 1 UPPER GRAPH SET UP (ANOMALIES)
C           2 LOWER GRAPH(BODIES)
C
C  ISCL  = 0 DO NOT PLOT SCALES ON SCREEN
C          1 PLOT SCALES
C
C  X SCALE DEPENDS ON MODE
C        MODE = 1 SCALES FROM 0 TO XMAX -DRAW
C               2 SCALES FROM XMINS TO XMAXS-SKET

      SUBROUTINE SETSCL(IWHICH,ISCL,ISO)

      DIMENSION PX(5),PZ(5)

      INCLUDE 'magrav.cmn'
 
      COMMON /VPST/ WIDMM,HTMM,NXPIX,NYPIX,ITERM,CHEI,YOR

      IF(ISCOPE.EQ.2)RETURN
      IF(MODE.NE.2) THEN
        XXMIN = 0.
        XXMAX = XMAX
      ELSE
        XXMIN = SKXMIN
        XXMAX = SKXMAX
      ENDIF

      IF(IWHICH.EQ.2) GOTO 2000

C  UPPER GRAPH - PLOTS ANOMALIES

      IF(IDIFF.GE.0) THEN
        ZZMIN = ANOMIN(ITYPE)
        ZZMAX = ANOMAX(ITYPE)
      ELSE
        ZZMIN = DIFMIN(ITYPE)
        ZZMAX = DIFMAX(ITYPE)
      ENDIF
C
      IF(ISO.NE.0) THEN
        ZZMIN = PMIN
        ZZMAX = PMAX
      ENDIF

      I1=0
      I2=NXPIX
      I3=0.7*NYPIX
      I4=NYPIX
      CALL TWINDO(I1,I2,I3,I4)
      GOTO 5000

C  LOWER GRAPH - BODIES PLOTTED HERE
C  DRAW MODE

2000  IF(MODE.NE.2) THEN
        ZZMIN = -ZMAX
        ZZMAX = 0.
      ELSE
        ZZMIN = -SKZMAX
        ZZMAX = -SKZMIN
      ENDIF

C  BODIES ARE PLOTTED ON NEGATIVE SCALE WITH ZERO AT TOP

2700  I1=0
      I2=NXPIX
      I3=0
      I4=0.7*NYPIX
      CALL TWINDO(I1,I2,I3,I4)
      LENX = I2-I1
      LENZ = I4-I3
      XDIS = ((XXMAX-XXMIN)/FLOAT(LENX))*4.
      ZDIS = ((ZZMAX-ZZMIN)/FLOAT(LENZ))*4.

5000  CALL DWINDO(XXMIN,XXMAX,ZZMIN,ZZMAX)
      IF(ISCL.EQ.0) RETURN

C  NOW PLOT SCALES

      CALL GWICOL(.01,1)
      CALL POINTA(XXMIN,ZZMAX)
      CALL DRAWA(XXMAX,ZZMAX)
      CALL POINTA(XXMAX,ZZMIN)
      CALL DRAWA(XXMIN,ZZMIN)

      IF(ITERM.EQ.2) RETURN
      IF(IWHICH.EQ.2) GOTO 5200

C  ANOMALY SCALE PRINTED

      Z1=0.
      Z2=NXPIX
      Z3=0.7*NYPIX
      Z4=NYPIX
      CALL DWINDO(Z1,Z2,Z3,Z4)
      HT=CHEI+1.
      NDEC=1
      TEST1=(ZZMIN-INT(ZZMIN))*10.0
      TEST2=(ZZMAX-INT(ZZMAX))*10.0
      ITEST1=INT(TEST1)
      ITEST2=INT(TEST2)
      IF(ITEST1.EQ.0.AND.ITEST2.EQ.0) NDEC=0
      CALL RTXJUS(0,4)
      ZPOS=Z4
      XPOS=Z1
      CALL RTXN(ZZMAX,NDEC,XPOS,ZPOS)
      CALL RTXJUS(0,0)
      ZPOS=Z3
      CALL RTXN(ZZMIN,NDEC,XPOS,ZPOS)
      CALL DWINDO(XXMIN,XXMAX,ZZMIN,ZZMAX)
      CALL RTXJUS(0,1)
      RETURN

C  MARK X AND Z SCALES ON GRAPH OF BODIES

5200  Z1=0.
      Z2=NXPIX
      Z3=0.
      Z4=0.7*NYPIX
      CALL DWINDO(Z1,Z2,Z3,Z4)
      IF(ITERM.EQ.1) THEN
      PX(1) = Z1
      PX(2) = Z1
      PX(3) = Z2
      PX(4) = Z2
      PZ(1) = Z3
      PZ(2) = Z4
      PZ(3) = Z4
      PZ(4) = Z3
      CALL RSURF(PX,PZ,4,30,0.0)
      ENDIF
C      HT=3.0
      ROT=0.0
      JUST=1
      NDEC=2
      TEST1=ABS((ZZMIN-INT(ZZMIN))*100.0)
      TEST2=ABS((ZZMAX-INT(ZZMAX))*100.0)
      ITEST1=INT(TEST1)
      ITEST2=INT(TEST2)
      IF(ITEST1.EQ.0.AND.ITEST2.EQ.0) NDEC=0
      CALL RTXJUS(0,4)
      ZPOS=Z4
      XPOS=Z1
      CALL RTXN(-ZZMAX,NDEC,XPOS,ZPOS)
      CALL RTXJUS(0,0)
      ZPOS=Z3
      CALL RTXN(-ZZMIN,NDEC,XPOS,ZPOS)
      NDEC=2
      TEST1=(XXMIN-INT(XXMIN))*100.0
      TEST2=(XXMAX-INT(XXMAX))*100.0
      ITEST1=INT(TEST1)
      ITEST2=INT(TEST2)
      IF(ITEST1.EQ.0.AND.ITEST2.EQ.0) NDEC=0
      CALL RTXJUS(0,4)
      ZPOS=Z4
      CALL RTXN(XXMIN,NDEC,XPOS,ZPOS)
      CALL RTXJUS(2,4)
      XPOS=Z2
      CALL RTXN(XXMAX,NDEC,XPOS,ZPOS)
      CALL RTXJUS(0,1)
      CALL DWINDO(XXMIN,XXMAX,ZZMIN,ZZMAX)

C DRAW GRID LINES

      IF(MODE.NE.2) THEN
        IF(ISUPG.GE.0) THEN
          NGX = XMAX/GX + 1
          NGZ = ZMAX/GZ
          XS = 0.
          DO I = 1, NGX
            CALL POINTA(XS,ZZMAX)
            CALL DASHA(XS,ZZMIN,4)
            XS = XS + GX
          ENDDO
          ZS = -GZ
          DO I = 1, NGZ
            CALL POINTA(XXMIN,ZS)
            CALL DASHA(XXMAX,ZS,4)
            ZS = ZS - GZ
          ENDDO
          RETURN
        ENDIF
      ENDIF

C      DRAW A LITTLE SQUARE THE SIZE OF TOLERANCE
C         ALLOWED FOR CURSOR INPUT

      CALL POINTA(XXMAX,ZZMIN)
      XXX = XXMAX - 2.*XDIS
      CALL DRAWA(XXX,ZZMIN)
      ZZ = ZZMIN + 2.*ZDIS
      CALL DRAWA(XXX,ZZ)
      CALL DRAWA(XXMAX,ZZ)
      CALL DRAWA(XXMAX,ZZMIN)
      RETURN
      END

C**************************************************************
C  PURPOSE TO CALCULATE THE GRAVITY OR MAGNETIC ANOMALY
C   AND PUT THE NEW ONE IN CALC(MAXCAL,1-NFIELD)

      SUBROUTINE CALCAN

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      DO I = 1, MAXOBS
        CALC(MAXCAL,I) = 0.D0
      ENDDO

C  CALCULATE ANOMALY FOR ANY BODY THAT HAS BEEN CHANGED
      NF = NFIELD(ITYPE)
      DO IBOD = 1, NBODS
        IF(NPTS(IBOD).GT.0) THEN
          IF(IANOM(IBOD).GE.0) THEN
            CALL GMAG(IBOD)
            IANOM(IBOD) = -1
          ENDIF
          DO I = 1, NF
            CALC(MAXCAL,I) = CALC(MAXCAL,I) + CALC(IBOD,I)
          ENDDO
        ENDIF
      ENDDO
      RETURN
      END



C*************************************************************
C  PURPOSE - THIS ROUTINE HANDLES ALL DISPLAYS AND INPUTS
C     IN GRAPHICS MODE.THE COMMANDS TYPED AT THE
C     BOTTOM OF THE SCREEN CAN BE ACTIVATED BY PLACING THE
C     CURSOR OVER THE COMMMAND AND HITTING ANY CHARACTER
C      (NOTE-SOME COMMANDS INTERPRET THE CHARACTER AS A
C      BODY NUMBER(0 = ALL))
C
C     A SINGLE BELL MEANS THE PROGRAM IS EXPECTING A
C   COMMAND.
C      A LONG BELL MEANS AN ERROR HAS OCCURED AND A NEW
C      COMMAND SHOULD BE CURSORD IN
C     CAN RE-ENTER OPTION MODE BY TYPING ?
C     CAN RE-ENTER COMMAND MODE BY TYPING C

      SUBROUTINE GRAP

      CHARACTER*1 CHR,BELL
      CHARACTER*11 LINE(26)
      DIMENSION XB(24),ZB(24)

      INCLUDE 'magrav.cmn'
 
      COMMON /VPST/ WIDMM,HTMM,NXPIX,NYPIX,ITERM,CHEI,YOR
      COMMON /SAVEIT/ NSAVE
      INTEGER GIPOLYX

      DATA LINE/'A-BODY_CON ','B-ENT_BODY ','C-CANCEL   ','D-DEL_PT   ',
     +'E-EXIT     ','F-FILL_BOD ','G-GRAVITY ','H-HELP  ','I-INSERT   ',
     +'J-PARAM    ','K-CROSS    ','L-REGC     ','M-MAGN     ',
     +'N-POINT_M  ','O-OFFSET   ','P-POINT_XZ ','Q-RECOVER  ',
     +'R-REDRAW   ','S-SKETCH   ','T-DRAW     ','U-DIFF     ',
     +'V-AM+BSHP  ','W-ISOSTASY ','X-BODNUM','Y-CAL_ANOM ','Z-ZOOM  '/

      IF(ISCOPE.EQ.2) RETURN
      BELL=CHAR(7)
      GOTO 1200

C  ERROR ON LAST GRAPHIC INPUT-RETRY COMMAND

25    CONTINUE
      WRITE(JZ,*) BELL

C  AWAIT NEXT COMMAND
100   CONTINUE
200   CALL SETSCL(2,0,0)
      WRITE(JZ,*)LINE
      CALL VCURSR(CHR,XXX,ZZ)
      IF(CHR.LT.'A'.OR.CHR.GT.'Z') GOTO 25
C
C  CALCULATE WHICH COMMAND HAS BEEN ISSUED
C
      ICHR=ICHAR(CHR)-64
C  NOTE*******************************************************************
C  The following goto statement controls branching for the GRAPHICS window
      GOTO(2500,1400,25,1500,9900,2800,2400,3400,1700,2200,1300,3200,
     1      2300,3000,2600,1800,2100,1200,2000,1900,2700,3100,
     2       2900,3300,1600,1100),ICHR
C
C ---------------------------------------------------------
C  ZOOM
C
C  SETS DRAW PARAMETERS : 
C      USER POINTS TO LOWER LEFT AND
C      UPPER RIGHT CORNERS TO RESET DRAW

1100  CALL SETSCL(2,0,0)

C   BOTTOM LEFT
      CALL VCURSR(CHR,XXB,ZZB)
      ZZB = -ZZB
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
C   TOP RIGHT
      CALL VCURSR(CHR,XXT,ZZT)
C      IF(CHR.EQ.' ') GOTO 25
      ZZT = -ZZT
      IF(CHR.EQ.'Q') GOTO 9900
      IF(CHR.EQ.'X') GOTO 25
      IF(ZZT.LT.0.) GOTO 25
      IF(XXB.GE.XXT) GOTO 25
      IF(ZZB.LE.ZZT) GOTO 25
      SKXMAX = XXT
      SKXMIN = XXB
      SKZMIN = ZZT
      SKZMAX = ZZB
      MODE=2

C  REDRAW

1200  CALL GCLEAR
      N3 = 2
      DO I = 1, N3
        CALL SETSCL(I,1,0)
      ENDDO
      IBOD = 0
      CALL PLBOD(IBOD)
      KALK = 1
      CALL PLANOM(IBOD,KALK)
      IF(IDIFF.GT.0) CALL PLOBS
      IF(ITERM.EQ.2) RETURN
      GOTO 100

C --------------------------------------------
C  CROSS
C
C     CROSSES CAN BE TOGGLED BY THIS COMMAND

1300  IF(ICROSS.EQ.1.OR.ICROSS.EQ.3) THEN
       ICROSS = ICROSS - 1
       GOTO 1200
      ELSE
       ICROSS = ICROSS + 1
       CALL SETSCL(2,0,0)
       IST = IBOD
       IEND = IBOD
       IF(IBOD.EQ.0) THEN
        IST = 1
        IEND = NBODS
       ENDIF
       DO I=IST,IEND
        DO J=1,NPTS(I)
         CALL DRCROS(X(I,J),-Z(I,J))
        ENDDO
       ENDDO
       GOTO 100
      ENDIF

C  BODY   

1400  LASTBD = -1

C  THE USER MOVES THE CURSOR AROUND, HITS THE BODY NUMBER
C  SUCCESSIVELY UNTIL HE WANTS TO FINISH DRAWING THE BODY.
C  HE CAN DO THIS BY HITTING '.' 

      CALL SETSCL(2,0,0)
1420  CALL VCURSR(CHR,XXX,ZZ)
      ZZ = -ZZ
      IF(CHR.EQ.' ') GOTO 1440
      IF(CHR.EQ.'Q') GOTO 1440
      IF(CHR.EQ.'X') GOTO 1440
      IF(CHR.NE.'.') GOTO 1450

C  IDOT IS THE LAST POINT IN THE BODY.
C  JOIN END TO BEGINNING
C  AND RETURN TO COMMAND MODE
C
C  IF LAST CHARACTER WAS A COMMAND, THERE IS AN ERROR
C   THAT IS IF THERE ARE NO POINTS IN THE BODY

      IF(LASTBD.EQ.-1) GOTO 25
      XXX = X(LASTBD,1)
      ZZ = Z(LASTBD,1)
      NPT = NPTS(LASTBD) + 1
      IF(NPT.LT.MAXNPT) THEN
        X(LASTBD,NPT) = XXX
        Z(LASTBD,NPT) = ZZ
        REVZ = -ZZ
        CALL DRAWA(XXX,REVZ)
        NPTS(LASTBD) = NPT
        CALL SAV(NBACK)
        IANOM(LASTBD) = 0
        GOTO 100
      ENDIF

C    BODY BUILDING ABORTED

1440  IF(NPT.LT.MAXNPT) NPT = NPT + 1
      X(LASTBD,NPT) = X(LASTBD,1)
      Z(LASTBD,NPT) = Z(LASTBD,1)
      IANOM(IBOD) = 0
      REVZ = -ZZ
      CALL DRAWA(XXX,REVZ)
      GOTO 25

C  NEW BODY POINT TO BE ADDED AND DRAWN

1450  IBOD = NUMB(CHR,MAXBOD)
      IF(IBOD.LT.1.OR.IBOD.GT.MAXBOD) GOTO 1440
      IF(IBOD.GT.NBODS) NBODS = IBOD

C  CALL ROUTINE TO CHECK WHETHER THIS POINT IS SUPPOSED
C  TO HAVE THE SAME CO-ORDS AS A PREVIOUS ONE

      NUMBER = 1
      KBR = 0
      CALL CHECK(NUMBER,XXX,ZZ,IDUPBD,IDUPPT,IRET,KBR)

C  IF POINT OUT OF RANGE, THEN ERROR

      IF(IRET.GT.0) GOTO 1440
      NPT = NPTS(IBOD)
      IF(NPT.GE.MAXNPT) GOTO 1440
      IF(NPT.EQ.0) GOTO 1470

C      IF ADDING POINTS TO PREVIOUS BODY
C      SKIP AROUND LAST POINT

      IF(LASTBD.EQ.-1) NPT = NPT - 1

C  DRAW THE NEW LINE OR POINT

      REVZ = -Z(IBOD,NPT)
      IF(ICROSS.EQ.1.OR.ICROSS.EQ.3) CALL DRCROS(X(IBOD,NPT),REVZ)
      IF(ICROSS.EQ.0.OR.ICROSS.EQ.2) CALL POINTA(X(IBOD,NPT),REVZ)
      REVZ = -ZZ
      CALL DRAWA(XXX,REVZ)
      IF(ICROSS.EQ.1.OR.ICROSS.EQ.3) CALL DRCROS(XXX,REVZ)
      GOTO 1475
1470  REVZ = -ZZ
      IF(ICROSS.EQ.1.OR.ICROSS.EQ.3) CALL DRCROS(XXX,REVZ)
      IF(ICROSS.EQ.0.OR.ICROSS.EQ.2) CALL POINTA(XXX,REVZ)
1475  LASTBD = IBOD
      NPT = NPT + 1
      X(IBOD,NPT) = XXX
      Z(IBOD,NPT) = ZZ
      NPTS(IBOD) = NPT
      IANOM(IBOD) = 0
      CALL SAV(NBACK)
      GOTO 1420

C  DELETE

1500  CALL SETSCL(2,0,0)

C  INPUT POINT TO BE DELETED

      CALL VCURSR(CHR,XOLD,ZOLD)
      ZOLD = -ZOLD
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      KOUNT = -1
      NUMBER = 1
1510  KOUNT = KOUNT + 1
      KBR = 0
      CALL CHECK(NUMBER,XOLD,ZOLD,IBOD,NPT,IRET,KBR)
      IF(IRET) 1520, 1530, 25

C  UNASSIGNED POINT

1520  IF(KOUNT.GT.0) GOTO 100
      GOTO 25

C  POINT BELONGS TO BODY IBOD, POINT NPT

1530  CALL DELETE(IBOD,NPT,IRET)
      IF(IRET.EQ.0) GOTO 25
C     FOR FIRST POINT IN BODY, REJOIN WITH LAST

      IF(NPT.EQ.1) THEN
        NPP = NPTS(IBOD)
        X(IBOD,NPP) = X(IBOD,1)
        Z(IBOD,NPP) = Z(IBOD,1)
      ENDIF
1540  IANOM(IBOD) = 0
      CALL SAV(NBACK)
      CALL GCLEAR
      N3 = 2
      DO I = 1, N3
        CALL SETSCL(I,1,0)
      ENDDO
      IBOD = 0
      CALL PLBOD(IBOD)
      KALK = 1
      CALL PLANOM(IBOD,KALK)
      IF(IDIFF.GT.0) CALL PLOBS
      GOTO 1510

C ------------------------------------------------------
C  ANOMALY

1600  CALL VCURSR(CHR,XXX,ZZ)
      IF(CHR.LT.'0'.OR.CHR.GT.'2') GOTO 25
      IMOD10 = (ICHAR(CHR) - 48)*10
      CALL VCURSR(CHR,XDUM,ZDUM)
      IF(CHR.LT.'0'.OR.CHR.GT.'9') GOTO 25
      IMOD1 = ICHAR(CHR) - 48
      IBOD = IMOD10 + IMOD1
      IF(IBOD.LT.0.OR.IBOD.GT.NBODS) GOTO 25 
      KALK = 1

C  CALCULATE ANOMALY AND PLOT

      CALL PLANOM(IBOD,KALK)
      GOTO 100

C  INSERT

1700  CALL SETSCL(2,0,0)

C  CURSOR IN THE 2 POINTS ON EITHER SIDE OF NEW ONE

      CALL VCURSR(CHR,XONE,ZONE)
      ZONE = -ZONE
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      CALL VCURSR(CHR,XNEW,ZNEW)
      ZNEW = -ZNEW
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      CALL VCURSR(CHR,XTWO,ZTWO)
      ZTWO = -ZTWO
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      NUMBER = 1
      KBR = 0
      CALL CHECK(NUMBER,XNEW,ZNEW,IDUPBD,IDUPPT,IRET,KBR)
      IF(IRET.EQ.1) GOTO 25
      NCHANG = 0
      KOUNT = 0
1720  NUMBER = 0
1723  NUMBER = NUMBER + 1

C  CHECK THE FIRST OCCURENCE OF POINT ONE

      CALL CHECK(NUMBER,XONE,ZONE,IBOD,NPT,IRET,KBR)
      IF(IRET) 1790, 1730, 25
1730  NUM2 = 0
1732  NUM2 = NUM2 + 1
1735  CALL CHECK(NUM2,XTWO,ZTWO,IBOD2,NPT2,IRET,KBR)
      IF(IRET) 1723, 1740, 25
1740  IF(NPT2.NE.(NPT + 1)) GOTO 1732
      IF(IBOD.NE.IBOD2) GOTO 1732

C  THE POINT FOUND IS THE ONE AFTER THE FIRST

      CALL INSERT(XNEW,ZNEW,IBOD,NPT,IRET)
      CALL PLBOD(IBOD)
      IANOM(IBOD) = 0
      NCHANG = NCHANG + 1

C  GO THRU ALL LINES IN BOTH DIRECTIONS

1790  IF(KOUNT.EQ.0) THEN
        KOUNT = 1
        SAVER = XONE
        XONE = XTWO
        XTWO = SAVER
        SAVER = ZONE
        ZONE = ZTWO
        ZTWO = SAVER
        GOTO 1720
      ENDIF

C  IF NO BODIES WERE INSERTED INTO, ERROR

1795  CALL SAV(NBACK)
      IF(NCHANG.EQ.0) GOTO 25
      GOTO 100

C ---------------------------------------------------
C  POINT

1800  CALL SETSCL(2,0,0)
      CALL SAV(NBACK)

C  ACCEPT CURSOR OF POINT TO BE MOVED

      CALL VCURSR(CHR,XOLD,ZOLD)
      ZOLD = -ZOLD
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      NBR = 0
      IF(CHR.EQ.'M')NBR = 3
      IF(CHR.EQ.'X')NBR = 1
      IF(CHR.EQ.'Z')NBR = 2
      IF(NBR.NE.0) GOTO 1850

C  ACCEPT NEW POINT

      CALL VCURSR(CHR,XNEW,ZNEW)
      ZNEW = -ZNEW
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      IF(CHR.EQ.'M')NBR = 3
      IF(CHR.EQ.'X')NBR = 1
      IF(CHR.EQ.'Z')NBR = 2
      IF(NBR.NE.0) GOTO 1860
      NUMBER = 1
      NCHANG = 0
      KBR = 0
      CALL CHECK(NUMBER,XNEW,ZNEW,IDUPBD,IDUPPT,IRET,KBR)
      IF(IRET.EQ.1) GOTO 25
      NUMBER = 1
1827  CALL CHECK(NUMBER,XOLD,ZOLD,IBOD,NPT,IRET,KBR)
      IF(IRET) 1840, 1830, 25
1830  IF(IDUPBD.EQ.IBOD.AND.IDUPPT.EQ.NPT) GOTO 25
      IANOM(IBOD) = 0
      NCHANG = NCHANG + 1
      X(IBOD,NPT) = XNEW
      Z(IBOD,NPT) = ZNEW
      IF(NPT.EQ.1) THEN
       X(IBOD,NPTS(IBOD)) = XNEW
       Z(IBOD,NPTS(IBOD)) = ZNEW
      ENDIF
      CALL PLBOD(IBOD)
      GOTO 1827

C FIRST DATA POINT NOT THERE

1840  IF(NCHANG.EQ.0) GOTO 25
      DO 1845 I = 1, NBODS
      NPS = NPTS(I) - 1
      IF(NPS.LE.2) GOTO 1845
      DO 1844 J = 1, NPS
      IF(X(I,J).NE.X(I,J+1)) GOTO 1844
      IF(Z(I,J).NE.Z(I,J+1)) GOTO 1844
      CALL DELETE(I,J,IRET)
      GOTO 1845
1844  CONTINUE
1845  CONTINUE
      CALL SAV(NBACK)
      GOTO 1200

1850  XNEW = 0.
      ZNEW = 0.
      NPC = 2
      CALL AMOVE(XOLD,ZOLD,XNEW,ZNEW,NPC)
      CALL SAV(NBACK)
      GOTO 1200

1860  NPC = 4
      CALL AMOVE(XOLD,ZOLD,XNEW,ZNEW,NPC)
      CALL SAV(NBACK)
      GOTO 1200

C -------------------------------------------
C  DRAW

1900  MODE = 1
      GOTO 1200

C -------------------------------------------
C  SKETCH

2000  MODE = 2
      GOTO 1200

C -------------------------------------------
C  RECOVER

2100  NSAVE = NSAVE - 1

C   NSAVE COUNTS THE NUMBER OF MODELS SAVED.
C   IF THE RECOVER COMMAND IS REPEATEDLY EXECUTED
C    EACH EXECUTION WILL RECOVER A MORE PREVIOUS MODEL

      IRET = (NSAVE + 1) * -1
      IF(IRET.GE.0) GOTO 25
      CALL READF(IRET,ISCR)
      IF(IRET.GE.0) GOTO 25
      GOTO 1200

C -------------------------------------------------------------
C  PARAM

2200  CALL GCLEAR
      CALL PARAM(IBOD)
      CALL SAV(NBACK)
      GOTO 1200

C ---------------------------------------------------
C     MAGNETIC MODE

2300  IF(ITYPE.NE.2) CALL REINIT
      ITYPE = 2
      IWFP = IWFLAG
      IWFLAG = 1
      GOTO 1200

C ---------------------------------------------------
C     GRAVITY MODE

2400  IF(ITYPE.NE.1) CALL REINIT
      IF(ITYPE.NE.1) IWFLAG = IWFP
      ITYPE = 1
      GOTO 1200

C --------------------------------------------------
C     ADJUST BODY CONTRASTS

2500  CONTINUE
      IMOD=0
      DO 2520 K=1,NBODS
      NBP=NPTS(K)-1
      IF(NBP.LE.0) GOTO 2520
      DO 2510 J=1,NBP
      XB(J)=X(K,J)
2510  ZB(J)=Z(K,J)
      M=GIPOLYX(NBP,XB,ZB,XXX,-ZZ)
      IF(M.NE.0) IMOD=K
2520  CONTINUE
      CALL AMPL(IMOD)
      IBOD = 0
      KALK = 1
      CALL PLANOM(IBOD,KALK)
      GOTO 100

C --------------------------------------------------
C    ADJUST OBSERVED DATA OFFSET

2600  WRITE(JZ,2601)
2601  FORMAT(//'ENTER ONE OF THE FOLLOWING'/,
     *5X,'O - TO DETERMINE OFFSET'/,
     *5X,'Y - TO TOGGLE ON AUTOMATIC DETERMINATION OF OFFSET'/,
     *5X,'N - TO TOGGLE OFF AUTOMATIC DETERMINATION OF OFFSET'/,
     *5X,'P - AN ARBITRARY VALUE IS ADDED TO OFFSET'/,
     *5X,'M - AN ARBITRARY VALUE IS SUBTRACTED FROM OFFSET'/,
     */'WITH P OR M ALSO ENTER A DIGIT FROM 0 to 9'/,
     *'SEE HELP FOR FURTHER EXPLANATION'//)
      CALL OFFST
      GOTO 100

C CHANGE ANOMALY PLOT

2700  IDIFF = -IDIFF
      GOTO 1200

C TOGGLE BODY FILLING

2800  IF(ICROSS.GT.1) THEN
       ICROSS = ICROSS - 2
       GOTO 1200
      ELSE
       ICROSS = ICROSS + 2
       CALL PLBOD(IBOD)
      ENDIF
      GOTO 100

C --------------------------------------------------------------
C   FIT OF CURVES CONSTRAINED BY DIFFERENTIAL COLUMN PRESSURES
C
2900  IF(ITYPE.GE.2.OR.NBODS.LE.0) GOTO 100
      CALL ISOSTA(1,IERS)
      CALL SETSCL(1,0,1)
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      XDUM=COORD(ISTART)
      CALL POINTA(XDUM,WD(ISTART))
      IS1=ISTART+1
      DO 120 I=IS1,IEND
      XDUM=COORD(I)
      CALL DASHA(XDUM,WD(I),3)
120   CONTINUE
      GOTO 100

C --------------------------------------------------------------
C   INCREMENTAL MANUAL FIT

3000  CALL SETSCL(2,0,0)
      IGO = 3
      STX = 2.*XDIS
      STZ = 2.*ZDIS
      SR2 = 2.**0.5
      CALL RMSR(RMS,IGO)
C
C     ACCEPT OLD POINT
C
      CALL VCURSR(CHR,XOLD,ZOLD)
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      ZOLD = -ZOLD
      NUMBER = 1
      KBR = 0
      CALL CHECK(NUMBER,XOLD,ZOLD,IDUPBD,IDUPPT,IRET,KBR)
      IF(IRET.NE.0) GOTO 25
      GOTO 3026
C
C     ACCEPT DIRECTION OF MOVEMENT
C
3025  CALL SETSCL(2,0,0)
      PRMS = RMS
      CALL VCURSR(CHR,XDUM,ZDUM)
      IF(CHR.EQ.'E') GOTO 9900
      IF(CHR.EQ.'C') GOTO 25
      IF(CHR.EQ.'R') GOTO 1200
3026  IF(CHR.LT.'1'.OR.CHR.GT.'9') GOTO 25
      ICHR = ICHAR(CHR)-48
      GOTO(3001,3002,3003,3004,25,3006,3007,3008,3009),ICHR
3001  XNEW = XOLD -STX/SR2
      ZNEW = ZOLD +STZ/SR2
      GOTO 3015
3002  XNEW = XOLD
      ZNEW = ZOLD + STZ
      GOTO 3015
3003  XNEW = XOLD + STX/SR2
      ZNEW = ZOLD + STZ/SR2
      GOTO 3015
3004  XNEW = XOLD - STX
      ZNEW = ZOLD
      GOTO 3015
3006  XNEW = XOLD + STX
      ZNEW = ZOLD
      GOTO 3015
3007  XNEW = XOLD - STX/SR2
      ZNEW = ZOLD - STZ/SR2
      GOTO 3015
3008  XNEW = XOLD
      ZNEW = ZOLD - STZ
      GOTO 3015
3009  XNEW = XOLD + STX/SR2
      ZNEW = ZOLD - STZ/SR2
3015  IF(ZNEW.LT.0.) ZNEW = 0.
      IF(ZNEW.GT.ZMAX) ZNEW = ZMAX
      IF(XNEW.LT.0.) XNEW = 0.
      IF(XNEW.GT.XMAX) XNEW = XMAX
      NCHANG = 0
      CALL CHECK(NUMBER,XNEW,ZNEW,IDUPBD,IDUPPT,IRET,KBR)
3027  CALL CHECK(NUMBER,XOLD,ZOLD,IBOD,NPT,IRET,KBR)
      IF(IRET)3040,3030,25
3030  IF(IDUPBD.EQ.IBOD.AND.IDUPPT.EQ.NPT) GOTO 25
      IANOM(IBOD) = 0
      NCHANG = NCHANG + 1
      X(IBOD,NPT) = XNEW
      Z(IBOD,NPT) = ZNEW
      GOTO 3027
3040  IF(NCHANG.EQ.0) GOTO 25
      DO 3045 I= 1, NBODS
      NPS = NPTS(I) - 1
      IF(NPS.LE.2) GOTO 3045
      DO 3044 J= 1, NPS
      IF(X(I,J).NE.X(I,J+1)) GOTO 3044
      IF(Z(I,J).NE.Z(I,J+1)) GOTO 3044
      CALL DELETE(I,J,IRET)
      GOTO 3045
3044  CONTINUE
3045  CONTINUE
      ZPLT = -ZNEW
      CALL DRCROS(XNEW,ZPLT)
      IBOD = 0
      KALK = 1
      CALL PLANOM(IBOD,KALK)
      CALL RMSR(RMS,IGO)
      XOLD = XNEW
      ZOLD = ZNEW
      CALL SAV(NBACK)
      IF(RMS.LT.PRMS) GOTO 3025
      WRITE(JZ,*) BELL
      GOTO 3025
 
C ----------------------------------------------------------------
C   TOGGLE TO PERMIT CHANGING BODY PARAMETER WHILE CHANGING BODY SHAPE 
C
3100  WRITE(JZ,3101)
3101  FORMAT(//'ENTER Y TO PERMIT OPTIMIZATION TO MODIFY BODY DENSITY',
     */' OR MAGNETIZATION AT THE SAME TIME AS MODIFYING BODY SHAPE')
      WRITE(JZ,3102)
3102  FORMAT(/'ENTER N TO CONFINE OPTIMIZATION TO MODIFYING BODY',
     */' WHILE HOLDING BODY DENSITY OR MAGNETIZATION CONSTANT')
      WRITE(JZ,3103)
3103  FORMAT(/'ENTER C TO CANCEL'//)
      CALL VCURSR(CHR,XDUM,ZDUM)
      IF(CHR.EQ.'C') GOTO 25
      IF(CHR.EQ.'Y') IVA = 1
      IF(CHR.EQ.'N') IVA = -1
      GOTO 100

C ----------------------------------------------------------------
C   TO APPLY A LINEAR CORRECTION TO THE DIFFERENCES
C
3200  WRITE(JZ,3201)
3201  FORMAT(/'WARNING - EXECUTION OF THIS OPTION ALTERS THE OBSERVED',
     */' DATA WHICH BECOMES PERMANENT WHEN THE WRIT OPTION IS EXECUTED'
     */' AND MAGRAV IS EXITED. TO RECOVER THE OBSERVED DATA USE THE',
     */' Q OPTION BEFORE EXITING MAGRAV.',
     */' THE Q OPTION BACKS UP ONE MODEL WITH EACH HIT OF THE Q KEY.'/)
      RCOR = 0.
      CALL REGC(RCOR)
      CALL PLOBS
      GOTO 100
C
C ----------------------------------------------------------------
C   LIST BODY NUMBER
C
3300  DO 3320 K=1,NBODS
      NBP=NPTS(K)-1
      IF(NBP.LE.0) GOTO 3320
      DO 3310 J=1,NBP
      XB(J)=X(K,J)
3310  ZB(J)=Z(K,J)
      M=GIPOLYX(NBP,XB,ZB,XXX,-ZZ)
      IF(M.EQ.0) GOTO 3320
      KMOD=K
      WRITE(LZ,3315) KMOD
3315  FORMAT(' BODY NUMBER:',I3)
3320  CONTINUE
      GOTO 100
C
C ----------------------------------------------------------------
C   HELP FILE 
C
3400  WRITE(JZ,*)LINE
      WRITE(JZ,*)
      WRITE(JZ,*)'ENTER A GRAPHICS COMMAND LETTER IN TEXT WINDOW,'
      WRITE(JZ,*)'   OR E TO EXIT HELP AND RETURN TO'
      WRITE(JZ,*)'   THE GRAPHICS WINDOW'
      READ(*,'(A1)') CHR
      IF(CHR.EQ.'E'. OR.CHR.EQ.'e') GOTO 100
      IF(CHR.GE.'A'.AND.CHR.LE.'Z') THEN
      ICHR=ICHAR(CHR)-64
      ELSE
      IF(CHR.GE.'a'.AND.CHR.LE.'z') THEN
      ICHR=ICHAR(CHR)-96
      ELSE
      GOTO 3400
      ENDIF
      ENDIF
      CALL GHELP(ICHR)
      WRITE(*,*)
      WRITE(JZ,*)' HIT ENTER TO CONTINUE'
      READ(*,*)
      WRITE(*,*)
      GOTO 3400
C
C ----------------------------------------------------------------
C
C END - RETURN TO OPTIONS

9900  CALL GCLEAR
      RETURN
      END



****************************************************************
C     CALCULATE BODY NUMBER FOR CHARACTER BODY NUMBER ICHAR
C     IF BODY IS INVALID NUMB = 0

      FUNCTION NUMB(CHR,MAXBOD)

      CHARACTER*1 CHR

      DATA IZERO/48/

      ICHR=ICHAR(CHR)
      NUMB = ICHR - IZERO
      IF(NUMB.GT.0.AND.NUMB.LE.MAXBOD) RETURN
      NUMB = 0
      RETURN
      END



********************************************************
C  CHECKS POINTS INPUT TO BODY
C   IF THIS POINT IS IN APPROX THE
C  SAME PLACE AS ANOTHER IT IS MADE EQUAL TO IT
C  OUT OF RANGE POINTS MAY ALSO BE CHECKED FOR
C
C  THE NUMBER TH DUPLICATE POINT IS CHECKED FOR
C
C
C  XXX-X VALUE TO BE CHECKED
C  ZZ-Z VALUE TO BE CHECKED
C  C  RETURN POINT OUT OF RANGE
C  IRET = 1
C  IDUPBD = 0
C  IDUPPT = 0
C
C  POINT DUPLICATED
C  IRET = 0
C  IDUPBD = BODYNUMBEROF IDENTICAL POINT
C  IDUPPT = POINT NO. OF IDENTICAL POINT
C
C  POINT UNIQUE
C  IRET = -1
C  IDUPBD = 0
C  IDUPPT = 0


      SUBROUTINE CHECK(NUMBER,XXX,ZZ,IDUPBD,IDUPPT,IRET,KBR)

      INCLUDE 'magrav.cmn'
 
      NUMDUP = 0
      DO IDUPBD = 1, NBODS
        NPT = NPTS(IDUPBD)
        IF(KBR.EQ.1)NPT=NPTS(IDUPBD)-1
        IF(NPT.GT.0) THEN
          DO 90 IDUPPT = 1, NPT
            IF(ABS(ZZ-Z(IDUPBD,IDUPPT)).GT.ZDIS) GOTO 80
            IF(ABS(XXX-X(IDUPBD,IDUPPT)).GT.XDIS) GOTO 80

C     DUPLICATE POINT

            NUMDUP = NUMDUP + 1
            IF(NUMDUP.LT.NUMBER) GOTO 90
            IRET = 0
            XXX = X(IDUPBD,IDUPPT)
            ZZ = Z(IDUPBD,IDUPPT)
            RETURN
80          CONTINUE
            IF(XXX.GT.XMAX.OR.XXX.LT.0.) GOTO 85
            IF(ZZ.GT.ZMAX.OR.ZZ.LT.0.) GOTO 85
90        CONTINUE
        ENDIF
      ENDDO

C  UNIQUE POINT

      IRET = -1
      IDUPBD = 0
      IDUPPT = 0
      RETURN

C  POINT OUT OF RANGE

85    IRET = 1
      IDUPBD = 0
      IDUPPT = 0
      RETURN
      END



******************************************************
C  PURPOSE : TO CALCULATE KM DISTANCE OF OBS. INDEX

      FUNCTION COORD(INDEX)

      INCLUDE 'magrav.cmn'
 
      IF(INDEX.LT.1.OR.INDEX.GT.MAXOBS) THEN
        COORD = 0.0
      ELSE
        COORD = XMINS(ITYPE) + SPACE(ITYPE) * (INDEX-1)
      ENDIF

      RETURN
      END



***************************************************************
C  PURPOSE TO INSERT POINT(XX,ZZ) IN BODY IBOD AFTER POINT NPT
C
C    IRET = 0   POINT NOT INSERTED
C          -1   POINT INSERTED

      SUBROUTINE INSERT(XXX,ZZ,IBOD,NPT,IRET)

      INCLUDE 'magrav.cmn'
 
      IRET = 0
      IF(IBOD.GT.NBODS) RETURN
      IF(IBOD.LT.1) RETURN
      IF(NPT.GT.NPTS(IBOD)) RETURN
      IF(NPTS(IBOD).GE.MAXNPT) RETURN
      IEND = NPTS(IBOD)
      NPT = NPT + 1
      NPTS(IBOD) = NPTS(IBOD) + 1
      IF(NPT.NE.NPTS(IBOD)) THEN
        L = NPTS(IBOD)
        DO I = NPT, IEND 
          X(IBOD,L) = X(IBOD,L-1)
          Z(IBOD,L) = Z(IBOD,L-1)
          L = L - 1
        ENDDO
      ENDIF
      IANOM(IBOD) = 0
      X(IBOD,NPT) = XXX
      Z(IBOD,NPT) = ZZ
      IRET = -1

C     SET LAST POINT  =  FIRST JUST TO BE SURE

      X(IBOD,IEND + 1) = X(IBOD,1)
      Z(IBOD,IEND + 1) = Z(IBOD,1)
      RETURN
      END



***************************************************
C  PURPOSE TO DELETE POINT NPT FROM BODY IBOD
C
C  IRET =  0 POINT NOT DELETED
C       = -1 POINT DELETED

      SUBROUTINE DELETE(IBOD,NPT,IRET)

      INCLUDE 'magrav.cmn'
 
      IRET = 0
      IF(IBOD.GT.NBODS.OR.IBOD.LT.1) RETURN
      IF(NPT.LT.1.OR.NPT.GT.NPTS(IBOD)) RETURN
      IF(NPTS(IBOD).LE.4) THEN
       RETURN
      ENDIF
      IEND = NPTS(IBOD) - 1
      NPTS(IBOD) = IEND 
      IF(NPT.LT.IEND) THEN
        DO I = NPT, IEND 
          II = I + 1     
          X(IBOD,I) = X(IBOD,II)     
          Z(IBOD,I) = Z(IBOD,II)
        ENDDO
      ENDIF
      IANOM(IBOD) = 0
      X(IBOD,IEND ) = X(IBOD,1)
      Z(IBOD,IEND ) = Z(IBOD,1)
      IRET = -1
      RETURN
      END

C*****************************************************************
C  PURPOSE TO CALCULATE THE GRAVITY/MAGNETIC ANOMALY FOR BODY IBOD
C
      SUBROUTINE GMAG(IBOD)

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC,X1,X2,Z1,Z2,YMAX,YMIN,SUM,DELTAG,XJ,YJ,ZJ,RAD,SIAI
      REAL*8 FDECL,COAI,ADUM,YCOR,YSP,Y1,Y2
      COMMON /CALDAT/ CALC(21,200)
      COMMON /MAGDAT/XJ,YJ,ZJ,RAD,SIAI,FDECL,COAI

      RAD = PI/180.
      NF = NFIELD(ITYPE)
      DO K = 1, NF
        CALC(IBOD,K) = 0.D0
      ENDDO
      IF(RHOMAG(IBOD,ITYPE).EQ.0.) RETURN
      ADUM = 6.6732D0*RHOMAG(IBOD,ITYPE)
      IF(ITYPE.EQ.2) THEN
      COAI = DEGCOS(FINCL)
      SIAI = DEGSIN(FINCL)
      FDECL= XTON-FDEC-YANG(IBOD)
      A = BINCL(IBOD)
      D = XTON-BDEC(IBOD)-YANG(IBOD)
      CORA =  DEGCOS(A)
      SIRA =  DEGSIN(A)
      CORD =  DEGCOS(D)
      SIRD =  DEGSIN(D)
      XJ = CORA*CORD
      YJ = -CORA*SIRD
      ZJ = SIRA
      ADUM = RHOMAG(IBOD,ITYPE)
      ENDIF
      YMIN = BDY(IBOD,1)
      YMAX = BDY(IBOD,2)
      CYANG = DEGCOS(YANG(IBOD))
      SYANG = DEGSIN(YANG(IBOD))
      YSP = SPACE(ITYPE)*SYANG
      YCOR = (X(IBOD,1) - COORD(1))*SYANG

C  CHECK EACH FIELD POINT

      DO 3100 K = 1, NF
        X1 = (X(IBOD,1) - COORD(K))*CYANG
        Z1 = Z(IBOD,1) + ZCON(ITYPE)
        IF(Z1.LE.0.) GOTO 9999
        NPT = NPTS(IBOD)
        SUM = 0.D0
        Y1 = YMAX+YCOR
        Y2 = YMIN+YCOR
        DO 3000 J = 2, NPT
          X2 = (X(IBOD,J) - COORD(K))*CYANG
          Z2 = Z(IBOD,J) + ZCON(ITYPE)
          IF(Z2.LE.0.) GOTO 9999
C  IF 2 POINTS THE SAME CHECK THE POINT AFTER
          IF(X1.EQ.X2.AND.Z1.EQ.Z2) GOTO 9999
          SUM = SUM + DELTAG(X2,X1,Z2,Z1,Y1,ITYPE,1)
          SUM = SUM - DELTAG(X2,X1,Z2,Z1,Y2,ITYPE,2)
          X1 = X2
          Z1 = Z2
3000    CONTINUE
       CALC(IBOD,K) = SUM*ADUM
3100  YCOR = YCOR - YSP
       RETURN

C    ERROR

9999  IF(ISCOPE.EQ.1) CALL GDIAGC('BELL')
      WRITE(JZ,1) ibod,k 
1     format(/,' *ERROR - Body :',i3, ' point :',i4)
      CALL GDIAGC('BELL')
      DO K = 1, NF
        CALC(IBOD,K) = 0.D0
      ENDDO
      IANOM(IBOD) = 0
      RETURN
      END
C***************************************************************
C
C  REFERENCES: RASMUSSEN R. AND PEDERSEN, L.B. (1979) END CORRECTIONS
C              IN POTENTIAL FIELD MODELING; GEOPHYSICAL PROSPECTING,
C              27, 749-760.
C
C              ENMARK, THOMAS (1981) A VERSATILE INTERACTIVE
C              COMPUTER PROGRAM FOR COMPUTATION AND AUTOMATIC
C              OPTIMIZATION OF GRAVITY MODELS; GEOEXPLORATION,19,47-66.
C
C              ENMARK, THOMAS (1982) DEVELOPMENT OF OPTIMIZATION
C              PROCEDURES FOR GRAVITY AND MAGNETIC INTERPRETATION AND
C              THEIR APPLICATION TO SOME GEOLOGICAL STRUCTURES IN
C              NORTHERN SWEDEN, DOCTORAL THESIS, UNIVERSITY OF LULEA,
C              LULEA, SWEDEN.
C
      DOUBLE PRECISION FUNCTION DELTAG(X1,X,Z1,Z,Y,ITYPE,L)
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON /MAGDAT/XJ,YJ,ZJ,RAD,SIAI,FDECL,COAI
      IF(L.EQ.1) THEN
      XP = X1 - X
      ZP = Z1 - Z
      AJ = DSQRT(XP*XP + ZP*ZP)
      ZN = -XP/AJ
      XN =  ZP/AJ
      U = -ZN*X + XN*Z
      U1 = -ZN*X1 + XN*Z1
      W = -XN*X - ZN*Z
      IF(W.EQ.0.D0) W=1.D-50
      R = DSQRT(U*U + W*W)
      R1 = DSQRT(U1*U1 + W*W)
      IF(ITYPE.EQ.1) AK = (X*Z1 - Z*X1)/(AJ*AJ)
      ENDIF
      RR = DSQRT(R*R + Y*Y)
      RR1 = DSQRT(R1*R1 + Y*Y)
      RATIO1 = (Y + RR)/(Y + RR1)
      AT1 = DATAN(Y*U1/RR1/W)-DATAN(Y*U/RR/W)
      IF(ITYPE.EQ.1) THEN
      RLOG1 = DLOG(R1*RATIO1/R)
      RLOG2 = DLOG((U1+RR1)/(U+RR))
      DELTAG = AK * (ZP*RLOG1 + XP*AT1) - ZN*RLOG2*Y
      ELSE
      RLOG11 = DLOG(RATIO1)
      RINT2 = -ZN*RLOG11-XN*AT1
      RINT3 = XN*RLOG11-ZN*AT1
      RINT4 = DLOG((RR+U)/(RR1+U1))
      BZ = -(XJ*RINT2+ZJ*RINT3+YJ*RINT4)
      BY = -(XJ*XN+ZJ*ZN)*RINT4+YJ*(XN*RINT2+ZN*RINT3)
      BZZ = BZ*ZN
      BX = BZ*XN
      DUM = FDECL*RAD
      DELTAG = BZZ*SIAI+(BX*DCOS(DUM)-BY*DSIN(DUM))*COAI
      ENDIF
      RETURN
      END
C*****************************************************
C  PURPOSE -TO CALCULATE THE SINE OF DEGREE IN DEGREES

      FUNCTION DEGSIN(DEGREE)

      DATA PI/3.1415926535/

      RADIAN = (PI/180.) * DEGREE
      DEGSIN = SIN(RADIAN)
      RETURN
      END



C*******************************************
C
C  PURPOSE TO CALCULATE THE COSINE OF DEGREE

      FUNCTION DEGCOS(DEGREE)
   
      DATA PI/3.1415926535/

      RADIAN = (PI/180.) * DEGREE
      DEGCOS = COS(RADIAN)
      RETURN
      END



C*************************************************************
C  PURPOSE TO INPUT PARAMETERS FOR GRAVITY OR MAGNETIC BODY
C  PARAMETERS ARE READ FOR BODY IBOD
C  IF IBOD  = 0, VALUES ARE ADDED TO ALL BODIES

      SUBROUTINE PARAM(IBOD)

      INCLUDE 'magrav.cmn'
 
10    WRITE(JZ,1) 
1     format(' Enter body number or 0 to add to all bodies :',$)
      READ(*,*,ERR=10) IBOD
      IF(IBOD.LT.0.OR.IBOD.GT.MAXBOD) THEN
       WRITE(JZ,*) ' *** INVALID BODY ***'
       GOTO 10
      ENDIF

C  BODY NUMBER  =  0 ADD THESE VALUES TO ALL BODIES

      IF(IBOD.EQ.0) THEN
        DO I = 1, NBODS
          IANOM(I) = 0
        ENDDO
        IF(ITYPE.EQ.1) THEN
C GRAVITY
11        WRITE(JZ,2) 
2      format(' Enter density contrast to be added to all bodies :',$)
          READ(*, *,ERR=11) DRHO
          DO I = 1, NBODS
            RHOMAG(I,1) = RHOMAG(I,1) + DRHO
          ENDDO
C  MAGNETIC
        ELSE
12        WRITE(JZ,13)
13     FORMAT(' Enter magnetization (emu*100,000), declination,',/,
     1        ' and inclination to be added to all bodies :',$)
          READ(*, *,ERR=12) DBXOR, DBDEC, DBINCL
          DO I = 1, NBODS
            RHOMAG(I,2) = RHOMAG(I,2) + DBXOR
            BDEC(I) = BDEC(I) + DBDEC
            BDEC(I) = AMOD(BDEC(I),360.)
            IF(BDEC(I).LT.0.)BDEC(I) = BDEC(I) + 360.
            BINCL(I) = BINCL(I) + DBINCL
            BINCL(I) = AMOD(BINCL(I),360.)
            IF(BINCL(I).LT.0.)BINCL(I) = BINCL(I) + 360.
          ENDDO
        ENDIF
        RETURN
      ENDIF

      IANOM(IBOD) = 0

      IF(ITYPE.EQ.1) THEN
C  GRAVITY
        WRITE(JZ,14) IBOD, RMMIN(IBOD,1), RHOMAG(IBOD,1), RMMAX(IBOD,1)
14      FORMAT( ' For body number',i3,' the contrasts are',/,3f9.3)
15      WRITE(JZ,*)'Enter contrast min, density contrast, contrast max.'
        READ(*,*,ERR=15)RMMIN(IBOD,1),RHOMAG(IBOD,1),RMMAX(IBOD,1)
      ELSE
C  MAGNETIC
        WRITE(JZ,16) IBOD,RHOMAG(IBOD,2),RMMIN(IBOD,2),RMMAX(IBOD,2),
     +               BDEC(IBOD), BINCL(IBOD)
16      FORMAT('For body number',i3,' the magnetization is',F9.1,/,
     +  'with lower and upper limits of', 2f9.1,/,
     +  'and a  polarization vector declination and inclination of',
     +  2f6.1) 
20      WRITE(JZ,21)
21      FORMAT('Enter minimum, possible and maximum magnetizations,'/
     +  ' declination and inclination of the polarization vector,'/,
     +  ' or / to retain current values.')
        READ(*, *,ERR=20)RMMIN(IBOD,2),RHOMAG(IBOD,2),RMMAX(IBOD,2),
     +              BDEC(IBOD), BINCL(IBOD)
        BDEC(IBOD) = AMOD(BDEC(IBOD),360.)
        IF(BDEC(IBOD).LT.0.)BDEC(IBOD) = BDEC(IBOD) + 360.
        BINCL(IBOD) = AMOD(BINCL(IBOD),360.)
        IF(BINCL(IBOD).LT.0.)BINCL(IBOD) = BINCL(IBOD) + 360.
      ENDIF
      RETURN
      END


C*****************************************************
C   TYPES SCALE ETC

      SUBROUTINE TSCALE

      INCLUDE 'magrav.cmn'
 
      WRITE(JZ, *) 'MODEL : ', NAME
      IF(ITYPE.EQ.1) WRITE(JZ, *) 'GRAVITY MODEL'
      IF(ITYPE.EQ.2) WRITE(JZ, *) 'MAGNETIC MODEL'
      NOBS = LEND(ITYPE) - LSTART(ITYPE) + 1
      IF(LEND(ITYPE).EQ.LSTART(ITYPE)) NOBS=0
      WRITE(JZ, 46) KOMMNT
46    FORMAT('COMMENTS '/A80)
      WRITE(JZ, 100) XMAX, ZMAX, XMINS(ITYPE), SPACE(ITYPE),
     +  NFIELD(ITYPE), LSTART(ITYPE), LEND(ITYPE), NOBS,
     +   ANOMIN(ITYPE), ANOMAX(ITYPE),
     +   DIFMIN(ITYPE), DIFMAX(ITYPE), OFFSET(ITYPE), ZCON(ITYPE)
100   FORMAT('X SCALE : ',F11.2/'Z SCALE : ',F11.2/
     +'X DISPLACEMENT : ',F13.2/'POINT SPACING : ',F10.5/
     +'NO. OF CALCULATED POINTS : ',I5/
     +'OBSERVED PROFILE BEGINS AT POINT : ',
     +I4/'OBSERVED PROFILE ENDS AT POINT : ',
     +I4/'NUMBER OF OBSERVED DATA POINTS : ',
     +I5/'ANOM. MIN.,MAX. : ',2F10.2/
     +'DIFF. MIN.,MAX. : ',2F10.2/'CALC. ANOM. OFFSET : ',F10.2,
     +',    ZCONSTANT : ',F8.4)
      JBODS=0
      DO 110 I=1,NBODS
110   IF(NPTS(I).GT.0) JBODS=JBODS+1
      WRITE(JZ,*) 'NUMBER OF BODIES : ', JBODS
      IF(ITYPE.EQ.1) WRITE(JZ,105) BKDEN
105   FORMAT('BACKGROUND DENSITY VALUE :',F6.2)
      WRITE(JZ,209) XTON
209   FORMAT('X TO NORTH ANGLE :',F8.2)
      IF(ITYPE.EQ.2) WRITE(JZ, 210) FDEC, FINCL
210   FORMAT(' FIELD DECLIN. : ',F8.2,8X,' INCLIN. : ',F8.2)
      IF(ISCOPE.EQ.2) WRITE(JZ, *) 'GRAPHICS OUTPUT SUPPRESSED'
      IF(MODE.EQ.2) WRITE(JZ, *) 'SKETCH MODE'
      IF(MODE.EQ.1) WRITE(JZ, *) 'DRAW MODE'
      WRITE(JZ, 227) SKXMIN, SKXMAX, SKZMIN, SKZMAX
227   FORMAT('SKETCH LIMITS X : ',2F10.2,/'              Z : ',2F10.2)
      WRITE(JZ, *) 'GRIDX : ', GX, ' GRIDZ : ', GZ
      RETURN
      END

C*****************************************************
C   READS A *.DMP FILE

      SUBROUTINE RDUMP

      CHARACTER*14 DTYPE

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      READ(JZ,100) NAME
100   FORMAT(8X,A10)
      READ(JZ,103) DTYPE
103   FORMAT(A14)
      IF(DTYPE.EQ.'GRAVITY MODEL ') THEN
      ITYPE=1
      ELSE
      ITYPE=2
      ENDIF
      READ(JZ, 101) KOMMNT
101   FORMAT(/A80)
      READ(JZ, 102) XMAX, ZMAX, XMINS(ITYPE), SPACE(ITYPE),
     +  NFIELD(ITYPE), LSTART(ITYPE), LEND(ITYPE), NOBS,
     +  ANOMIN(ITYPE), ANOMAX(ITYPE), DIFMIN(ITYPE), DIFMAX(ITYPE),
     +  OFFSET(ITYPE), ZCON(ITYPE), NBODS
102   FORMAT(10X,F11.2/10X,F11.2/17X,F13.2/16X,F10.5/27X,I5/35X,I4/33X,
     +I4/33X,I5/18X,2F10.2/18X,2F10.2/21X,F10.2,17X,F8.4/19X,I12)
      XMAXS(ITYPE)=COORD(NFIELD(ITYPE))
      IF(ITYPE.EQ.1) READ(JZ,203) BKDEN
203   FORMAT(26X,F6.2)
      READ(JZ,209) XTON
209   FORMAT(18X,F8.2)
      IF(ITYPE.EQ.2) READ(JZ, 210) FDEC, FINCL
210   FORMAT(17X,F8.2,19X,F8.2)
      READ(JZ, 227) SKXMIN, SKXMAX, SKZMIN, SKZMAX
227   FORMAT(/18X,2F10.2,/18X,2F10.2)
      READ(JZ, 228) GX,GZ
228   FORMAT(8X,F11.5,9X,F11.5,/)
      DO K=1,MAXBOD
      NPTS(K)=0
      ENDDO
      DO 250 K1=1,100
      READ(JZ,229) I
229   FORMAT(6X,I5)
      IF(I.EQ.999) GOTO 260
      READ(JZ,230) BDY(I,1),BDY(I,2)
230   FORMAT(21X,2F11.3)
      READ(JZ,233) YANG(I)
233   FORMAT(24X,F5.1)
      IF(ITYPE.EQ.1) THEN
      READ(JZ,231) RMMIN(I,1),RHOMAG(I,1),RMMAX(I,1)
231   FORMAT(11X,3F10.3)
      ELSE
      READ(JZ,232) RMMIN(I,2),RHOMAG(I,2),RMMAX(I,2),BDEC(I),BINCL(I)
232   FORMAT(14X,3F10.3,/,4X,F10.2,5X,F10.2)
      ENDIF
      L=0
      DO 240 K2=1,100
      L=L+1
      READ(JZ,241) J, X(I,L), Z(I,L)
      IF(J.EQ.999) GOTO 242
240   CONTINUE
241   FORMAT(I5,2F11.4)
242   NPTS(I)=L
      X(I,L)=X(I,1)
      Z(I,L)=Z(I,1)
      L=L-1
      NBODS=I
250   CONTINUE
260   CONTINUE
      READ(JZ,270) OFFSET(ITYPE)
270   FORMAT(36X,F9.2/)
      NF = NFIELD(ITYPE)
      DO I = 1, NF
        IF(ITYPE.EQ.1) THEN
          READ(JZ, 23100) I1, Z1, OBS(ITYPE,I), OBS(3,I),
     1    CALC(MAXCAL,I), OBS(4,I)
23100     FORMAT(I4,4F11.3,F4.0)
        ELSE
          READ(JZ, 23101) I1,Z1,OBS(ITYPE,I),CALC(MAXCAL,I), 
     1    OBS(5,I)
23101     FORMAT(I4,3F11.3,F4.0)
        ENDIF
      ENDDO
      CLOSE(JZ)
      RETURN
      END

C***************************************************************
C
C        DETERMINES THE OFFSET BETWEEN THE CALCULATED AND
C                    THE OBSERVED DATA.
C
C
      SUBROUTINE OFFST

      CHARACTER*1 CHR,CHA

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      CALL VCURSR(CHR,XDUM,ZDUM)
C     THERE ARE FIVE POSSIBLE CHOICES FOR CHR
C        IF CHR = Y, IOFS = 1
C               = N, IOFS = 0
C               = O, OFFSET BETWEEN CALC. AND OBSER. CURVES IS DETERMINED
C               = P, AN ARBITRARY VALUE IS ADDED TO OFFSET(ITYPE)
C               = M, AN ARBITRARY VALUE IS SUBTRACTED FROM OFFSET(ITYPE)
C
      IF(CHR.EQ.'Y') THEN
        IOFS=1
        RETURN
      ENDIF
      IF(CHR.EQ.'N') THEN
        IOFS=0
        RETURN
      ENDIF
C
C   IF IOFS = 0, THEN OFFSET OF CALC. ANOM. IS NOT VARIED BY
C                SUBROUTINES AMOVE AND AMPL.
C           = 1, THEN OFFSET OF CALC. ANOM. IS VARIED BY
C                SUBROUTINES AMOVE AND AMPL.
C
      IF(CHR.EQ.'O') THEN
        SUM = 0.
        ISTART = LSTART(ITYPE)
        IEND = LEND(ITYPE)
        DO I = ISTART, IEND
          SUM = SUM + OBS(ITYPE,I) - CALC(MAXCAL,I)
        ENDDO
        OFFSET(ITYPE) = SUM/(IEND - ISTART + 1)
        CALL SAV(NBACK)
        IBOD = 0
        KALK = 0
        CALL PLANOM(IBOD,KALK)
        RETURN
      ENDIF
      CALL VCURSR(CHA,XDUM,ZDUM)
C
C     CHA*OFS = ARBITRARY VALUE TO BE + OR - FROM OFFSET(ITYPE)
C     
      OFS=(ANOMAX(ITYPE)-ANOMIN(ITYPE))/100.
      IDUM = ICHAR(CHA) - 48
      IF(IDUM.LT.0.OR.IDUM.GT.9) RETURN
      IF(IDUM.EQ.0)IDUM = 10
      IF(CHR.EQ.'M')GO TO 50
      IF(CHR.NE.'P') RETURN
      OFFSET(ITYPE)=OFFSET(ITYPE)+IDUM*OFS
      IBOD = 0
      KALK = 0
      CALL PLANOM(IBOD,KALK)
      CALL SAV(NBACK)
      RETURN
50    OFFSET(ITYPE)=OFFSET(ITYPE)-IDUM*OFS
      IBOD = 0
      KALK = 0
      CALL PLANOM(IBOD,KALK)
      CALL SAV(NBACK)
      RETURN
      END

C***************************************************************

      SUBROUTINE ISOSTA(IGO,IERS)

      INCLUDE 'magrav.cmn'
 
      DIMENSION ZP(24),XB(24),ZB(24)
      INTEGER GIPOLYX
C=======================================================================
C PURPOSE:- TO CALCULATE AT EACH DATA POSITION A WEIGHT FOR A ONE-SQUARE-
C    METER VERTICAL COLUMN EXTENDING BETWEEN THE UPPER AND LOWER SURFACES
C    OF THE MODEL. AT EACH LOCATION THE INTERSECTION INTERVALS BETWEEN
C    THE VERTICAL COLUMN AND ANY MODEL BODIES ARE DETERMINED, AND A
C    COLUMN WEIGHT IS DETERMINED BY MULTIPLYING THESE INTERVALS WITH THE
C    APPROPRIATE DENSITY CONTRASTS, AND SUMMING THE RESULTANTS WEIGHTS
C    FOR EACH COLUMN. CHANGES IN TOPOGRAPHY ALONG THE PROFILE ARE ALLOWED
C    FOR BY EXTENDING EACH COLUMN UPWARD A DISTANCE EQUAL TO THE ELEVATION
C    ABOVE SEA LEVEL OF A GIVEN DATA POSITION, AND THEN MULTIPLYING THE
C    ADDED INTERVAL BY THE DENSITY CONTRAST OF THE APPROPRIATE MODEL BODY.
C    DIFFERENCES IN WEIGHT OF EACH COLUMN AS CONTRASTED WITH THE MEAN COLUMN
C    WEIGHT CAN BE USED TO CONSTRAIN THE MODELLING PROCESS IN ADDITION
C    TO THE NORMAL ONE OF IMPROVING THE FIT OF A CALCULATED PROFILE TO
C    THE OBSERVED PROFILE, AND HENCE TO PROVIDE A MODEL WHICH IS ALSO
C    IN ISOSTATIC EQUILIBRIUM AS WELL AS SATISFYING THE GRAVITY DATA. THE
C    LEVEL OF ISOSTATIC COMPENSATION IS ASSUMED TO BE THE LOWER SURFACE
C    OF THE MODEL. DIFFERENCES IN COLUMN WEIGHT ARE EXPRESSED IN TERMS OF
C    DIFFERENTIAL PRESSURES WHERE 98 BARS IS EQUAL TO A WEIGHT DIFFERENCE
C    OF ONE GIGAGRAM PER SQUARE METER (GGM/M**2).
C
      IERS=0
      IF(ITYPE.GE.2.OR.NBODS.LE.0)RETURN
      IF(IGO.EQ.0)ISET=1
      IF(ISET.NE.1) THEN
      WRITE(JZ,*)' WARN - TO INCLUDE SURFACE ELEV. WEIGHTS USE ISOS'
      ENDIF
      NF=NFIELD(ITYPE)
      DO 10 I=1,NF
      WD(I)=0.
10    CONTINUE
C         IF IGO = 0, ISOSTA WAS ENTERED FROM THE MAIN PROGRAM
C                  1, ISOSTA WAS ENTERED FROM SUBROUTINE GRAP
C                  3, ISOSTA WAS ENTERED FROM SUBROUTINE CALFUN
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      IF(IGO.NE.0)GO TO 16
      X0=COORD(ISTART)
      DO 11 I=1,NF
      SDEN(I)=0.
11    CONTINUE
      DO 15 I=ISTART,IEND
      DO 13 K=1,NBODS
      NBP=NPTS(K)-1
      IF(NBP.LE.0)GO TO 13
      DO 12 J=1,NBP
      XB(J)=X(K,J)
      ZB(J)=Z(K,J)
12    CONTINUE
      M=GIPOLYX(NBP,XB,ZB,X0,.001)
      IF(M.GE.1)GO TO 14
13    CONTINUE
      GO TO 15
14    SDEN(I)=RHOMAG(K,1)
15    X0=X0+SPACE(ITYPE)
      WRITE(JZ,17)(SDEN(I),I=ISTART,IEND)
17    FORMAT(' SURFACE DENSITIES '//,(10F8.3))
16    X0=COORD(ISTART)
      DO 90 I=ISTART,IEND
      SUM=0.
      DO 40 K=1,NBODS
      NBP=NPTS(K)-1
      IF(NBP.LE.0)GO TO 40
      ICO=0
      DO 25 L=1,NBP
      IF(X0.NE.X(K,L))GO TO 22
      IF(X0.NE.X(K,L+1))GO TO 22
      IERS=1
      WRITE(JZ,21) X0
21    FORMAT(' ERROR RETURN FROM ISOSTA - VERTICAL BOUNDARY AT ',F10.3)
      RETURN
22    CONTINUE
      IF(X(K,L).LT.X0)GO TO 23
      IF(X(K,L).EQ.X0)GO TO 25
      IF(X(K,L+1).GT.X0)GO TO 25
      GO TO 24
23    IF(X(K,L+1).LT.X0)GO TO 25
24    ICO=ICO+1
      ZP(ICO)=(X0-X(K,L))*(Z(K,L+1)-Z(K,L))/(X(K,L+1)-X(K,L))+Z(K,L)
25    CONTINUE
      IF(ICO.LE.1)GO TO 40
      IM1=ICO-1
      DO 27 I1=1,IM1
      I2=I1+1
      DO 27 I3=I2,ICO
      IF(ZP(I1).LT.ZP(I3))GO TO 27
      DUM=ZP(I1)
      ZP(I1)=ZP(I3)
      ZP(I3)=DUM
27    CONTINUE
      DO 50 I1=1,NBP
      XB(I1)=X(K,I1)
      ZB(I1)=Z(K,I1)
50    CONTINUE
      DO 28 I1=1,IM1
      ZI=ZP(I1)+(ZP(I1+1)-ZP(I1))/2.
      M=GIPOLYX(NBP,XB,ZB,X0,ZI)
      IF(M.EQ.0)GO TO 28
      SUM=SUM+(ZP(I1+1)-ZP(I1))*RHOMAG(K,1)
28    CONTINUE
40    CONTINUE
      IF(OBS(3,I).LT.-9998.) THEN
        WD(I)=SUM
      ELSE
        WD(I)=SUM+OBS(3,I)*SDEN(I)
      ENDIF
90    X0=X0+SPACE(ITYPE)
      WMEAN = 0.
      DO 91 I=ISTART,IEND
      WMEAN = WMEAN + WD(I)
91    CONTINUE
      WMEAN = WMEAN/(IEND-ISTART+1)
      DO 92 I=ISTART,IEND
92    WD(I) = (WD(I)-WMEAN)*98.
      IF(IGO.EQ.0) THEN
      WX=WD(ISTART)
      WN=WX
      DO 110 I=ISTART,IEND
      IF(WX.LT.WD(I))WX=WD(I)
      IF(WN.GT.WD(I))WN=WD(I)
110   CONTINUE
      WRITE(JZ,140)WX,WN,PMAX,PMIN
140   FORMAT(' MAX. AND MIN. COLUMN DIFFERENTIAL PRESSURES = ',2F10.3,
     +//,' MAX. AND MIN. GRAPH LIMITS, PMAX AND PMIN = ',2F7.0)
141   WRITE(JZ,145)
145   FORMAT(' ENTER NEW PMAX AND PMIN OR / :',$)
      READ(*,*,ERR=141) PMAX,PMIN
      WRITE(JZ,*) ' PMAX AND PMIN ARE NOW ',PMAX,PMIN,' BARS'
      WRITE(JZ,*) ' IWFLAG AND WW = ',IWFLAG,WW
146   WRITE(JZ,147)
147   FORMAT(' ENTER NEW IWFLAG, OR / TO RETAIN THE OLD VALUE',/,
     *10X,'1 - FOR GRAVITY CONSTRAINT ONLY',/,
     *10x,'2 - FOR ISOSTATIC CONSTRAINT')
      READ(*,*,ERR=146) IWFLAG
      IF(IWFLAG.LT.1.OR.IWFLAG.GT.2) GOTO 146
      IF(IWFLAG.EQ.1) THEN
      WW = 0.
      WRITE(JZ,*) ' IWFLAG, AND WW ARE NOW ',IWFLAG,WW
      ELSE
      WRITE(JZ,148)
148   FORMAT(' ENTER NEW WW, OR / TO RETAIN THE OLD VALUE',/,
     *10X,' 0 - FOR ISOSTATIC CONSTRAINT ONLY',/,
     *10X,'>0 - FOR ISOSTATIC + GRAVITY CONSTRAINT')
      READ(*,*) WW
      WRITE(JZ,*) ' IWFLAG, AND WW ARE NOW ',IWFLAG,WW
      ENDIF
C     OPTIMIZATION CONTROL
C        IWFLAG = 1, FITS OBSERVED GRAVITY
C               = 2, USES ISOSTATIC CONSTRAINT IF WW = 0.
C               = 2, USES BOTH GRAVITY AND ISOSTATIC DATA IF WW > 0.
C
      WRITE(JZ,18)(WD(I),I=ISTART,IEND)
18    FORMAT(//' COLUMN DIFFERENTIAL PRESSURES(BARS) '//,(10F8.3))
      ENDIF
      RETURN
      END

C***************************************************************
C
C        PURPOSE- TO APPLY A LINEAR CORRECTION TO THE OBSERVED DATA
C
      SUBROUTINE REGC(RCOR)

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC
      COMMON /CALDAT/ CALC(21,200)

      NF=NFIELD(ITYPE)
      IF(NF.EQ.0)RETURN
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      ICO = IEND - ISTART + 1
      IF(ABS(RCOR).GT.1.E-6) GOTO 50
      ICM1=ICO-1
      ISUM=ICM1*ICO/2
      SQIS=ICM1*ICO*(ICO+ICO+1)/6
      XIND=0.
      XSUM=0.
      DUM=OFFSET(ITYPE)
      ICO=0
      DO I=ISTART,IEND
        XIND=XIND+(OBS(ITYPE,I)-CALC(MAXCAL,I)-DUM)*ICO
        XSUM=XSUM+OBS(ITYPE,I)-CALC(MAXCAL,I)-DUM
        ICO=ICO+1
      ENDDO
      DET=ICO*SQIS-ISUM*ISUM
      SL=(ICO*XIND-ISUM*XSUM)/DET
      P1=(SQIS*XSUM-ISUM*XIND)/DET
      XI=0.
      DO I = ISTART, IEND
        OBS(ITYPE,I)=OBS(ITYPE,I)-P1-SL*XI
        XI=XI+1.
      ENDDO
      RETURN
50    RCOR=RCOR/(ICO-1)
      ICO=0
      DO I=ISTART,IEND
        OBS(ITYPE,I)=OBS(ITYPE,I)-ICO*RCOR
        ICO=ICO+1
      ENDDO
      RETURN
      END

C***************************************************************************
C
C   SUBROUTINE DFS001
C
C     THIS PROGRAM MINIMIZES THE SUM OF THE SQUARES OF NON LINEAR FUNCTIONS.
C     THE METHOD USED HAS BEEN DEVELOPED AND DESCRIBED BY M.J.D.POWELL
C     IN 'THE COMPUTER JOURNAL' VOL.7,NO.4,JAN.1965, PAGE 303.
C     THE METHOD FINDS X(1)..........X(N) SUCH THAT SM IS A MINIMUM
C        WHERE SM(X(1)..........X(N)) = SUM 
C     OVER K OF (F(K,X(1)............X(N))**2)
C        WHERE K RUNS FROM 1 TO M WITH M  GREATER THAN N
C
C
C     THE PARAMETER NAMES ARE AS FOLLOWS
C
C     M =  NUMBER OF OBSERVATIONS OF THE FUNCTION F
C     N =  NUMBER OF INDEPENDANT VARIABLES
C
C     F() IS ARRAY OF SIZE GREATER THAN M, ON LEAVING DFS001 THIS WILL
C     CONTAIN THE VALUES OF F(I,X(1)......X(N))  I = 1 TO M
C
C     X() IS AN ARRAY OF SIZE GREATER THAN N WHICH CONTAINS THE VALUES OF
C     THE VARIABLES X(1) TO X(N). ON ENTERING DFS001 THEY ARE THE INITIAL
C     APPROXIMATIONS TO THE MINIMUM. ON LEAVING DFS001 THESE WILL BE THE
C     BEST VALUES OBTAINED.
C
C     E() IS AN ARRAY OF SIZE EQUAL TO 2*N. THE FIRST N VALUES ARE THE
C     FRACTIONAL ACCURACIES OF THE PARAMETERS REQUIRED.
C     I.E. E(I) = 0.0100 REQUESTS AN ACCURACY OF 1 IN X(I).
C     THE REST OF THE ARRAY E(N) TO E(2*N) IS USED AS WORKING SPACE.
C     NOTE: E(I) IS EFFECTIVELY USED AS A MESH ON THE FIRST ITERATION
C     TO FORM THE FIRST DERIVATIVE OF THE FUNCTION F AND MUST THEREFORE
C     BE REASONABLY SMALL ESCALE IS A NUMBER WHOSE VALUE LIMITS
C     THE MOVEMENT OF THE VARIABLE IN ANY ONE ITERATION
C     TO AN AMOUNT EQUAL TO ESCALE*E(I)
C
C     IWRITE IS AN INTEGER WHICH CONTROLS THE AMOUNT OF INFORMATION
C     PRINTED BY THE ROUTINE. THERE WILL BE A WRITEOUT EVERY IWRITE
C     ITERATIONS. THE WRITEOUT CONSISTS OF THE ITERATION NUMBER, NUMBER
C     OF FUNCTION EVALUATIONS, THE VALUES OF THE VARIABLES, THE VALUE
C     OF THE SUM OF THE SQUARES, AND THE INDIVIDUAL FUNCTION VALUES.
C     IF IWRITE IS NEGATIVE, THERE WILL BE A WRITEOUT AFTER EVERY IWRITE
C     ITERATIONS BUT WITHOUT THE FUNCTION VALUES.
C     IF IWRITE IS ZERO, THERE WILL BE NO PRINTED OUTPUT.
C
C     MAXITC IS AN INTEGER WHICH WILL RETURN CONTROL TO
C     THE CALLING ROUT AFTER MAXITC ITERATIONS.
C
C     ON LEAVING DFS001, THE FIRST N*N ELEMENTS OF ARRAY W() WILL CONTAIN
C     THE VARIANCE-COVARIANCE MATRIX ELEMENTS V(I,J) STORED IN ORDER
C     V(1,1),V(1,2).....V(1,N),V(2,1)............V(N,N).
C
C     W() IS A WORKING ARRAY WHOSE SIZE MUST BE EQUAL TO OR GREATER THAN
C     N + ((M + (3*N/2))*N + 1))
C
C     THE PROGRAM CALLS TWO OTHER ROUTINES DFS002 AND CALFUN
C     WHICH MUST CALCULATE THE FUNCTION VALUES F FOR THE PASSED VALUES
C     OF THE PARAMETERS X(1)......X(N).
C
C     SUBROUTINE DFS002 IS SUPPLIED INTACT AND FINDS THE MINIMUM OF A
C     FUNCTION IN ONE DIMENSION

      SUBROUTINE DFS001 (M,N,X,E,ESCALE,IWRITE,MAXITC,IERR,IROUTE,IMOD)     

      DIMENSION  X(1), E(1)
      REAL*8 F, W
      COMMON /DFS/ F(200)
      COMMON /WORK/ W(4900)

C     IN THIS SECTION WE INITIALIZE SOME INTEGER CONSTANTS FOR THE
C     LOCATION OF INFORMATION WITHIN THE ARRAY W

      DO I = 1, N
        IF(X(I).EQ.0.) THEN
          WRITE(JZ, *) '* CANNOT USE ZERO ESTIMATE FOR PARAMETER ', I
          IERR = 1
          RETURN
        ENDIF
      ENDDO
      MPLUSN = M + N
      KST = N + MPLUSN
      NPLUS = N + 1
      KINV = NPLUS * (MPLUSN + 1)
      KSTORE = KINV - MPLUSN - 1
      NN = N + N
      MAXFUN = (2*N) + 2 + (6*MAXITC)
      INVAR = 1

C     THE INTEGER INVAR IS NORMALLY SET TO 1 ,AFTER LAST ITERATION IT
C     IS SET TO 2 AND THE VARIANCE.CO-VARIANCE MATRIX IS CALCULATED
C     STORES THE FRACTIONAL ACCURACY REQUESTED AND THEN CALCULATES THE
C     ABSOLUTE VALUES OF THE ERRORS.

      DO LLM = 1, N
        LLN = LLM + N
        E(LLN) = E(LLM)
        E(LLM) = E(LLN) * X(LLM)
      ENDDO
120   K = NN

C     THIS REGION CALCULATES THE FIRST DERIVATIVES OF THE FUNCTION IN
C     THE CO-ORDINATE DIRECTIONS AND NORMALIZES THEM SUCH THAT SUM OVER
C     K OF (DERIVATIVE(I)**2) IS UNITY FOR ALL VALUES OF I

      IAMP = 1
      CALL CALFUN (M,N,IAMP,IERS,IROUTE,IMOD)
      IF(IERS.EQ.0) GOTO 125
      IERR = 1
      RETURN
125   CONTINUE

C     STORES THE INITIAL FUNCTION VALUES

      DO I = 1, M
        K = K + 1
        W(K) = F(I)
      ENDDO
      IINV = 2
      K = KST
      I = 1
140   X(I) = X(I) + E(I)
      CALL CALFUN (M,N,IAMP,IERS,IROUTE,IMOD)
      IF(IERS.EQ.O) GOTO 145
      IERR = 1
      RETURN
145   CONTINUE 

C     CALCULATES THE FUNCTION VALUES FOR  F(X + H) WHERE H = ABSOLUTE
C     ACCURACY REQUESTED.

      X(I) = X(I) - E(I)
      DO J = 1, N
        K = K + 1
        W(K) = 0.0
        W(J) = 0.0
      ENDDO
      SUM = 0.0
      KK = NN

C     CALCULATES VALUES OF THE DERIVATIVES IN THE COORDINATE DIRECTIONS
C     SUMS THE INDIVIDUAL DERIVATIVES

      DO J = 1, M
        KK = KK + 1
        F(J) = F(J) - W(KK)
        SUM = SUM + F(J) * F(J)
      ENDDO

C     THIS ERROR CONDITION USUALLY OCCURS BECAUSE OF A CODING ERROR IN
C     THE SUBROUTINE CALFUN.

      IF(SUM.LE.0) THEN
        IF(IWRITE.NE.0) THEN
          WRITE (6,180) I
180       FORMAT('DFS001: E(',I3,') UNREASONABLY SMALL')
        ENDIF
        IERR = I
        DO J = 1, M
          NN = NN + 1
          F(J) = W(NN)
        ENDDO
        RETURN
      ENDIF

C     IN STATEMENT 210 WE CANCEL SCALING FOR CALCULATION OF CO-VARIANCE
C     MATRIX ,NORMALLY WE GO DIRECTLY TO STATEMENT 220

200   IF(INVAR.NE.1) THEN
        SUM = E(I) * E(I)
      ENDIF
220   SUM = 1.0  / SQRT(SUM)
      J = K - N + I

C     CALCULATES THE COMPONENTS OF THE DIRECTION VECTORS D(I)

      W(J) = E(I) * SUM
      DO J = 1, M
        K = K + 1
        W(K) = F(J) * SUM
        KK = NN + J

C     CALCULATES ELEMENTS OF THE NORMAL MATRIX

        DO II = 1, I
          KK = KK + MPLUSN
          W(II) = W(II) + W(KK) * W(K)
        ENDDO
      ENDDO

      ILESS = I - 1
      IGAMAX = N + I - 1
      INCINV = N - ILESS
      INCINP = INCINV + 1

C     INVERTS THE ONE BY ONE MATRIX  W(1)    

      IF(ILESS.GT.0) GOTO 265 
        W(KINV) = 1.0/W(1)
260     IF(IINV.EQ.1) GOTO 970
        I = I + 1      
        IF(I-N.LE.0) GOTO 140
        IINV = 1     
C  
C     THIS REGION IS PASSED THROUGH ONLY ON ITERATION 0 AND SETS UP THE  
C  
C     WRITING CONTROL PARAMETERS     
C  
        FF = 0.0     
        KL = NN      

C  
C     EVALUATES THE SUM OF THE SQUARES OF THE FUNCTIONS    
C  
        DO I = 1, M
          KL = KL + 1
          F(I) = W(KL)
          FF = FF + F(I) * F(I)
        ENDDO
        IF(INVAR.EQ.2) GOTO 1140
        ICONT = 1
        ISS = 1
        MC = N + 1
        IPP = IABS(IWRITE) * (IABS(IWRITE)-1)
        ITC = 0
        IPS = 1
        IPC = 0
        GOTO 970
265   CONTINUE

C     THIS NEXT REGION PERFORMS THE MATRIX INVERSION OF THE NORMAL
C     EQUATION MATRIX BY PARTITIONING. THE FULLY INVERTED MATRIX IS
C     OBTAINED BY REPEATED PASSES THROUGH THIS REGION.
C     AN OUTLINE OF THE INVERSION OF A MATRIX BY PARTITIONING IS GIVEN
C     IN  ELEMENTARY MATRIX ALGEBRA  BY  F.E.HOHN  P.109
C     IN  THIS ROUTINE  B = 1/C  AND BB = -B/C

310   IF(INVAR.EQ.1) THEN
        B = 1.0
      ELSE
        B = W(I)
      ENDIF
      DO J = NPLUS, IGAMAX
        W(J) = 0.0
      ENDDO
      KK = KINV
      DO II = 1, ILESS
        IIP = II + N
        W(IIP) = W(IIP) + W(KK) * W(II)
        JL = II + 1
        IF(JL-ILESS.LE.0) THEN
          DO JJ = JL, ILESS
            KK = KK + 1
            JJP = JJ + N
            W(IIP) = W(IIP) + W(KK) * W(JJ)
            W(JJP) = W(JJP) + W(KK) * W(II)
          ENDDO
        ENDIF
        B = B - W(II) * W(IIP)
        KK = KK + INCINP
      ENDDO
      B = 1.0/B
      KK = KINV

C  
C     CALCULATES AND STORES THE ELEMENTS OF THE INVERTED MATRIX   
C  
      DO II = NPLUS, IGAMAX
        BB = -B * W(II)
        DO JJ = II, IGAMAX
          W(KK) = W(KK) - BB * W(JJ)
          KK = KK + 1
        ENDDO
        W(KK) = BB
        KK = KK + INCINV
      ENDDO
      W(KK) = B
      GOTO 260

C     NOW START AN ITERATION , EXACT DETAILS OF THE METHOD ARE FOUND IN
C     THE PAPER BY POWELL

420   ITC = ITC + 1
      IF(MAXITC-ITC.LT.0) THEN
        IF(IWRITE.EQ.0) GOTO 1060
        WRITE (6, *) 'THE MAXIMUM NUMBER OF ALLOWED ITERATIONS ',
     +                  'HAVE BEEN PERFORMED'
        GOTO 1060
      ENDIF
      K = N
      KK = KST

C     INITIALLY WE CALCULATE THE VECTOR P
C       WHERE P(I) = SUM OVER K (GAMMA(K,I)*F(K))

      DO I = 1, N
        K = K + 1
        W(K) = 0.0
        KK = KK + N
        W(I) = 0.0
        DO J = 1, M
          KK = KK + 1
          W(I) = W(I) + W(KK)*F(J)
        ENDDO
      ENDDO

C     NOW CALCULATE THE MOVEMENT COMPONENT Q(I) FROM THE INVERSE DERIV-
C     ATIVE MATRIX AND THE VECTOR P IN THE DIRECTION D(I)

      DM = 0.0
      K = KINV
      DO II = 1, N
        IIP = II + N
        W(IIP) = W(IIP) + W(K) * W(II)
        JL = II + 1

C     NOW SELECT THE DIRECTION TO BE REPLACED BY DELTA AFTER THIS ITER-
C     ATION. THE DIRECTION BEING REPLACED IS STORED IN KL
C     THE VALUE OF ABS(P(KL).Q(KL))IS STORED IN DM

        IF(JL-N.LE.0) THEN
          DO JJ = JL, N
            JJP = JJ + N
            K = K + 1
            W(IIP) = W(IIP) + W(K) * W(JJ)
            W(JJP) = W(JJP) + W(K) * W(II)
          ENDDO
          K = K + 1
        ENDIF
        IF(DM-ABS(W(II)*W(IIP)).LT.0.) THEN
          DM =  ABS(W(II) * W(IIP))
          KL = II
        ENDIF
      ENDDO

C     CALCULATES THE ABSOLUTE ERROR REQUESTED

      DO LLM = 1, N
        LLN = LLM + N
        E(LLM) = E(LLN) * X(LLM)
      ENDDO

C     CALCULATE THE DIRECTION AND DISTANCE TO THE MINIMUM, A COMPONENT   
C  
C     IS DELTA(I)  =  Q(I)*D(I) WHERE D(I) IS AN N COMPONENT VECTOR 


      II = N + MPLUSN * KL
      CHANGE = 0.0
      DO I = 1, N
        JL = N + I     
        W(I) = 0.0   
        DO J = NPLUS, NN 
          JL = JL + MPLUSN      
          W(I) = W(I) + W(J) * W(JL)
        ENDDO
        II = II + 1
        W(II) = W(JL)
        W(JL) = X(I)

C     SELECT CHANGE TO BE MAX(DELTA(I)/E(I)) I.E. CO-ORDINATE DIRECTION
C     WHOSE DISTANCE FROM THE MINIMUM IS FARTHEST WITH RESPECT TO THE
C     REQUESTED ACCURACY.

        IF(ABS(E(I)*CHANGE)-ABS(W(I)).LE.0.) THEN
          CHANGE =  ABS(W(I)/E(I))
        ENDIF
      ENDDO

      DO I = 1, M
        II = II + 1
        JL = JL + 1
        W(II) = W(JL)
        W(JL) = F(I)
      ENDDO

      FC = FF
      ACC = 0.10/CHANGE
      IT = 3
      XC = 0.0
      XL = 0.0
      IS = 3
      ITMAX = 6
      RELAC = 0.1
      XSTEP = -AMIN1(0.5000,ESCALE/CHANGE)

C     SELECTS A GRID VALUE OF EITHER 0.500 OR ESCALE/CHANGE TO BE A
C     MINIMUM. I.E. IN DFS002 WE SEARCH IN ONE DIMENSION ALONG DELTA
C     FOR THE MINIMUM VALUE OF SM IN STEPS OF XSTEP. THEREFOR IF ESCALE
C     IS SMALL WE WILL MOVE ONLY SMALL DISTANCES IN DIRECTION OF DELTA-
C     IF ESCALE IS GREATER THAN 0.05 * CHANGE, THEN WE MOVE JUST HALF
C     WAY TO THE COMPUTED MINIMUM IN ONE STEP.
C     THE CONDITION OF CHANGE BEING LESS THAN 1.0D0 IS ACCEPTED AS THE
C     MINIMUM. IN THIS NEXT PART WE ENTER AND RETURN FROM DFS002 IN
C     ORDER TO DETERMINE THE VALUE OF LAMBDA SUCH THAT (SU(X(I) + LAMDA*
C     DELTA(I)) IS A MINIMUM DFS002 ASSUMES THAT SM APPROACHES THE MIN-
C     IMUM IN THE FORM OF A QUADRATIC AND CALCULATES ON THIS ASSUMPTION
C     TO PERFORM ITS TASK DFS001 ASKS FOR A MINIMUM OF THREE EVALUA-
C     TIONS OF THE FUNCTION. IF IT HAS NOT FOUND THE MINIMUM IN THIS
C     NUMBER, THEN THE VALUE OF XSTEP IS CHANGED. A MAXIMUM OF JUST SIX
C     FUNCTION EVALUATIONS IS ALLOWED BEFORE GOING ON WITH THE CALCULA-
C     TION. THE FUNCTION EVALUATIONS ARE STORED AND USED TO EVALUATE
C     THE FUNCTION DERIVATIVES IN THE DIRECTION DELTA.

      IF(CHANGE-1.0.LE.0.0) ICONT = 2
590   CALL DFS002 (IT,XC,FC,ITMAX,ACC,RELAC,XSTEP)

C     FC CONTAINS INTERMEDIATE VALUES OF THE SUM OF SQUARES
C
C     XC CONTAINS THE VALUE OF LAMBDA
C
C     XL CONTAINS INTERMEDIATE VALUES OF LAMBDA

      GOTO (600,780,780,780), IT

600   MC = MC + 1
      IF(MC-MAXFUN.GT.0) THEN
        IF(IWRITE.NE.0) THEN
          WRITE(JZ, *) 'DFS001 :', MAXFUN, ' CALLS OF CALFUN'
        ENDIF
        ISS = 2
        GOTO 780
      ENDIF

C     CALCULATES THE FUNCTION VALUES FOR X  + LAMDA(A)*DELTA AND
C     X + LAMDA(B)*DELTA AND ALSO EVALUATES SM FOR THESE PARAMETER VALUES

630   XL = XC - XL
      DO J = 1, N
        X(J) = X(J) + XL * W(J)
      ENDDO
      XL = XC
      IAMP = 0
      CALL CALFUN(M,N,IAMP,IERS,IROUTE,IMOD)
      IF(IERS.EQ.0) GOTO 645
      IERR = 1
      RETURN
645   FC = 0.0

C     CALCULATES THE NEW SUM OF SQUARES OF THE FUNCTION.

      DO J = 1, M
        FC = FC + F(J) * F(J)
      ENDDO

      IF(IS.LE.2) GOTO 690
      K = N
      IF(FC-FF) 670, 590, 680

C     SETS  FMIN TO THE LOWEST AND FSEC TO THE SECOND LOWEST SUM OF
C     SQUARES OF THE FUNCTION.

670   IS = 2
      FMIN = FC
      FSEC = FF
      GOTO 750

680   IS = 1
      FMIN = FF
      FSEC = FC
      GOTO 750

690   IF(FC-FSEC.GE.0.0) GOTO 590
      IF(IS.NE.2) THEN
        K = N
      ELSE
        K = KSTORE
      ENDIF

720   IF(FC-FMIN) 740, 590, 730
730   FSEC = FC
      GOTO 750
740   IS = 3 - IS
      FSEC = FMIN
      FMIN = FC

C     STORES INTERMEDIATE VALUES OF THE PARAMETERS.

750   DO J = 1, N
        K = K + 1
        W(K) = X(J)
      ENDDO

C     STORES THE INTERMEDIATE FUNCTION VALUES FOR THE LOWEST AND SECOND
C     LOWEST SUMS OF SQUARES.

      DO J = 1, M
        K = K + 1
        W(K) = F(J)
      ENDDO

C     AT THIS POINT WE RETURN TO DFS002

      GOTO 590
780   K = KSTORE

C********************************************************
C      THIS IS AN ALTERNATE EXIT FOR THIS APPLICATION

C      IF(ITC.EQ.MAXITC) RETURN

C     WE ARRIVE AT THIS POINT WHEN DFS002 HAS FINISHED

      KK = N
      IF(IS.EQ.0) THEN
        K = N
        KK = KSTORE
      ENDIF
800   SUM = 0.0
      DM = 0.0
      JJ = KSTORE

C     WE STORE THE NEW VALUES OF X(I), F(I) AND THE APPROXIMATE NEW
C     DERIVATIVES.

      DO J = 1, N
        K = K + 1
        KK = KK + 1
        JJ = JJ + 1
        X(J) = W(K)
        W(JJ) = W(K) - W(KK)
      ENDDO

C     AGAIN CALCULATE A SCALING FACTOR,THIS TIME FOR THE NEW DERIVATIVE
C     CALCULATES DM AS SUM OVER K OF  U(K,DELTA)*F(K,NEW(X(I)))

      DO J = 1, M
        K = K + 1
        KK = KK + 1
        JJ = JJ + 1
        F(J) = W(K)
        W(JJ) = W(K) - W(KK)
        SUM = SUM + W(JJ) * W(JJ)
        DM = DM + F(J) * W(JJ)
      ENDDO

      IF(ISS.EQ.2) GOTO 1060
      J = KINV
      KK = NPLUS - KL

C     REPOSITIONS  ELEMENTS IN THE NORMAL EQUATION MATRIX AND IN THE     
C     INVERTED MATRIX.  

      DO I = 1, KL
        K = J + KL - I
        J = K + KK
        W(I) = W(K)
        W(K) = W(J-1)
      ENDDO

      IF(KL-N.LT.0) THEN
        KL = KL + 1
        JJ = K
        DO I = KL, N
          K = K + 1
          J = J + NPLUS - I
          W(I) = W(K)
          W(K) = W(J-1)
        ENDDO
        W(JJ) = W(K)
        B = 1.0/W(KL-1)
        W(KL-1) = W(N)
      ELSE
        B = 1.0/W(N)
      ENDIF
880   K = KINV
      DO I = 1, ILESS
        BB = B * W(I)
        DO J = I, ILESS
          W(K) = W(K) - BB * W(J)
          K = K + 1
        ENDDO
        K = K + 1
      ENDDO
      IF(FMIN-FF.GE.0.) THEN
        CHANGE = 0.0
      ELSE
        FF = FMIN
        CHANGE =  ABS(XC) * CHANGE
      ENDIF
930   XL = -DM/FMIN
      DUM = SUM + DM * XL
      IF(DUM.LE.0.) RETURN
      SUM = 1.0/SQRT(DUM)
      K = KSTORE


C     CALCULATES THE COMPONENTS OF THE NEW VECTOR DIRECTION.

      DO I = 1, N
        K = K + 1
        W(K) = SUM * W(K)
        W(I) = 0.0
      ENDDO

C     CALCULATES THE NEW AND CORRECTED DERIVATIVES OF THE FUNCTION IN
C     THE DIRECTION OF DELTA

      DO I = 1, M
        K = K + 1
        W(K) = SUM * (W(K) + XL*F(I))
        KK = NN + I
C     REPLACES ELEMENTS IN THE MATRIX I.E. CHANGES THE KL DIRECTION FOR
        DO J = 1, N
          KK = KK + MPLUSN
          W(J) = W(J) + W(KK) * W(K)
        ENDDO
      ENDDO

C     GO BACK FOR NEW MATRIX INVERSION

      GOTO 310
970   IPC = IPC - IABS(IWRITE)

C     THE FOLLOWING INSTRUCTION CONTROLS THE OUTPUT OF INFORMATION

      IF(IPC.GE.0.0) GOTO 1040
980   IF(IWRITE.EQ.0) GOTO 1030
      WRITE(JZ, 990) ITC, MC, FF
990   FORMAT(//,5X,'ITERATION',I4,I9,' CALLS OF CALFUN',5X,'F=',1PE20.
     +8)
      WRITE(JZ,1000) (X(I),I = 1,N)
1000  FORMAT(5X,'PARAMETERS',/,(1P5E20.8))
      IF(IWRITE.GE.0) THEN
1010    WRITE (6,1020) (F(I),I = 1,M)
1020    FORMAT (5X,'FUNCTIONS',/,(1P5E20.8))
      ENDIF
1030  IPC = IPP
      IF(IPS.EQ.2) GOTO 1120
1040  IF(ICONT.EQ.1) GOTO 420
      IF(CHANGE-1..GT.0.0) GOTO 1130
1060  IF(IWRITE) 1070, 1120, 1090
1070  WRITE(JZ, *) 'DFS001 FINAL VALUES OF VARIABLES'
      GOTO 1110
1090  WRITE (6, *) 'DFS001 FINAL VALUES OF FUNCTIONS AND VARIABLES'
1110  IPS = 2
      GOTO 980
1120  INVAR = 2
      GOTO 120
1130  ICONT = 1
      GOTO 420

C     THE STATEMENTS BELOW STORE THE VARIANCE CO-VARIANCE MATRIX IN THE
C     ARRAY W

1140  JJVAR = KINV - 1
      FF = FF/(M-N)
      DO JVAR = 1, N
        DO IVAR = JVAR, N
          JJVAR = JJVAR + 1
          JKVAR = (JVAR-1) * N + IVAR
          W(JKVAR) = W(JJVAR) * FF
        ENDDO
        IF(JVAR-1.GT.0) THEN
          LVAR = JVAR - 1
          MVAR = 0
          LKVAR = LVAR * N
          DO KVAR = 1, LVAR
            JKVAR = LKVAR + KVAR
            JNVAR = (KVAR-1) * N + JVAR - MVAR + KINV - 1
            W(JKVAR) = W(JNVAR) * FF
            MVAR = MVAR + KVAR
          ENDDO
        ENDIF
      ENDDO
      JVAR = N * N
      RETURN
      END



*****************************************************************
C
C  SUBROUTINE DFS002
C
C  THIS SUBROUTINE FINDS THE MINIMUM OF A FUNCTION IN ONE
C     DIMENSION. THE METHOD USED HAS BEEN DESCRIBED BY M.J.D. POWELL IN
C     'THE COMPUTER JOURNAL',VOL.7,NUMBER 2,JULY 1964,P.155.
C  THE METHOD ASSUMES THAT THE FUNCTION APPROACHES THE MINIMUM
C     QUADRATICALLY. ON FIRST ENTRY IT HAS A VALUE OF THE FUNCTION AT
C     ONE POINT. IT THEN CALCULATES TWO ADDITIONAL POINTS IN THE DIREC-
C     TION REQUESTED (RETURNING TO THE CALLING ROUTINE TO DO SO). THE
C     METHOD THEN PREDICTS THE MINIMUM OF THE  QUADRATIC THAT PASSES
C     THROUGH THE THREE DATA POINTS. IF THE MINIMUM IS BRACKETED BY THE
C     THREE DATA POINTS THEN MINIMUM PREDICTED IS USED. IF IT IS NOT
C     THEN FURTHER STEPS ARE TAKEN ALONG THE DIRECTION REQUESTED. A
C     TOTAL NUMBER OF STEPS MAXFUN IS ALLOWED.
C
C     ITEST IS A CONTROL INTEGER WHICH MUST BE SET TO 2 OR 3 ON INITIAL
C     ENTRY INTO THE ROUTINE AND TO 1 ON SUBSEQUENT ENTRIES DURING THE
C     SAME SEARCH.
C
C     X CONTAINS THE DISTANCE BEING MOVED ON INTERMEDIATE RETURNS TO
C     THE CALLING ROUTINE AND THE DISTANCE TO THE MINIMUM ON THE FINAL
C     RETURN F IS USED TO TRANSMIT THE FUNCTION VALUES.
C
C     MAXFUN IS THE TOTAL NUMBER OF FUNCTION EVALUATIONS ALLOWED , IF
C     MAXFUN IS EXCEEDED THEN ROUTINE RETURNS WITH THE NEAREST VALUE TO
C     THE MINIMUM IN X
C
C     ABSACC IS THE ABSOLUTE ACCURACY REQUIRED FOR THE MINIMUM
C
C     RELACC IS THE RELATIVE ACCURACY REQUIRED FOR THE MINIMUM
C
C     XSTEP  IS THE STEPPING OR INCREMENT DISTANCE IN THE DIRECTION OF
C     THE MINIMUM

      SUBROUTINE DFS002 (ITEST,X,F,MAXFUN,ABSACC,RELACC,XSTEP)


      IF(ITEST.EQ.1) GOTO 7

C     FOR THE FIRST ENTRY INTO DFS002 ITEST IS SET EQUAL TO 2 OR 3 WHICH
C     SETS UP THE INITIAL CONDITIONS FOR THE COMPUTATION. ITEST = 3
C     STORES THE INITIAL FUNCTION VALUE AND INCREMENTS X. ITEST = 2
C     CAUSES A RETURN TO THE CALLING ROUTINE REQUESTING A FIRST VALUE
C     OF THE FUNCTION.
C
C     AFTER SETTING THE INTIAL CONDITIONS ITEST IS RESET TO 1 UNTIL THE
C     MINIMUM HAS BEEN OBTAINED.ITEST  = 2 IS A SUCESSFUL CALCULATION
C     ITEST  =  3 IS ALSO A SUCESSFUL COMPLETION ,WHILE ITEST  =  4 IS A
C     RETURN THAT INDICATES THAT MAXFUN HAS BEEN EXCEEDED.

      IS = 6 - ITEST
      ITEST = 1
      IINC = 1
      XINC = XSTEP + XSTEP
      MC = IS - 3
      IF(MC) 9, 9, 6

C     INCREMENTS FUNCTION CALL COUNTER ,IF LESS THAN MAXFUN RETURN TO
C     CALLING ROUTINE FOR NEW FUNCTION EVALUATION

2     MC = MC + 1
      IF(MAXFUN-MC.GE.0) RETURN
3     ITEST = 4
4     X = DB
      F = FB
      IF(FB-FC.GT.0.0) THEN
        X = DC
        F = FC
      ENDIF
6     RETURN
7     GOTO (17,15,10,8), IS
8     IS = 3

C     WE COME TO THIS POINT AFTER CALCULATING THE FIRST FUNCTION
C     VALUE FOR THE ITEST = 2 INITIAL OPTION ,FROM THE NEXT STATEMENT (4)
C     ON ITEST  = 3 AND ITEST  = 2 ARE IDENTICAL.

C     STORES INTIAL FUNCTION AND POSITION VALUES

9     DC = X
      FC = F

C     INCREMENTS POSITION

      X = X + XSTEP
      GOTO 2
10    IF(FC-F) 12, 11, 13

C     COMES TO THIS POINT AFTER EVALUATING SECOND FUNCTION, GOES TO 10
C     IF FIRST  + SECOND FUNCTION VALUES ARE IDENTICAL, GOES TO 11 IF NEW
C     ONE IS LESS AND TO 9 IF NEW ONE IS LARGER

11    X = X + XINC

C     IN THIS SECTION NOTHING IS STORED AS NEW AND OLD FUNCTION VALUES
C     ARE IDENTICAL , BUT X IS INCREMENT AND A NEW FUNCTION VALUES IS
C     REQUESTED , IS REMAINS UNCHANGED SO WE COME BACK TO LABEL 7 AGAIN

      XINC = XINC + XINC
      GOTO 2
12    DB = X

C     IN THIS SECTION THE NEW FUNCTION VALUE IS LARGER THAN THE INITIAL
C     ONE ,AGAIN LARGER ONE IS PUT INTO FB , IT THEN CHANGES THE SIGN OF
C     XINC AND THEN INCREMENTS AGAIN ALONG X THE SAME AMOUNT AS FOR
C     SECTION 11.

      FB = F
      XINC = -XINC
      GOTO 14
13    DB = DC

C     IN THIS SECTION THE NEW FUNCTION VALUE IS LESS THAN THE INITIAL
C     ONE. IT STORES THE HIGHER IN FB AND THE LOWER IN FC. X IS INCRE-
C     MENTED AGAIN IN THE SAME DIRECTION
C     IS IS CHANGED TO 2 SO NEXT ENTRY BRINGS CALCULATION TO LABEL 6

      FB = FC
      DC = X
      FC = F
14    X = DC + DC - DB
      IS = 2
      GOTO 2
15    DA = DB

C     ARRIVES IN THIS SECTION HAVING OBTAINED THE THIRD FUNCTION VALUE
C     THE LARGEST OF THE FIRST TWO IS STORED IN FA AND THE SMALLEST IN
C     FB WITH THE THIRD IN FC

      DB = DC
      FA = FB
      FB = FC
16    DC = X
      FC = F
      GOTO 27

C     COMES TO THIS SECTION WHEN MINIMUM WAS WITHIN THE THREE FUNCTION
C     VALUES BUT NOT CLOSE ENOUGH TO ONE END. ALL THE NEXT DOES IS
C     DECIDE WHICH FUNCTION VALUES TO TAKE AND IN WHAT ORDER SO AS TO
C     MINIMIZE ROUNDING ERRORS

17    IF(FB-FC.GE.0.0) THEN
        IF(F-FB.GE.0.0) GOTO 16
19      FA = FB
        DA = DB
20      FB = F
        DB = X
        GOTO 27
      ENDIF
21    IF(FA-FC.GT.0.0) THEN
        XINC = FA
        FA = FC
        FC = XINC
        XINC = DA
        DA = DC
        DC = XINC
      ENDIF
23    XINC = DC
      IF((D-DB)*(D-DC).LT.0.0) GOTO 16
24    IF(F-FA.LT.0.0) THEN
        FC = FB
        DC = DB
        GOTO 20
      ENDIF
26    FA = F
      DA = X

C     TESTS TO SEE IF THE THIRD IS SMALLEST. IF SO, GOTO 29 AND CAL-
C     CULATE SECOND DERIVATIVE

C     IF THIRD FUNCTION IS EQUAL TO OR LARGER THAN THE SMALLEST,COME TO
C     THIS POINT AND SET XINC TO TWICE XSTEP AND SETS INTEGER IINC TO 2,
C     THIS INTEGER CONTROLS LATER COMPUTATIONS . STILL CALCULATES THE
C     SECOND DERIVATIVE ,PROVIDED THAT THE FUNCTION VALUES FB AND FC ARE
C     NOT EQUAL. IF THEY ARE WE GOTO LABEL 45 WHERE WE RECALCULATE WITH
C     A NEW X VALUE EQUAL TO THE MID POINT OF FB AND FC WHICH GIVES A
C     NEW FUNCTION VALUE

27    IF(FB-FC.LE.0.0) THEN
        IINC = 2     
        XINC = DC    
        IF(FB-FC.EQ.0.) GOTO 40
      ENDIF
29    IF((DA-DB).EQ.0) GOTO 3
      IF((DA-DC).EQ.0) GOTO 3
      D = (FA-FB)/(DA-DB) - (FA-FC)/(DA-DC)

C     TESTS SIGN OF SECOND DERIVATIVE IF NEGATIVE THERE IS A MINIMUM
C     AT THE CALCULATED VALUE OF D , IF POSITIVE WE HAVE FOUND A MAXIMUM
C     AND GOTO LABEL 33 TO RESTORE CALCULATION FOR A MINIMUM

C     HAVING CALCULATED A MINIMUM NOW TEST FOR IT OCCURING NEAR ENOUGH
C     TO THE LAST FUNCTION EVALUATION . IF IT IS WITHIN EITHER THE
C     ABSOLUTE OR RELATIVE ACCURACY THEN TERMINATE WITH ITEST  =  2
C     IF NOT WE GOTO LABEL 36.

      IF(D*(DB-DC).LE.0.0) GOTO 36
30    D = 0.50 * (DB + DC-(FB-FC)/D)
      IF(ABS(D-X)-ABS(ABSACC).LE.0.0.OR.
     +   ABS(D-X)-ABS(D*RELACC).LE.0.0) THEN
        ITEST = 2
        GOTO 4
      ENDIF
33    IS = 1

C     SET X TO PREDICTED VALUE OF MINIMUM ,IF THIS IS WITHIN THE RANGE
C     EXAMINED PREVIOUSLY GO BACK WITH IS = 1 AND CALCULATE NEW FUNCTION
C     VALUE. IF OUTSIDE PREVIOUS RANGE WE RECALCULATE THE THIRD FUNCTION
C     VALUE. IF LESS THEN 4*INCREMENT USE PREDICTED VALUE FOR X. IF
C     GREATER THEN USE 4*INCREMENT

      X = D
      IF((DA-DC)*(DC-D)) 2, 41, 34
34    IS = 2
      IF(IINC.EQ.2) GOTO 38
      IF(ABS(XINC)-ABS(DC-D)) 37, 2, 2
36    IS = 2
      IF(IINC.EQ.2) GOTO 39
37    X = DC
      GOTO 11
38    IF(ABS(XINC-X)-ABS(X-DC).GT.0.0) GOTO 2
39    X = 0.50 * (XINC + DC)
      IF((XINC-X)*(X-DC)) 41, 41, 2
40    X = 0.50 * (DB + DC)
      IF((DB-X)*(X-DC).GT.0.0) GOTO 2
41    ITEST = 3
      GOTO 4
      END


*****************************************************************
C      SUBROUTINE AMPL(IMOD)
C
C        AUTOMATICALLY ADJUSTS THE MAGNETIZATION CONTRASTS
C        (DENSITY CONTRASTS) OF THE BODIES IN ORDER TO IMPROVE
C        THE FIT OF THE CALCULATED PROFILE TO THE OBSERVED DATA.
C

      SUBROUTINE AMPL(IMOD)

      INCLUDE 'magrav.cmn'
 
      DIMENSION R1(20),R2(20)
      REAL*8 CALC, F, W
      COMMON /CALDAT/ CALC(21,200)
      COMMON /DFS/ F(200)
      COMMON /WORK/ W(4900)

      DATA MAXIT/1/

      IROUTE = 0
      NF=NFIELD(ITYPE)
      IF(NF.EQ.0)RETURN
      CALL CALCAN
      DO 3 I=1,NBODS
      R1(I)=RMMIN(I,ITYPE)
      R2(I)=RMMAX(I,ITYPE)
3     CONTINUE
      IF(IMOD.EQ.0)GO TO 6
      DO 5 I=1,NBODS
      RMMIN(I,ITYPE)=RHOMAG(I,ITYPE)
      RMMAX(I,ITYPE)=RHOMAG(I,ITYPE)
5     CONTINUE
      RMMIN(IMOD,ITYPE)=R1(IMOD)
      RMMAX(IMOD,ITYPE)=R2(IMOD)
6     CONTINUE
      NC=0
      DO 20 I=1,NBODS
      IF(ABS(RHOMAG(I,ITYPE)).LT.1.E-6) GOTO 20
      DO 10 J=1,NF
10    CALC(I,J)=CALC(I,J)/RHOMAG(I,ITYPE)
      IF(I.EQ.IMOD.OR.IMOD.EQ.0) THEN
      NC=NC+1
      XX(NC)=RHOMAG(I,ITYPE)
      E(NC)=FRAC
      ENDIF
20    CONTINUE
      IF(NC.EQ.0)RETURN
      IF(IOFS.EQ.0)GO TO 22
      NC=NC+1
      XX(NC)=OFFSET(ITYPE)
      E(NC)=FRAC
22    CONTINUE
      IERR=0
      NOBS = LEND(ITYPE) - LSTART(ITYPE) + 1
      CALL DFS001(NOBS,NC,XX,E,ESC,IWRITE,MAXIT,IERR,IROUTE,IMOD)
      IF(IERR.NE.0) THEN
      WRITE(JZ,*)' ERROR RETURN FROM DFS001 IN SUBROUTINE AMPL'
      GOTO 35
      ENDIF
      J=1
      DO 24 I=1,NC
      E(I)=SQRT(ABS(W(J)))
24    J=J+NC+1
      NC=0
      DO 25 I=1,NBODS
      IF(ABS(RHOMAG(I,ITYPE)).LT.1.E-6)GO TO 25
      IF(I.EQ.IMOD.OR.IMOD.EQ.0) THEN
      NC=NC+1
      RHOMAG(I,ITYPE)=XX(NC)
      ENDIF
25    CONTINUE
35    CONTINUE
      DO 26 I=1,NF
26    CALC(MAXCAL,I)=0.
      DO 30 I=1,NBODS
      DO 30 J=1,NF
      CALC(I,J)=CALC(I,J)*RHOMAG(I,ITYPE)
      CALC(MAXCAL,J)=CALC(MAXCAL,J)+CALC(I,J)
30    CONTINUE
      DO 40 I=1,NBODS
      RMMIN(I,ITYPE)=R1(I)
      RMMAX(I,ITYPE)=R2(I)
40    CONTINUE
      IF(IERR.NE.0) RETURN
      IF(IMOD.EQ.0) THEN
      DO I=1,NC
      WRITE(JZ,*) XX(I),E(I)
      ENDDO
      ELSE
      WRITE(JZ,*) XX(1),E(1)
      ENDIF
      IF(IOFS.EQ.0)RETURN
      OFFSET(ITYPE)=XX(NC+1)
      WRITE(JZ,*) XX(NC+1),E(NC+1)
      RETURN
      END
***********************************************************************
C    SUBROUTINE CALFUN
C
C        CALCULATES MODEL ANOMALY PROFILES
C

      SUBROUTINE CALFUN(M,N,IAMP,IERS,IROUTE,IMOD)

      INCLUDE 'magrav.cmn'
 
      REAL*8 CALC, F
      COMMON /CALDAT/ CALC(21,200)
      COMMON /DFS/ F(200)

      NF=NFIELD(ITYPE)
      ISTART = LSTART(ITYPE)
      IEND = LEND(ITYPE)
      DO 5 I=1,NF
      F(I)=0.
5     CALC(MAXCAL,I)=0.
C     IF IROUTE = 0, CALFUN WAS CALLED FROM DFS001 AND AMPL
C               = 1, CALFUN WAS CALLED FROM DFS001 AND AMOVE
      IF(IROUTE.EQ.1) GOTO 30
      NC=0
      DO 20 I=1,NBODS
      IF(ABS(RHOMAG(I,ITYPE)).LT.1.E-6)GO TO 20
      IF(I.EQ.IMOD.OR.IMOD.EQ.0) THEN
      NC=NC+1
      IF(IAMP.EQ.1)GO TO 11
      IF(XX(NC).LT.RMMIN(I,ITYPE))XX(NC)=RMMIN(I,ITYPE)
      IF(XX(NC).GT.RMMAX(I,ITYPE))XX(NC)=RMMAX(I,ITYPE)
11    CONTINUE
      RHOMAG(I,ITYPE)=XX(NC)
      ENDIF
      DO 15 J=1,NF
      CALC(MAXCAL,J)=CALC(MAXCAL,J)+CALC(I,J)*RHOMAG(I,ITYPE)
15    CONTINUE
20    CONTINUE
      IF(IWFLAG.EQ.1)GO TO 70
      IGO=3
      CALL ISOSTA(IGO,IERS)
      IF(IERS.EQ.1)RETURN
      DO 27 I=ISTART,IEND
27    F(I)=WD(I)
      GO TO 70
30    DO 40 I=1,3
      IF(IB(I).EQ.0)GO TO 400
      IF(NBR.NE.1)GO TO 31
      X(IB(I),NP(I))=XX(1)
      IF(NP(I).EQ.1)X(IB(I),NPTS(IB(I)))=XX(1)
      GO TO 39
31    IF(NBR.NE.2)GO TO 32
      Z(IB(I),NP(I))=ABS(XX(1))
      IF(NP(I).EQ.1)Z(IB(I),NPTS(IB(I)))=ABS(XX(1))
      GO TO 39
32    X(IB(I),NP(I))=XX(1)
      Z(IB(I),NP(I))=ABS(XX(2))
      IF(NP(I).EQ.1)X(IB(I),NPTS(IB(I)))=XX(1)
      IF(NP(I).EQ.1)Z(IB(I),NPTS(IB(I)))=ABS(XX(2))
39    IANOM(IB(I))=0
40    CONTINUE
400   CONTINUE
      DO 50 I=4,6
      IF(IB(I).EQ.0)GO TO 500
      IF(NBR.NE.1)GO TO 41
      X(IB(I),NP(I))=XX(2)
      IF(NP(I).EQ.1)X(IB(I),NPTS(IB(I)))=XX(2)
      GO TO 49
41    IF(NBR.NE.2)GO TO 42
      Z(IB(I),NP(I))=ABS(XX(2))
      IF(NP(I).EQ.1)Z(IB(I),NPTS(IB(I)))=ABS(XX(2))
      GO TO 49
42    X(IB(I),NP(I))=XX(3)
      Z(IB(I),NP(I))=ABS(XX(4))
      IF(NP(I).EQ.1)X(IB(I),NPTS(IB(I)))=XX(3)
      IF(NP(I).EQ.1)Z(IB(I),NPTS(IB(I)))=ABS(XX(4))
49    IANOM(IB(I))=0
50    CONTINUE
500   CONTINUE
      IF(IWFLAG.EQ.1)GO TO 65
      IGO=3
      CALL ISOSTA(IGO,IERS)
      IF(IERS.EQ.1)RETURN
      DO 63 I=ISTART,IEND
63    F(I)=WD(I)
65    CONTINUE
      IF(IVA.LT.0)GO TO 67
      NC=NTOT
      IF(NBR.EQ.3)NC=2*NTOT
      DO 66 I=1,NBV
      NC=NC+1
      RHOMAG(IVB(I),ITYPE)=XX(NC)
      IF(IAMP.EQ.1)GO TO 66
      K=IVB(I)
      L=ITYPE
      IF(RHOMAG(K,L).LT.RMMIN(K,L))RHOMAG(K,L)=RMMIN(K,L)
      IF(RHOMAG(K,L).GT.RMMAX(K,L))RHOMAG(K,L)=RMMAX(K,L)
66    CONTINUE
67    CONTINUE
      DO 60 IBOD=1,NBODS
      IF(NPTS(IBOD).LE.0)GO TO 60
      IF(IANOM(IBOD).LT.0)GO TO 55
      CALL GMAG(IBOD)
      IANOM(IBOD)=-1
55    CONTINUE
      DO 56 I=1,NF
      CALC(MAXCAL,I)=CALC(MAXCAL,I)+CALC(IBOD,I)
56    CONTINUE
60    CONTINUE
      NC=NTOT+NBV
      IF(NBR.EQ.3)NC=2*NTOT+NBV
70    CONTINUE
      ZDUM=OFFSET(ITYPE)
      IF(IOFS.EQ.1)ZDUM=XX(NC+1)
      ICO=1
      IF(IWFLAG.EQ.1.OR.ITYPE.EQ.2) THEN
      DO 80 I=ISTART,IEND
      F(ICO) = (ABS(OBS(ITYPE,I)-CALC(MAXCAL,I)-ZDUM))*OBS(ITYPE+3,I)
80    ICO=ICO+1
      ELSE
      CALL RMSR(RRR,1)
      IF(IWRITE.NE.0) WRITE(JZ,*)'WFACT, WW ',WFACT,WW
      WWW=WFACT*WW
      DO 81 I=ISTART,IEND
      F(ICO) = (ABS(OBS(ITYPE,I)-CALC(MAXCAL,I)-ZDUM)*WWW+
     *ABS(F(I)))*OBS(ITYPE+3,I)
81    ICO=ICO+1
      ENDIF
      IF(IAMP.EQ.1)RETURN
      IF(IROUTE.EQ.0)RETURN
      IF(NTOT.NE.1)RETURN
      ICO=ICO-1
      Z1=Z(IB(1),NP(1))
      ZDUM=Z1*Z1
      IF(Z1.EQ.0.)RETURN
      X1=X(IB(1),NP(1))-COORD(ISTART)
      XINC=SPACE(ITYPE)
      IF(ITYPE.EQ.2)GO TO 95
      DO 90 I=1,ICO
      F(I)=F(I)*ABS(Z1/((ZDUM+X1*X1)**.5))
90    X1=X1-XINC
      GO TO 99
95    CONTINUE
      DO 98 I=1,ICO
      F(I)=F(I)*Z1/(ZDUM+X1*X1)
98    X1=X1-XINC
99    CONTINUE
      RETURN
      END


*********************************************************************
C   SUBROUTINE AMOVE
C
C  MOVES THE BODY POINT(S) DESIGNATED WITH THE CURSOR SO AS
C  TO IMPROVE THE FIT OF THE CALCULATED ANOMALY CURVE TO
C  THE OBSERVED DATA IN A LEAST SQUARES SENSE

      SUBROUTINE AMOVE(X1,X2,X3,X4,NPC)

      INCLUDE 'magrav.cmn'

      REAL*8 F,W 
      COMMON /DFS/ F(200)
      COMMON /WORK/ W(4900)
      DATA MAXIT/3/

      IROUTE = 1
      IMOD = 0
      NF=NFIELD(ITYPE)
      IF(NF.EQ.0)RETURN
      N1=0
      N2=0
      NBV=0
      DO 1 I=1,6
      IB(I)=0
      NP(I)=0
      IVB(I)=0
1     CONTINUE
      DO 2 I=1,11
2     XX(I)=0.
      DO 20 NUMBER=1,3
      KBR=1
      CALL CHECK(NUMBER,X1,X2,IBOD,NPT,IRET,KBR)
      IF(IRET)10,5,900
5     IB(NUMBER)=IBOD
      NP(NUMBER)=NPT
      N1=1
10    IF(NPC.EQ.2)GO TO 20
      CALL CHECK(NUMBER,X3,X4,IBOD,NPT,IRET,KBR)
      IF(IRET)20,15,900
15    IB(NUMBER+3)=IBOD
      NP(NUMBER+3)=NPT
      N2=1
20    CONTINUE
      NTOT=N1+N2
      IF(NTOT.EQ.0)GO TO 900
      IF(NBR.NE.1)GO TO 21
      XX(1)=X1
      XX(2)=X3
      N3=NTOT
      GO TO 29
21    IF(NBR.NE.2)GO TO 22
      XX(1)=X2
      XX(2)=X4
      N3=NTOT
      GO TO 29
22    XX(1)=X1
      XX(2)=X2
      XX(3)=X3
      XX(4)=X4
      N3=NTOT*2
29    N5=N3
      N4=N3
      IF(IVA.LT.0)GO TO 70
      IC=0
      DO 65 I=1,6
      IF(IB(I).EQ.0)GO TO 65
      IF(ABS(RHOMAG(IB(I),ITYPE)).LT.1.E-6)GO TO 65
      IC=IC+1
      IVB(IC)=IB(I)
65    CONTINUE
      DO 67 I=1,5
      L2=I+1
      DO 67 J=L2,6
      IF(IVB(I).NE.IVB(J))GO TO 67
      IF(J.EQ.6)GO TO 64
      DO 66 K=J,5
66    IVB(K)=IVB(K+1)
64    IVB(6)=0.
67    CONTINUE
      DO 63 I=1,6
      IF(IVB(I).NE.0)NBV=NBV+1
63    CONTINUE
      DO 68 I=1,NBV
      N4=N4+1
      XX(N4)=RHOMAG(IVB(I),ITYPE)
68    CONTINUE
      N5=N4
70    CONTINUE
      IF(IOFS.EQ.0)GO TO 28
      N5=N4+1
      XX(N5)=OFFSET(ITYPE)
28    CONTINUE
      DO 30 I=1,N5
30    E(I)=FRAC
      IERR=0
      NOBS = LEND(ITYPE) - LSTART(ITYPE) + 1
      CALL DFS001(NOBS,N5,XX,E,ESC,IWRITE,MAXIT,IERR,IROUTE,IMOD)
      IF(IERR.NE.0) THEN
      WRITE(JZ,*)' ERROR RETURN FROM SUBROUTINE DFS001 IN AMOVE'
      RETURN
      ENDIF
      J=1
      DO 24 I=1,N5
      E(I)=SQRT(ABS(W(J)))
24    J=J+N5+1
      DO I=1,N5
      WRITE(JZ,*) XX(I),E(I)
      ENDDO
      IF(NBR.NE.1)GO TO 31
      X1=XX(1)
      IF(NPC.EQ.4)X3=XX(2)
      GO TO 37
31    IF(NBR.NE.2)GO TO 32
      X2=XX(1)
      IF(NPC.EQ.4)X4=XX(2)
      GO TO 37
32    X1=XX(1)
      X2=XX(2)
      IF(NPC.EQ.2)GO TO 37
      X3=XX(3)
      X4=XX(4)
37    CONTINUE
      IF(X1.GT.XMAX)X1=XMAX
      IF(X1.LT.0.)X1=0.
      IF(X2.LT.0.)X2=0.
      IF(X2.GT.ZMAX)X2=ZMAX
      IF(NPC.EQ.2)GO TO 39
      IF(X3.GT.XMAX)X3=XMAX
      IF(X3.LT.0.)X3=0.
      IF(X4.LT.0.)X4=0.
      IF(X4.GT.ZMAX)X4=ZMAX
39    CONTINUE
      DO 40 J=1,3
      IBL=IB(J)
      NPL=NP(J)
      IF(IBL.EQ.0)GO TO 35
      X(IBL,NPL)=X1
      Z(IBL,NPL)=X2
      IANOM(IBL)=0
      IF(NPL.NE.1)GO TO 34
      X(IBL,NPTS(IBL))=X(IBL,1)
      Z(IBL,NPTS(IBL))=Z(IBL,1)
34    CONTINUE
      CALL PLBOD(IBL)
35    IF(NPC.EQ.2)GO TO 40
      IBL=IB(J+3)
      NPL=NP(J+3)
      IF(IBL.EQ.0)GO TO 40
      X(IBL,NPL)=X3
      Z(IBL,NPL)=X4
      IF(NPL.NE.1)GO TO 38
      X(IBL,NPTS(IBL))=X(IBL,1)
      Z(IBL,NPTS(IBL))=Z(IBL,1)
38    CONTINUE
      IANOM(IBL)=0
      CALL PLBOD(IBL)
40    CONTINUE
      IF(IVA.LT.0)GO TO 80
      DO 79 I=1,NBV
      RHOMAG(IVB(I),ITYPE)=XX(N3+I)
79    CONTINUE
80    CONTINUE
      IF(IOFS.EQ.0)GO TO 50
      OFFSET(ITYPE)=XX(N5)
50    CONTINUE
      IBOD=0
      KALK=1
      CALL PLANOM(IBOD,KALK)
      CALL SAV(NBACK)
900   CONTINUE
      RETURN
      END


********************************************************************
C   SUBROUTINE TO READ IN ALL PARAMETERS FOR MODELS FROM FILE LUN

      SUBROUTINE BUFFIN(LUN)

      INCLUDE 'magrav.cmn'

      READ(LUN, ERR=10) NAME, KOMMNT, ISCOP, MODE, 
     + ITYPE, XMAX, ZMAX, XMINS(1), XMINS(2), XMAXS(1), XMAXS(2),
     + NFIELD(1), NFIELD(2), SPACE(1), SPACE(2), ((OBS(J,I),J=1,5),I=1,
     + 200), (LSTART(I), LEND(I), I=1,2), ANOMAX(1), ANOMAX(2), 
     + ANOMIN(1), ANOMIN(2), IDIFF, DIFMIN(1), DIFMIN(2), DIFMAX(1), 
     + DIFMAX(2), SKXMAX, SKXMIN, SKZMAX, SKXMIN, ISUPG, GX, GZ, NBODS, 
     + (NPTS(I),I=1,20), XTON, FDEC, FINCL, (BINCL(I),I=1,20),(BDEC(I),
     + I=1,20), ((BDY(J,I),J=1,20),I=1,2), ((X(J,I), J=1,20), I=1,25),
     + ((Z(J,I), J=1,20), I=1,25), ((RHOMAG(J,I), J=1,20), I=1,2),
     + ((RMMIN(J,I), J=1,20), I=1,2), ((RMMAX(J,I), J=1,20), I=1,2),
     + ZCON(1), ZCON(2), ICROSS, OFFSET(1),OFFSET(2),(IANOM(I),I=1,20),
     + IWFLAG, WW, PMAX, PMIN, (YANG(I),I=1,20), BKDEN 

      LUN=-1
10    RETURN
      END



**************************************************************************

      SUBROUTINE BUFFOUT(LUN)

      INCLUDE 'magrav.cmn'

      WRITE(LUN) NAME, KOMMNT, ISCOPE, MODE, 
     + ITYPE, XMAX, ZMAX, XMINS(1), XMINS(2), XMAXS(1), XMAXS(2),
     + NFIELD(1), NFIELD(2), SPACE(1), SPACE(2), ((OBS(J,I),J=1,5),I=1,
     + 200), (LSTART(I), LEND(I), I=1,2), ANOMAX(1), ANOMAX(2), 
     + ANOMIN(1), ANOMIN(2), IDIFF, DIFMIN(1), DIFMIN(2), DIFMAX(1), 
     + DIFMAX(2), SKXMAX, SKXMIN, SKZMAX, SKXMIN, ISUPG, GX, GZ, NBODS, 
     + (NPTS(I),I=1,20), XTON, FDEC, FINCL, (BINCL(I),I=1,20), (BDEC(I),
     + I=1,20), ((BDY(J,I),J=1,20),I=1,2), ((X(J,I), J=1,20), I=1,25),
     + ((Z(J,I), J=1,20), I=1,25), ((RHOMAG(J,I), J=1,20), I=1,2),
     + ((RMMIN(J,I), J=1,20), I=1,2), ((RMMAX(J,I), J=1,20), I=1,2),
     + ZCON(1), ZCON(2), ICROSS, OFFSET(1), OFFSET(2),(IANOM(I),I=1,20),
     + IWFLAG, WW, PMAX, PMIN, (YANG(I),I=1,20), BKDEN 
 
      RETURN
      END


**************************************************************************

      SUBROUTINE MODREAD(LUN,IREC)

      CHARACTER INAME*10

      INCLUDE 'magrav.cmn'
 
      READ(LUN, REC=IREC) INAME, KOMMNT, ISCOP, MODE, 
     + ITYPE, XMAX, ZMAX, XMINS(1), XMINS(2), XMAXS(1), XMAXS(2),
     + NFIELD(1), NFIELD(2), SPACE(1), SPACE(2), ((OBS(J,I),J=1,5),I=1,
     + 200), (LSTART(I), LEND(I), I=1,2), ANOMAX(1), ANOMAX(2), 
     + ANOMIN(1), ANOMIN(2), IDIFF, DIFMIN(1), DIFMIN(2), DIFMAX(1), 
     + DIFMAX(2), SKXMAX, SKXMIN, SKZMAX, SKXMIN, ISUPG, GX, GZ, NBODS, 
     + (NPTS(I),I=1,20), XTON, FDEC, FINCL, (BINCL(I),I=1,20), (BDEC(I)
     + ,I=1,20), ((BDY(J,I),J=1,20),I=1,2), ((X(J,I), J=1,20), I=1,25),
     + ((Z(J,I), J=1,20), I=1,25), ((RHOMAG(J,I), J=1,20), I=1,2),
     + ((RMMIN(J,I), J=1,20), I=1,2), ((RMMAX(J,I), J=1,20), I=1,2),
     + ZCON(1), ZCON(2), ICROSS, OFFSET(1), OFFSET(2),(IANOM(I),I=1,20),
     + IWFLAG, WW, PMAX, PMIN, (YANG(I),I=1,20), BKDEN 

      RETURN
      END



**************************************************************************

      SUBROUTINE MODWRITE(LUN,INAME,IREC)

      CHARACTER INAME*10

      INCLUDE 'magrav.cmn'

      WRITE(LUN, REC = IREC) INAME, KOMMNT,ISCOPE, MODE, ITYPE, XMAX,
     + ZMAX, XMINS(1), XMINS(2), XMAXS(1), XMAXS(2), NFIELD(1),
     + NFIELD(2), SPACE(1), SPACE(2), ((OBS(J,I),J=1,5),I=1,200),
     + (LSTART(I), LEND(I), I=1,2), ANOMAX(1), ANOMAX(2), ANOMIN(1),
     + ANOMIN(2), IDIFF, DIFMIN(1), DIFMIN(2), DIFMAX(1), DIFMAX(2),
     + SKXMAX, SKXMIN, SKZMAX, SKXMIN, ISUPG, GX, GZ, NBODS,(NPTS(I),
     + I=1,20), XTON, FDEC, FINCL, (BINCL(I),I=1,20), (BDEC(I),I=1,20),
     + ((BDY(J,I),J=1,20),I=1,2), ((X(J,I), J=1,20), I=1,25),
     + ((Z(J,I), J=1,20),I=1,25), ((RHOMAG(J,I), J=1,20), I=1,2),
     + ((RMMIN(J,I), J=1,20),I=1,2), ((RMMAX(J,I), J=1,20), I=1,2),
     + ZCON(1), ZCON(2), ICROSS, OFFSET(1), OFFSET(2), (IANOM(I),I=1,20)
     + , IWFLAG, WW, PMAX, PMIN, (YANG(I),I=1,20), BKDEN 
      RETURN
      END

************************************************************************
C   SUBROUTINE DELMOD
C
C   DELETES A FILE FROM THE MODEL FILE

      SUBROUTINE DELMOD(IRET,LUN,INAME,MODDEL,MODFILE)

      CHARACTER*50 MODFILE, SCRATCH
      CHARACTER*10 LIST(20), COMM(8), INAME
      REAL CRP2307(2307)

      INCLUDE 'magrav.cmn'
 
      COMMON /LIST/ ILIST, LIST
C
      WRITE(JZ,711)
711   FORMAT(' ENTER NAME OF MODEL TO BE DELETED:',$)
      READ(*,710) INAME
710   FORMAT(A10)
      DO 705 I = 1, ILIST
      IF(INAME.EQ.LIST(I)) THEN
      IREC=I
      GOTO 712
      ENDIF
705   CONTINUE
      IRET = 1
      RETURN
712   IRET = -1
      SCRATCH='SCR.mdl'
      OPEN(MODDEL,FILE=SCRATCH,ACCESS='DIRECT',STATUS='NEW',
     +    FORM='UNFORMATTED',RECL=9318)
      IC=1
      DO 720 J=1,ILIST
      READ(LUN,REC=J) NAME,(COMM(I),I=1,8),CRP2307
      IF(J.EQ.IREC) GOTO 720
      WRITE(MODDEL,REC=IC) NAME,(COMM(I),I=1,8),CRP2307
      IC=IC+1
720   CONTINUE
      REWIND MODDEL
      CLOSE (LUN, STATUS='DELETE')
      OPEN(LUN,FILE=MODFILE,ACCESS='DIRECT',STATUS='NEW',
     +    FORM='UNFORMATTED',RECL=9318)
      IFIL=1
      DO 725 I = 1, ILIST
      IF(INAME.EQ.LIST(I)) GOTO 725
      LIST(IFIL)=LIST(I)
      IFIL=IFIL+1
725   CONTINUE
      ILIST=ILIST-1
      DO 730 J = 1, ILIST
      READ(MODDEL,REC=J) NAME,(COMM(I),I=1,8),CRP2307
      WRITE(LUN,REC=J) NAME,(COMM(I),I=1,8),CRP2307
730   CONTINUE
      CLOSE (MODDEL, STATUS='DELETE')
      RETURN
      END
      INTEGER FUNCTION GIPOLYX(N,X,Y,X0,Y0)
C
C     THIS FUNCTION DETERMINES IF A POINT LIES WITHIN AN ARBITRARY
C     POLYGON. ITS VALUE IS 0 IF THE POINT IS OUTSIDE THE POLYGON
C                           1 IF IT IS INSIDE THE POLYGON
C     N - NUMBER OF VERTICES OF THE POLYGON
C     X,Y - ARRAYS OF X AND Y VALUES DEFINING THE VERTICES.
C     X0,Y0 - COORDINATES OF THE GIVEN POINT
C
      DIMENSION X(1),Y(1)
      GIPOLYX=0
      IF(Y0.GT.Y(N)) THEN
        IF(Y0.GT.Y(1)) GOTO 13
        GOTO 12
      ELSE
        IF(Y0.GT.Y(1)) GOTO 12
        GOTO 13
      ENDIF
12    S=X0-X(N)-(Y0-Y(N))*(X(1)-X(N))/(Y(1)-Y(N))
      IF(S.LT.0.) THEN
        GIPOLYX=1
      ENDIF
13      N1=N-1
      DO I=1,N1
      IF(Y0.GT.Y(I)) THEN
        IF(Y0.GT.Y(I+1)) GOTO 6
        GOTO 9
      ELSE
        IF(Y0.GT.Y(I+1)) GOTO 9
        GOTO 6
      ENDIF
9     S=X0-X(I)-(Y0-Y(I))*(X(I+1)-X(I))/(Y(I+1)-Y(I))
      IF(S.LT.0.) THEN
        GIPOLYX=1-GIPOLYX
      ENDIF
6     ENDDO
      RETURN
      END
      SUBROUTINE TWINDO(IXL,IXH,IYL,IYH)
      COMMON /VPST/ WIDMM,HTMM,NXPIX,NYPIX,ITERM,CHEI,YOR
      ZNX=NXPIX
      ZNY=NYPIX
      XMIN=IXL/ZNX*WIDMM
      XMAX=IXH/ZNX*WIDMM
      XSIZE=XMAX-XMIN
      YMIN=IYL/ZNY*HTMM+YOR
      YMAX=IYH/ZNY*HTMM+YOR
      YSIZE=YMAX-YMIN
      CALL GVPORT(XMIN,YMIN,XSIZE,YSIZE)
      CALL GWBOX(XSIZE,YSIZE,0.)
      RETURN
      END
      SUBROUTINE DWINDO(XL,XH,YL,YH)
      CALL GLIMIT(XL,XH,YL,YH,0.,0.)
      CALL GSCALE
      RETURN
      END
      SUBROUTINE POINTA(X,Y)
      CALL GVECT(X,Y,0)
      RETURN
      END
      SUBROUTINE DASHA(X,Y,L)
      CALL GDASH(L)
      CALL GVECT(X,Y,1)
      RETURN
      END
      SUBROUTINE DRAWA(X,Y)
      CALL GVECT(X,Y,1)
      RETURN
      END
      SUBROUTINE VCURSR(CHR,XC,YC)
      CHARACTER*1 CHR
      CALL RGLOC(XC,YC,ICHR,KCLASS)
      IF(ICHR.GE.97.AND.ICHR.LE.122) THEN
       ICHR=ICHR-32
      ENDIF
      CHR=CHAR(ICHR)
      RETURN
      END
      SUBROUTINE MOVABS(X,Y)
      CALL GCURMV(X,Y)
      CALL GVECT(XC,YC,0)
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE IHELP(IWHAT)
      WRITE(*,*)
      GOTO (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,
     +24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44),
     +IWHAT
C
1     WRITE(*,*)'SCAL - SCALes'
      WRITE(*,*)'Sets gravity or magnetic scales for plotting'
      WRITE(*,*)'MAGRAV prints out the active mode - GRAV or MAGN'
      WRITE(*,*)'MAGRAV displays current scale values'
      WRITE(*,*)'To change values Enter A - K, or X to exit'
      RETURN
2     WRITE(*,*)'BODY - to enter or delete a BODY'
      RETURN
3     WRITE(*,*)'POIN - to alter the coordinates of a body POINt'
      RETURN
4     WRITE(*,*)'PARA - to enter PARAmeters for body ibod'
      WRITE(*,*)'if ibod=0, the parameter value is added to all bodies'
      RETURN
5     WRITE(*,*)'CONT - CONTrast'
      WRITE(*,*)'Automatically adjusts the magnetization or density'
      WRITE(*,*)'contrasts of body(ibod) in order to improve the fit'
      WRITE(*,*)'of the calculated profile to the observed data'
      WRITE(*,*)'ibod = 0 for all bodies'
      RETURN
6     WRITE(*,*)'STAT - STATistics of current array sizes'
      WRITE(*,*)'Displays the array sizes limiting the maximum'
      WRITE(*,*)'1) number of bodies,'
      WRITE(*,*)'2) points per body, and'
      WRITE(*,*)'3) number of observations'
      WRITE(*,*)'The values are stored in MAXBOD, MAXNPT and MAXOBS'
      RETURN
7     WRITE(*,*)'DELM - DELete Model'
      WRITE(*,*)'Deletes from the models file, *.mdl, the model'
      WRITE(*,*)'designated using the text command NAME.'
      RETURN
8     WRITE(*,*)'READ - READs the model file designated using the'
      WRITE(*,*)'text command NAME from the models file, *.mdl.'
      RETURN
9     WRITE(*,*)'WRIT - WRITes a model file'
      WRITE(*,*)'Writes the model designated using the text command'
      WRITE(*,*)'NAME to the models file.'
      RETURN
10    WRITE(*,*)'CANO - Calculate ANOmaly'
      WRITE(*,*)'Calculates Anomaly of any body(ies) changed since'
      WRITE(*,*)'the last calculation and stores total for all bodies'
      WRITE(*,*)'in the array CALC(MAXCAL,I), where I designates the'
      WRITE(*,*)'number of the calculated point'
      RETURN
11    WRITE(*,*)'TANO - Type Anomaly'
      WRITE(*,*)'Lists all or part of the calculated anomaly in the'
      WRITE(*,*)'text window.'
      RETURN
12    WRITE(*,*)'OBSE - permits entry of observed gravity/topographic'
      WRITE(*,*)'or magnetic data from either the keyboard or from a'
      WRITE(*,*)'disk file provided by the User.'
      RETURN
13    WRITE(*,*)'TOBS - Type OBServations'
      WRITE(*,*)'Lists all or part of the observed data in the'
      WRITE(*,*)'text window.'
      RETURN
14    WRITE(*,*)'COMM - COMMents'
      WRITE(*,*)'To store profile name and geographic coordinates of'
      WRITE(*,*)'the first observed data point. Array accepts only'
      WRITE(*,*)'80 characters.'
      RETURN
15    WRITE(*,*)'TCOM - Type COMments'
      WRITE(*,*)'Outputs contents of KOMMNT array.'
      RETURN
16    WRITE(*,*)'NAME - model NAME'
      WRITE(*,*)'A model must be named before it can be written to a'
      WRITE(*,*)'models file.'
      RETURN
17    WRITE(*,*)'TMOD - Type MODels'
      WRITE(*,*)'Displays the model names included in a models file.'
      RETURN
18    WRITE(*,*)'INSE - INSErt a body point'
      WRITE(*,*)'MAGRAV requests the body and point number after which'
      WRITE(*,*)'a new point is to be inserted. The program requests'
      WRITE(*,*)'the X and Z coordinates of the new point.'
      RETURN
19    WRITE(*,*)'DELE - DELEte a body point'
      WRITE(*,*)'MAGRAV requests the body and point numbers of the'
      WRITE(*,*)'to be deleted.'
      RETURN
20    WRITE(*,*)'EXIT - END MAGRAV SESSION'
      RETURN
21    WRITE(*,*)'GRAP - switch to GRAPhics window'
      RETURN
22    WRITE(*,*)'RECO - RECOver'
      WRITE(*,*)'Permits User to recover backedup models saved in a'
      WRITE(*,*)'temporary scratch file, or to recover models after a'
      WRITE(*,*)'program crash. Models are automatically backedup in'
      WRITE(*,*)'the scratch file when 3 (NBACK in code) changes are'
      WRITE(*,*)'made to a given model. The scratch file is purged if'
      WRITE(*,*)'the program termination is normal, otherwise it is'
      WRITE(*,*)'saved.' 
      RETURN
23    WRITE(*,*)'DUMP'
      WRITE(*,*)'MAGRAV offers the following choices'
      WRITE(*,*)' 1) List model parameters on screen,'
      WRITE(*,*)' 2) Create the disk file *.DMP, for input to WORD'
      WRITE(*,*)' 3) Reads the disk file *.DMP, to re-enter a model'
      WRITE(*,*)'       into a models file'
      RETURN
24    WRITE(*,*)'REST - RESeT'
      WRITE(*,*)'To reset the default values of ESC(100.) and'
      WRITE(*,*)'FRAC(.001) used by Subroutine DFS001.  ESC is a'
      WRITE(*,*)'number whose value limits the change in a variable'
      WRITE(*,*)'in one iteration of DFS001. FRAC is used as a'
      WRITE(*,*)'fractional accuracy in the calculation of the first'
      WRITE(*,*)'derivative of the gravity(magnetic) profile with'
      WRITE(*,*)'to a given body parameter.'
      RETURN 
25    WRITE(*,*)'TPAR - Type body PARameters'
      WRITE(*,*)'Lists body densities or magnetizations on screen.'
      RETURN
26    WRITE(*,*)'MAGN - MAGNetics mode'
      RETURN
27    WRITE(*,*)'GRAV - GRAVity mode'
      RETURN
28    WRITE(*,*)'CROS - CROSses'
      WRITE(*,*)'Toggles body crosses on and off.'
      RETURN
29    WRITE(*,*)'ZOOM'
      WRITE(*,*)'Enter the minimum and maximum values of X and Z in'
      WRITE(*,*)'km to define the SKETCH window.'
      RETURN
30    WRITE(*,*)'LIMI- LIMIts'
      WRITE(*,*)'Determines the maximum and minimum values along the'
      WRITE(*,*)'observed data profile.'
      RETURN
31    WRITE(*,*)'SUPP - SUPPresses graphics'
      WRITE(*,*)'1) Disables graphics options, and/or'
      WRITE(*,*)'2) Switches grid on and off.'
      WRITE(*,*)'Grid spacings are changed with the SCAL option,'
      WRITE(*,*)'Letters G and H.'
      RETURN
32    WRITE(*,*)'TBOD - Type BODy parameters'
      WRITE(*,*)'Lists on screen the parameters of body (IBOD), or'
      WRITE(*,*)'all bodies if IBOD = 0.'
      RETURN
33    WRITE(*,*)'TSCA - Type SCAle'
      WRITE(*,*)'Lists plot parameters on screen.'
      RETURN
34    WRITE(*,*)'OFFS - OFFSet'
      WRITE(*,*)'Determines the base level shift required to move'
      WRITE(*,*)'the calculated profile to the observed profile.'
      WRITE(*,*)'The offset value is the sum of the differences'
      WRITE(*,*)'between the calculated and observed profiles'
      WRITE(*,*)'divided by the number of data points.'
      WRITE(*,*)'NOTE - an option to use arbitrary values of offset'
      WRITE(*,*)'is available using the text option command RMSR.' 
      RETURN
35    WRITE(*,*)'DIFF - DIFFerences'
      WRITE(*,*)'Displays the differences between the calculated and'
      WRITE(*,*)'observed profiles.'
      RETURN
36    WRITE(*,*)'HELP'
      WRITE(*,*)'Opens the text help file. To exit and return to the'
      WRITE(*,*)'OPTION mode hit the X key.'
      RETURN
37    WRITE(*,*)'FILL'
      WRITE(*,*)'Toggles between coloured and open bodies.'
      RETURN
38    WRITE(*,*)'STOP'
      WRITE(*,*)'Terminates MAGRAV - same as EXIT and QUIT.'
      RETURN
39    WRITE(*,*)'QUIT'
      WRITE(*,*)'Terminates MAGRAV - same as STOP and QUIT.'
      RETURN
40    WRITE(*,*)'HALF'
      WRITE(*,*)'Used to change the strike lengths, YMIN and YMAX'
      WRITE(*,*)'of body designated by its body number, and the Y axis' 
      WRITE(*,*)'to body strike direction. Enter / to retain current'
      WRITE(*,*)'values.'
      RETURN
41    WRITE(*,*)'ISOS - ISOStasy'
      WRITE(*,*)'The weight of a one-square-meter vertical column'
      WRITE(*,*)'of rock extending between the upper and lower surfaces'
      WRITE(*,*)'of the model is calculated at each data position. The'
      WRITE(*,*)'upper and lower surfaces of the model are assumed to'
      WRITE(*,*)'be at mean sea level and at the level of isostatic'
      WRITE(*,*)'compensation respectively. For each column the'
      WRITE(*,*)'intersection intervals through each model body are'
      WRITE(*,*)'determined and are multiplied by the appropriate body'
      WRITE(*,*)'density. The weight of each colunm is determined by'
      WRITE(*,*)'summing these interval column weights. Changes in'
      WRITE(*,*)'topography are accounted for by extending each column'
      WRITE(*,*)'upward a distance equal to its elevation above sea' 
      WRITE(*,*)'level, and then multiplying by the density contrast'
      WRITE(*,*)'of the appropriate model body. The column weights are'
      WRITE(*,*)'expressed in units of gigagrams per square meter. The'
      WRITE(*,*)'mean column weight is calculated, and the differences'
      WRITE(*,*)'between this value and the weight of each column is'
      WRITE(*,*)'tabulated as a differential pressure, where 1 Ggm/m/m'
      WRITE(*,*)'= 98 bars. Optimization of this model is constrained'
      WRITE(*,*)'by minimizing SUM, where when IWFLAG = 2'
      WRITE(*,*)'  SUM = DIFG(I)*WFACT*WW + DIFW(I), and'
      WRITE(*,*)'    I = 1 to the number of observed data points'
      WRITE(*,*)' DIFG = the absolute value of the differences between'
      WRITE(*,*)'        the calculated and observed gravity data,'
      WRITE(*,*)' DIFW = the absolute value of the differential column'
      WRITE(*,*)'        pressures,'   
      WRITE(*,*)'WFACT = WRMS/RMS, and'
      WRITE(*,*)' WRMS = the root mean square of DIFW'
      WRITE(*,*)'  RMS = the root mean square of DIFG'
      WRITE(*,*)'   WW = an arbitrary number used to mix DIFG and DIFW'
      WRITE(*,*)'        in varying amounts. A value of zero restricts'
      WRITE(*,*)'        optimization to the isostatic constraint.'
      WRITE(*,*)'When IWFLAG = 1, optimization minimizes DIFG only.'
      RETURN
42    WRITE(*,*)'REGC - REGional Correction'
      WRITE(*,*)'This option is implemented when the User chooses to'
      WRITE(*,*)'remove a linear trend from the observed profile data.'
      WRITE(*,*)'Normally before implementing this option, the User has'
      WRITE(*,*)'previously determined a value for the offset between'
      WRITE(*,*)'the observed and calculated profiles. Note that this'
      WRITE(*,*)'option moves the observed anomaly profile to the'
      WRITE(*,*)'calculated profile. A trend is subtracted from the'
      WRITE(*,*)'observed profile data using either a value input by'
      WRITE(*,*)'the User, or automatically by MAGRAV by removing the'
      WRITE(*,*)'linear trend obtained by fitting a least squares'
      WRITE(*,*)'straight line to the differences between the observed'
      WRITE(*,*)'and calculated profiles. The original observed profile'
      WRITE(*,*)'can be recovered using either the Q option from the'
      WRITE(*,*)'GRAPHICS window, or the RECO option from the TEXT'
      WRITE(*,*)'window. The alteration to the observed data profile'
      WRITE(*,*)'becomes permanent when the model is written to the'
      WRITE(*,*)'models file using the WRIT option, and the User exits'
      WRITE(*,*)'normally from MAGRAV.'
      RETURN
43    WRITE(*,*)'RMSR - Root Mean Square of Residuals'
      WRITE(*,*)'Calculates as required,'
      WRITE(*,*)' 1) the RMS of the differences between the calculated'
      WRITE(*,*)'    and observed gravity or magnetic profiles,'
      WRITE(*,*)' 2) the RMS of the differential column pressures.'
      WRITE(*,*)'This subroutine can also be used to assign an'
      WRITE(*,*)'arbitrary value to OFFSET(ITYPE), where OFFSET is the'
      WRITE(*,*)'base level shift between the calculated and observed'
      WRITE(*,*)'gravity(ITYPE=1) and magnetic(ITYPE=2) profiles.'
      RETURN
44    WRITE(*,*)'IWRT - I WRiTe'
      WRITE(*,*)'IWRITE is an integer which controls the amount of'
      WRITE(*,*)'information written to the screen text window by'
      WRITE(*,*)'subroutine DFS001. There will be output every IWRITE'
      WRITE(*,*)'iterations. The output consists of the iteration'
      WRITE(*,*)'number, number of CALFUN evaluations, the values of'
      WRITE(*,*)'variables, the value of the sum of the squares of the'
      WRITE(*,*)'residuals, and the individual function values. If'
      WRITE(*,*)'IWRITE is negative, there will be output after every'
      WRITE(*,*)'IWRITE iterations but without the function values. IF'
      WRITE(*,*)'IWRITE is zero(default), there will be no output from'
      WRITE(*,*)'subroutine DFS001'
      RETURN
      END
C***********************************************************************
      SUBROUTINE GHELP(ICHR)
      WRITE(*,*)
      GOTO(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,
     +22,23,24,25,26),ICHR
C
1     WRITE(*,*)'A - Automatically adjusts the magnetization or density'
      WRITE(*,*)'    contrast of either the body under crosshairs, or'
      WRITE(*,*)'    all bodies when crosshairs are external to a body.'
      RETURN
2     WRITE(*,*)'B - To enter a new body move the cursor around while'
      WRITE(*,*)'    hitting the new body number to define each body'
      WRITE(*,*)'    vertex until closing the body at the last vertex'
      WRITE(*,*)'    by hitting the period "." This option can only be'
      WRITE(*,*)'    used with body numbers 1-9.'
      WRITE(*,*)'    Points are entered in clockwise order.'
      RETURN 
3     WRITE(*,*)'C - To cancel an graphics option and return to the'
      WRITE(*,*)'    graphics menu'
      RETURN
4     WRITE(*,*)'D - To delete a body point position crosshairs over'
      WRITE(*,*)'    the selected point and hit LEFT Mouse button'
      RETURN
5     RETURN
6     WRITE(*,*)'F - Switch to fill(unfill) body with color'
      RETURN
7     WRITE(*,*)'G - Switch active mode to GRAVITY'
      RETURN
8     WRITE(*,*)'H - Switch to GRAPHICS help mode'
      RETURN
9     WRITE(*,*)'I - To insert a new body point successively cursor in'
      WRITE(*,*)'    an old body point on one side of the new point, a'
      WRITE(*,*)'    new body point, followed by an old body point on'
      WRITE(*,*)'    the other side of the new point'
      RETURN
10    WRITE(*,*)'J - To change the magnetization or density contrast of'
      WRITE(*,*)'    a selected body'
      RETURN
11    WRITE(*,*)'K - To switch body crosses on or off'
      RETURN
12    WRITE(*,*)'L - This option is implemented when the User chooses'
      WRITE(*,*)'    to remove a linear trend from the observed profile'
      WRITE(*,*)'    data. Normally before implementing this option,'
      WRITE(*,*)'    the User has previously determined a value for the'
      WRITE(*,*)'    offset between the observed and calculated'
      WRITE(*,*)'    profiles. Note that this option moves the observed'
      WRITE(*,*)'    anomaly profile to the calculated profile. A trend'
      WRITE(*,*)'    is subtracted from the observed profile data using'
      WRITE(*,*)'    either a value input by the User, or automatically'
      WRITE(*,*)'    by MAGRAV by removing the linear trend obtained by'
      WRITE(*,*)'    fitting a least squares straight line to the'
      WRITE(*,*)'    differences between the observed and calculated'
      WRITE(*,*)'    profiles. The original observed profile can be'
      WRITE(*,*)'    recovered using the Q option. The alteration to'
      WRITE(*,*)'    the observed data profile becomes permanent when'
      WRITE(*,*)'    the model is written to the models file using the'
      WRITE(*,*)'    WRIT option, and the User exits normally from'
      WRITE(*,*)'    MAGRAV.'
      RETURN
13    WRITE(*,*)'M - Switch active mode to MAGNETIC'
      RETURN
14    WRITE(*,*)'N - Incremental manual fit - a point is selected'
      WRITE(*,*)'    with the cursor, and is moved an arbitrary'
      WRITE(*,*)'    distance defined by the small square at the'
      WRITE(*,*)'    lower right corner of the graphics screen.'
      WRITE(*,*)'    The direction of movement is controlled with'
      WRITE(*,*)'    the NUMBER keys as follows:'
      WRITE(*,*)'        1 - down to left'
      WRITE(*,*)'        2 - straight down'
      WRITE(*,*)'        3 - down to right'
      WRITE(*,*)'        4 - laterally to left'
      WRITE(*,*)'        5 - return to graphics menu'
      WRITE(*,*)'        6 - laterally to right'
      WRITE(*,*)'        7 - upward to left'
      WRITE(*,*)'        8 - straight up'
      WRITE(*,*)'        9 - upward to right'
      WRITE(*,*)'    Bell rings if the error-of-fit increases.'
      RETURN
15    WRITE(*,*)'O - Determines the value of OFFSET(ITYPE) in'
      WRITE(*,*)'    mGals(nT) in order to move a calculated'
      WRITE(*,*)'    gravity(ITYPE=1) or magnetic(ITYPE=2)'
      WRITE(*,*)'    profile to the observed data profile. '
      WRITE(*,*)'    The five choices are as follows:'
      WRITE(*,*)'       Y - the current OFFSET(ITYPE) value is varied'
      WRITE(*,*)'           by automatic curve fitting subroutines'
      WRITE(*,*)'       N - the current OFFSET(ITYPE) value is not'
      WRITE(*,*)'           varied by automatic curve fitting'
      WRITE(*,*)'           subroutines.'
      WRITE(*,*)'       O - the offset is determined using the mean'
      WRITE(*,*)'           value of the differences between the'
      WRITE(*,*)'           calculated and observed profiles'
      WRITE(*,*)'           divided by the number of data points'
      WRITE(*,*)'       P - an arbitrary value is added to OFFSET()'
      WRITE(*,*)'       M - an arbitrary value is subtracted from'
      WRITE(*,*)'           OFFSET(ITYPE).'
      WRITE(*,*)'    The arbitrary value is DUM*1/100th of the'
      WRITE(*,*)'    difference between the maximum and minimum'
      WRITE(*,*)'    values in mGAL (nT) along the gravity (magnetic)' 
      WRITE(*,*)'    axis. After hitting the P or M keys, DUM is'
      WRITE(*,*)'    specified by hitting any of the 1 to 9 keys.'
      RETURN
16    WRITE(*,*)'P - After hitting P several choices are available'
      WRITE(*,*)'      - To move a body point manually,'
      WRITE(*,*)'        Move crosshairs to select a body point'
      WRITE(*,*)'        Hit LEFT Mouse button'
      WRITE(*,*)'        Move crosshairs to new location'
      WRITE(*,*)'        Hit LEFT Mouse button again'
      WRITE(*,*)'        Computer draws new body and returns to menu'
      WRITE(*,*)'      - To move a body point automatically'
      WRITE(*,*)'        Move crosshairs to select a point'
      WRITE(*,*)'        Hit an M, X or Z key'
      WRITE(*,*)'        Computer redraws body'
      WRITE(*,*)'      - To move two body points at one time'
      WRITE(*,*)'        Move crosshairs to select a  body point'
      WRITE(*,*)'        Hit LEFT Mouse key'
      WRITE(*,*)'        Move crosshairs to second point'
      WRITE(*,*)'        Hit an M, X or Z key'
      WRITE(*,*)'        Computer redraws body'
      WRITE(*,*)'     Note  M - permits movement in X and Z directions'
      WRITE(*,*)'           X - restricts movement to X direction'
      WRITE(*,*)'           Z - restricts movement to Z direction'
      WRITE(*,*)'     Up to three bodies can be linked at single point'
      RETURN
17    WRITE(*,*)'Q - Recover command. NSAVE counts the number of'
      WRITE(*,*)'    models saved. If recover command is executed'
      WRITE(*,*)'    repeatedly, each execution will recover an'
      WRITE(*,*)'    earlier model.'
      RETURN
18    WRITE(*,*)'R - Redraws the graphics using current'
      WRITE(*,*)'        body parameters.'
      RETURN
19    WRITE(*,*)'S - Switches active mode to SKETCH and redraws the'
      WRITE(*,*)'    the graphics window based on parameters defined'
      WRITE(*,*)'    by the Z or ZOOM options.'
      RETURN
20    WRITE(*,*)'T - Switches active mode to the standard graphics'
      WRITE(*,*)'    window defined by the SCAL option.' 
      RETURN
21    WRITE(*,*)'U - Switches between plots of the calculated(solid'
      WRITE(*,*)'    red line) and the observed (dashed green line)'
      WRITE(*,*)'    profiles to a plot of the differences(solid red'
      WRITE(*,*)'    line) between these two profiles. The zero level'
      WRITE(*,*)'    may be shown as a dashed white line. The'
      WRITE(*,*)'    plotting limits are set from the TEXT window'
      WRITE(*,*)'    with the SCAL option using letter F.'
      RETURN
22    WRITE(*,*)'V - Permits automatic curve fitting routines to'
      WRITE(*,*)'    modify body magnetization or density contrast(s)'
      WRITE(*,*)'    at the same time as the body shape(s) are being'
      WRITE(*,*)'    being changed. After hitting the V key'
      WRITE(*,*)'      HIT Y - to permit body contrast changes'
      WRITE(*,*)'      HIT N - to restrict body contrast changes'
      RETURN
23    WRITE(*,*)'W - Calculates and plots the column differential'
      WRITE(*,*)'    pressures. See ISOS in text help.'
      RETURN
24    WRITE(*,*)'X - Body number of body under crosshairs is'
      WRITE(*,*)'    output in the TEXT window.'
      RETURN
25    WRITE(*,*)'Y - Calculates anomaly using the current body'
      WRITE(*,*)'    parameters. After hitting Y either,'
      WRITE(*,*)'       1) Hit 0 twice to include all bodies,'
      WRITE(*,*)'       2) Hit 0 and 8 for example to calculate'
      WRITE(*,*)'          the anomaly for body 8.'
      WRITE(*,*)'    The resultant calculated anomaly is plotted.'
      RETURN
26    WRITE(*,*)'Z - To set the ZOOM window (SKETCH mode) position'
      WRITE(*,*)'    the cursor at the lower left corner of the'
      WRITE(*,*)'    selected window and click the LEFT button.'
      WRITE(*,*)'    Move the cursor to the upper right corner of'
      WRITE(*,*)'    the selected window and again click the LEFT'
      WRITE(*,*)'    button. Choose S to display the new window.'
      WRITE(*,*)'    Choose T to return to the normal window.'
      RETURN
      END
